@@ -6,10 +6,7 @@
 #include "binder/ddl/bound_drop.h"
 #include "binder/expression_visitor.h"
 #include "catalog/catalog.h"
-#include "catalog/catalog_entry/index_catalog_entry.h"
 #include "catalog/catalog_entry/node_table_catalog_entry.h"
-#include "catalog/catalog_entry/rel_group_catalog_entry.h"
-#include "catalog/catalog_entry/rel_table_catalog_entry.h"
 #include "catalog/catalog_entry/sequence_catalog_entry.h"
 #include "common/enums/extend_direction_util.h"
 #include "common/exception/binder.h"
@@ -18,6 +15,7 @@
 #include "common/system_config.h"
 #include "common/types/types.h"
 #include "function/cast/functions/cast_from_string_functions.h"
+#include "function/sequence/sequence_functions.h"
 #include "main/client_context.h"
 #include "parser/ddl/alter.h"
 #include "parser/ddl/create_sequence.h"
@@ -25,6 +23,7 @@
 #include "parser/ddl/create_table_info.h"
 #include "parser/ddl/create_type.h"
 #include "parser/ddl/drop.h"
+#include "parser/expression/parsed_function_expression.h"
 #include "parser/expression/parsed_literal_expression.h"
 
 using namespace kuzu::common;
@@ -56,6 +55,10 @@ std::vector<PropertyDefinition> Binder::bindPropertyDefinitions(
         auto type = LogicalType::convertFromString(def.getType(), clientContext);
         auto defaultExpr =
             resolvePropertyDefault(def.defaultExpr.get(), type, tableName, def.getName());
+        auto boundExpr = expressionBinder.bindExpression(*defaultExpr);
+        if (boundExpr->dataType != type) {
+            expressionBinder.implicitCast(boundExpr, type);
+        }
         auto columnDefinition = ColumnDefinition(def.getName(), std::move(type));
         definitions.emplace_back(std::move(columnDefinition), std::move(defaultExpr));
     }
@@ -63,13 +66,14 @@ std::vector<PropertyDefinition> Binder::bindPropertyDefinitions(
     return definitions;
 }
 
-std::unique_ptr<parser::ParsedExpression> Binder::resolvePropertyDefault(
-    ParsedExpression* parsedDefault, const common::LogicalType& type, const std::string& tableName,
-    const std::string& propertyName) {
+std::unique_ptr<ParsedExpression> Binder::resolvePropertyDefault(ParsedExpression* parsedDefault,
+    const LogicalType& type, const std::string& tableName, const std::string& propertyName) {
     if (parsedDefault == nullptr) { // No default provided.
         if (type.getLogicalTypeID() == LogicalTypeID::SERIAL) {
             auto serialName = SequenceCatalogEntry::getSerialName(tableName, propertyName);
-            return ParsedExpressionUtils::getSerialDefaultExpr(serialName);
+            auto literalExpr = std::make_unique<ParsedLiteralExpression>(Value(serialName), "");
+            return std::make_unique<ParsedFunctionExpression>(function::NextValFunction::name,
+                std::move(literalExpr), "" /* rawName */);
         } else {
             return std::make_unique<ParsedLiteralExpression>(Value::createNullValue(type), "NULL");
         }
@@ -116,41 +120,12 @@ static void validatePrimaryKey(const std::string& pkColName,
     }
 }
 
-void Binder::validateNoIndexOnProperty(const std::string& tableName,
-    const std::string& propertyName) const {
-    auto transaction = clientContext->getTransaction();
-    auto catalog = clientContext->getCatalog();
-    if (catalog->containsRelGroup(transaction, tableName)) {
-        // RelGroup does not have indexes.
-        return;
-    }
-    auto tableEntry = catalog->getTableCatalogEntry(transaction, tableName);
-    if (!tableEntry->containsProperty(propertyName)) {
-        return;
-    }
-    auto propertyID = tableEntry->getPropertyID(propertyName);
-    for (auto indexCatalogEntry : catalog->getIndexEntries(transaction)) {
-        auto propertiesWithIndex = indexCatalogEntry->getPropertyIDs();
-        if (indexCatalogEntry->getTableID() == tableEntry->getTableID() &&
-            std::find(propertiesWithIndex.begin(), propertiesWithIndex.end(), propertyID) !=
-                propertiesWithIndex.end()) {
-            throw BinderException{stringFormat(
-                "Cannot drop property {} in table {} because it is used in one or more indexes. "
-                "Please remove the associated indexes before attempting to drop this property.",
-                propertyName, tableName)};
-        }
-    }
-}
-
 BoundCreateTableInfo Binder::bindCreateTableInfo(const CreateTableInfo* info) {
     switch (info->type) {
-    case CatalogEntryType::NODE_TABLE_ENTRY: {
+    case TableType::NODE: {
         return bindCreateNodeTableInfo(info);
     }
-    case CatalogEntryType::REL_TABLE_ENTRY: {
-        return bindCreateRelTableInfo(info);
-    }
-    case CatalogEntryType::REL_GROUP_ENTRY: {
+    case TableType::REL: {
         return bindCreateRelTableGroupInfo(info);
     }
     default: {
@@ -169,12 +144,27 @@ BoundCreateTableInfo Binder::bindCreateNodeTableInfo(const CreateTableInfo* info
         info->onConflict, std::move(boundExtraInfo), clientContext->useInternalCatalogEntry());
 }
 
-static void validateNodeTableType(const TableCatalogEntry* entry) {
+void Binder::validateNodeTableType(const TableCatalogEntry* entry) {
     if (entry->getType() != CatalogEntryType::NODE_TABLE_ENTRY) {
         throw BinderException(stringFormat("{} is not of type NODE.", entry->getName()));
     }
 }
 
+void Binder::validateTableExistence(const main::ClientContext& context,
+    const std::string& tableName) {
+    if (!context.getCatalog()->containsTable(context.getTransaction(), tableName)) {
+        throw BinderException{stringFormat("Table {} does not exist.", tableName)};
+    }
+}
+
+void Binder::validateColumnExistence(const TableCatalogEntry* entry,
+    const std::string& columnName) {
+    if (!entry->containsProperty(columnName)) {
+        throw BinderException{
+            stringFormat("Column {} does not exist in table {}.", columnName, entry->getName())};
+    }
+}
+
 static ExtendDirection getStorageDirection(const case_insensitive_map_t<Value>& options) {
     if (options.contains(TableOptionConstants::REL_STORAGE_DIRECTION_OPTION)) {
         return ExtendDirectionUtil::fromString(
@@ -183,75 +173,120 @@ static ExtendDirection getStorageDirection(const case_insensitive_map_t<Value>&
     return DEFAULT_EXTEND_DIRECTION;
 }
 
-BoundCreateTableInfo Binder::bindCreateRelTableInfo(const CreateTableInfo* info) {
-    auto& extraInfo = info->extraInfo->constCast<ExtraCreateRelTableInfo>();
-    return bindCreateRelTableInfo(info, extraInfo.options);
-}
-
-BoundCreateTableInfo Binder::bindCreateRelTableInfo(const CreateTableInfo* info,
-    const options_t& parsedOptions) {
+std::vector<PropertyDefinition> Binder::bindRelPropertyDefinitions(const CreateTableInfo& info) {
     std::vector<PropertyDefinition> propertyDefinitions;
     propertyDefinitions.emplace_back(
         ColumnDefinition(InternalKeyword::ID, LogicalType::INTERNAL_ID()));
-    for (auto& definition : bindPropertyDefinitions(info->propertyDefinitions, info->tableName)) {
+    for (auto& definition : bindPropertyDefinitions(info.propertyDefinitions, info.tableName)) {
         propertyDefinitions.push_back(definition.copy());
     }
-    auto& extraInfo = info->extraInfo->constCast<ExtraCreateRelTableInfo>();
-    auto srcMultiplicity = RelMultiplicityUtils::getFwd(extraInfo.relMultiplicity);
-    auto dstMultiplicity = RelMultiplicityUtils::getBwd(extraInfo.relMultiplicity);
-    auto srcEntry = bindNodeTableEntry(extraInfo.srcTableName);
-    validateNodeTableType(srcEntry);
-    auto dstEntry = bindNodeTableEntry(extraInfo.dstTableName);
-    validateNodeTableType(dstEntry);
-    auto boundOptions = bindParsingOptions(parsedOptions);
-    auto storageDirection = getStorageDirection(boundOptions);
-    auto boundExtraInfo = std::make_unique<BoundExtraCreateRelTableInfo>(srcMultiplicity,
-        dstMultiplicity, storageDirection, srcEntry->getTableID(), dstEntry->getTableID(),
-        std::move(propertyDefinitions));
-    return BoundCreateTableInfo(CatalogEntryType::REL_TABLE_ENTRY, info->tableName,
-        info->onConflict, std::move(boundExtraInfo), clientContext->useInternalCatalogEntry());
-}
-
-static void validateUniqueFromToPairs(
-    const std::vector<std::pair<std::string, std::string>>& pairs) {
-    std::unordered_set<std::string> set;
-    for (auto [from, to] : pairs) {
-        auto key = from + to;
-        if (set.contains(key)) {
-            throw BinderException(stringFormat("Found duplicate FROM-TO {}-{} pairs.", from, to));
-        }
-        set.insert(key);
-    }
+    return propertyDefinitions;
 }
 
 BoundCreateTableInfo Binder::bindCreateRelTableGroupInfo(const CreateTableInfo* info) {
-    auto relGroupName = info->tableName;
+    auto propertyDefinitions = bindRelPropertyDefinitions(*info);
     auto& extraInfo = info->extraInfo->constCast<ExtraCreateRelTableGroupInfo>();
-    auto relMultiplicity = extraInfo.relMultiplicity;
-    std::vector<BoundCreateTableInfo> boundCreateRelTableInfos;
-    auto relCreateInfo =
-        std::make_unique<CreateTableInfo>(CatalogEntryType::REL_TABLE_ENTRY, "", info->onConflict);
-    relCreateInfo->propertyDefinitions = copyVector(info->propertyDefinitions);
-    validateUniqueFromToPairs(extraInfo.srcDstTablePairs);
+    auto srcMultiplicity = RelMultiplicityUtils::getFwd(extraInfo.relMultiplicity);
+    auto dstMultiplicity = RelMultiplicityUtils::getBwd(extraInfo.relMultiplicity);
+    auto boundOptions = bindParsingOptions(extraInfo.options);
+    auto storageDirection = getStorageDirection(boundOptions);
+    // Bind from to pairs
+    node_table_id_pair_set_t nodePairsSet;
+    std::vector<NodeTableIDPair> nodePairs;
     for (auto& [srcTableName, dstTableName] : extraInfo.srcDstTablePairs) {
-        relCreateInfo->tableName =
-            RelGroupCatalogEntry::getChildTableName(relGroupName, srcTableName, dstTableName);
-        relCreateInfo->extraInfo = std::make_unique<ExtraCreateRelTableInfo>(relMultiplicity,
-            srcTableName, dstTableName, options_t{});
-        auto boundInfo = bindCreateRelTableInfo(relCreateInfo.get(), extraInfo.options);
-        boundInfo.hasParent = true;
-        boundCreateRelTableInfos.push_back(std::move(boundInfo));
+        auto srcEntry = bindNodeTableEntry(srcTableName);
+        validateNodeTableType(srcEntry);
+        auto dstEntry = bindNodeTableEntry(dstTableName);
+        validateNodeTableType(dstEntry);
+        NodeTableIDPair pair{srcEntry->getTableID(), dstEntry->getTableID()};
+        if (nodePairsSet.contains(pair)) {
+            throw BinderException(
+                stringFormat("Found duplicate FROM-TO {}-{} pairs.", srcTableName, dstTableName));
+        }
+        nodePairsSet.insert(pair);
+        nodePairs.emplace_back(pair);
     }
     auto boundExtraInfo =
-        std::make_unique<BoundExtraCreateRelTableGroupInfo>(std::move(boundCreateRelTableInfos));
+        std::make_unique<BoundExtraCreateRelTableGroupInfo>(std::move(propertyDefinitions),
+            srcMultiplicity, dstMultiplicity, storageDirection, std::move(nodePairs));
     return BoundCreateTableInfo(CatalogEntryType::REL_GROUP_ENTRY, info->tableName,
         info->onConflict, std::move(boundExtraInfo), clientContext->useInternalCatalogEntry());
 }
 
 std::unique_ptr<BoundStatement> Binder::bindCreateTable(const Statement& statement) {
-    auto createTable = statement.constPtrCast<CreateTable>();
-    auto boundCreateInfo = bindCreateTableInfo(createTable->getInfo());
-    return std::make_unique<BoundCreateTable>(std::move(boundCreateInfo));
+    auto& createTable = statement.constCast<CreateTable>();
+    if (createTable.getSource()) {
+        return bindCreateTableAs(createTable);
+    }
+    auto boundCreateInfo = bindCreateTableInfo(createTable.getInfo());
+    return std::make_unique<BoundCreateTable>(std::move(boundCreateInfo),
+        BoundStatementResult::createSingleStringColumnResult());
+}
+
+std::unique_ptr<BoundStatement> Binder::bindCreateTableAs(const Statement& statement) {
+    auto& createTable = statement.constCast<CreateTable>();
+    auto boundInnerQuery = bindQuery(*createTable.getSource()->statement.get());
+    auto innerQueryResult = boundInnerQuery->getStatementResult();
+    auto columnNames = innerQueryResult->getColumnNames();
+    auto columnTypes = innerQueryResult->getColumnTypes();
+    std::vector<PropertyDefinition> propertyDefinitions;
+    propertyDefinitions.reserve(columnNames.size());
+    for (size_t i = 0; i < columnNames.size(); ++i) {
+        propertyDefinitions.emplace_back(
+            ColumnDefinition(std::string(columnNames[i]), columnTypes[i].copy()));
+    }
+    if (columnNames.empty()) {
+        throw BinderException("Subquery returns no columns");
+    }
+    auto createInfo = createTable.getInfo();
+    switch (createInfo->type) {
+    case TableType::NODE: {
+        // first column is primary key column temporarily for now
+        auto pkName = columnNames[0];
+        validatePrimaryKey(pkName, propertyDefinitions);
+        auto boundCopyFromInfo = bindCopyNodeFromInfo(createInfo->tableName, propertyDefinitions,
+            createTable.getSource(), options_t{}, columnNames, columnTypes, false /* byColumn */);
+        auto boundExtraInfo =
+            std::make_unique<BoundExtraCreateNodeTableInfo>(pkName, std::move(propertyDefinitions));
+        auto boundCreateInfo = BoundCreateTableInfo(CatalogEntryType::NODE_TABLE_ENTRY,
+            createInfo->tableName, createInfo->onConflict, std::move(boundExtraInfo),
+            clientContext->useInternalCatalogEntry());
+        auto boundCreateTable = std::make_unique<BoundCreateTable>(std::move(boundCreateInfo),
+            BoundStatementResult::createSingleStringColumnResult());
+        boundCreateTable->setCopyInfo(std::move(boundCopyFromInfo));
+        return boundCreateTable;
+    }
+    case TableType::REL: {
+        auto& extraInfo = createInfo->extraInfo->constCast<ExtraCreateRelTableGroupInfo>();
+        // Currently we don't support multiple from/to pairs for create rel table as
+        if (extraInfo.srcDstTablePairs.size() > 1) {
+            throw BinderException(
+                "Multiple FROM/TO pairs are not supported for CREATE REL TABLE AS.");
+        }
+        propertyDefinitions.insert(propertyDefinitions.begin(),
+            PropertyDefinition(ColumnDefinition(InternalKeyword::ID, LogicalType::INTERNAL_ID())));
+        auto catalog = clientContext->getCatalog();
+        auto transaction = clientContext->getTransaction();
+        auto fromTable =
+            catalog->getTableCatalogEntry(transaction, extraInfo.srcDstTablePairs[0].first)
+                ->ptrCast<NodeTableCatalogEntry>();
+        auto toTable =
+            catalog->getTableCatalogEntry(transaction, extraInfo.srcDstTablePairs[0].second)
+                ->ptrCast<NodeTableCatalogEntry>();
+        auto boundCreateInfo = bindCreateRelTableGroupInfo(createInfo);
+        auto boundCopyFromInfo = bindCopyRelFromInfo(createInfo->tableName, propertyDefinitions,
+            createTable.getSource(), options_t{}, columnNames, columnTypes, fromTable, toTable);
+        boundCreateInfo.extraInfo->ptrCast<BoundExtraCreateTableInfo>()->propertyDefinitions =
+            std::move(propertyDefinitions);
+        auto boundCreateTable = std::make_unique<BoundCreateTable>(std::move(boundCreateInfo),
+            BoundStatementResult::createSingleStringColumnResult());
+        boundCreateTable->setCopyInfo(std::move(boundCopyFromInfo));
+        return boundCreateTable;
+    }
+    default: {
+        KU_UNREACHABLE;
+    }
+    }
 }
 
 std::unique_ptr<BoundStatement> Binder::bindCreateType(const Statement& statement) const {
@@ -350,6 +385,10 @@ std::unique_ptr<BoundStatement> Binder::bindAlter(const Statement& statement) {
     case AlterType::COMMENT: {
         return bindCommentOn(statement);
     }
+    case AlterType::ADD_FROM_TO_CONNECTION:
+    case AlterType::DROP_FROM_TO_CONNECTION: {
+        return bindAlterFromToConnection(statement);
+    }
     default: {
         KU_UNREACHABLE;
     }
@@ -398,7 +437,6 @@ std::unique_ptr<BoundStatement> Binder::bindDropProperty(const Statement& statem
     auto extraInfo = info->extraInfo->constPtrCast<ExtraDropPropertyInfo>();
     auto tableName = info->tableName;
     auto propertyName = extraInfo->propertyName;
-    validateNoIndexOnProperty(tableName, propertyName);
     auto boundExtraInfo = std::make_unique<BoundExtraDropPropertyInfo>(propertyName);
     auto boundInfo = BoundAlterInfo(AlterType::DROP_PROPERTY, tableName, std::move(boundExtraInfo),
         info->onConflict);
@@ -430,5 +468,21 @@ std::unique_ptr<BoundStatement> Binder::bindCommentOn(const Statement& statement
     return std::make_unique<BoundAlter>(std::move(boundInfo));
 }
 
+std::unique_ptr<BoundStatement> Binder::bindAlterFromToConnection(
+    const Statement& statement) const {
+    auto& alter = statement.constCast<Alter>();
+    auto info = alter.getInfo();
+    auto extraInfo = info->extraInfo->constPtrCast<ExtraAddFromToConnection>();
+    auto tableName = info->tableName;
+    auto srcTableEntry = bindNodeTableEntry(extraInfo->srcTableName);
+    auto dstTableEntry = bindNodeTableEntry(extraInfo->dstTableName);
+    auto srcTableID = srcTableEntry->getTableID();
+    auto dstTableID = dstTableEntry->getTableID();
+    auto boundExtraInfo = std::make_unique<BoundExtraAlterFromToConnection>(srcTableID, dstTableID);
+    auto boundInfo =
+        BoundAlterInfo(info->type, tableName, std::move(boundExtraInfo), info->onConflict);
+    return std::make_unique<BoundAlter>(std::move(boundInfo));
+}
+
 } // namespace binder
 } // namespace kuzu
