@@ -1,17 +1,17 @@
 #include "binder/binder.h"
 #include "binder/expression/expression_util.h"
-#include "binder/expression/literal_expression.h"
 #include "binder/expression/path_expression.h"
 #include "binder/expression/property_expression.h"
 #include "binder/expression_visitor.h"
 #include "catalog/catalog.h"
 #include "catalog/catalog_entry/node_table_catalog_entry.h"
 #include "catalog/catalog_entry/rel_group_catalog_entry.h"
-#include "catalog/catalog_entry/rel_table_catalog_entry.h"
+#include "common/enums/rel_direction.h"
 #include "common/exception/binder.h"
 #include "common/string_format.h"
 #include "common/utils.h"
 #include "function/cast/functions/cast_from_string_functions.h"
+#include "function/gds/rec_joins.h"
 #include "function/rewrite_function.h"
 #include "function/schema/vector_node_rel_functions.h"
 #include "main/client_context.h"
@@ -71,18 +71,15 @@ static LogicalType getRecursiveRelLogicalType(const LogicalType& nodeType,
     std::vector<StructField> recursiveRelFields;
     recursiveRelFields.emplace_back(InternalKeyword::NODES, std::move(nodesType));
     recursiveRelFields.emplace_back(InternalKeyword::RELS, std::move(relsType));
-    return LogicalType::RECURSIVE_REL(
-        std::make_unique<StructTypeInfo>(std::move(recursiveRelFields)));
+    return LogicalType::RECURSIVE_REL(std::move(recursiveRelFields));
 }
 
 static void extraFieldFromStructType(const LogicalType& structType,
-    std::unordered_set<std::string>& nameSet, std::vector<std::string>& names,
-    std::vector<LogicalType>& types) {
+    std::unordered_set<std::string>& set, std::vector<StructField>& structFields) {
     for (auto& field : StructType::getFields(structType)) {
-        if (!nameSet.contains(field.getName())) {
-            nameSet.insert(field.getName());
-            names.push_back(field.getName());
-            types.push_back(field.getType().copy());
+        if (!set.contains(field.getName())) {
+            set.insert(field.getName());
+            structFields.emplace_back(field.getName(), field.getType().copy());
         }
     }
 }
@@ -90,35 +87,28 @@ static void extraFieldFromStructType(const LogicalType& structType,
 std::shared_ptr<Expression> Binder::createPath(const std::string& pathName,
     const expression_vector& children) {
     std::unordered_set<std::string> nodeFieldNameSet;
-    std::vector<std::string> nodeFieldNames;
-    std::vector<LogicalType> nodeFieldTypes;
+    std::vector<StructField> nodeFields;
     std::unordered_set<std::string> relFieldNameSet;
-    std::vector<std::string> relFieldNames;
-    std::vector<LogicalType> relFieldTypes;
+    std::vector<StructField> relFields;
     for (auto& child : children) {
         if (ExpressionUtil::isNodePattern(*child)) {
-            auto node = ku_dynamic_cast<NodeExpression*>(child.get());
-            extraFieldFromStructType(node->getDataType(), nodeFieldNameSet, nodeFieldNames,
-                nodeFieldTypes);
+            auto& node = child->constCast<NodeExpression>();
+            extraFieldFromStructType(node.getDataType(), nodeFieldNameSet, nodeFields);
         } else if (ExpressionUtil::isRelPattern(*child)) {
             auto rel = ku_dynamic_cast<RelExpression*>(child.get());
-            extraFieldFromStructType(rel->getDataType(), relFieldNameSet, relFieldNames,
-                relFieldTypes);
+            extraFieldFromStructType(rel->getDataType(), relFieldNameSet, relFields);
         } else if (ExpressionUtil::isRecursiveRelPattern(*child)) {
             auto recursiveRel = ku_dynamic_cast<RelExpression*>(child.get());
             auto recursiveInfo = recursiveRel->getRecursiveInfo();
             extraFieldFromStructType(recursiveInfo->node->getDataType(), nodeFieldNameSet,
-                nodeFieldNames, nodeFieldTypes);
-            extraFieldFromStructType(recursiveInfo->rel->getDataType(), relFieldNameSet,
-                relFieldNames, relFieldTypes);
+                nodeFields);
+            extraFieldFromStructType(recursiveInfo->rel->getDataType(), relFieldNameSet, relFields);
         } else {
             KU_UNREACHABLE;
         }
     }
-    auto nodeExtraInfo = std::make_unique<StructTypeInfo>(nodeFieldNames, nodeFieldTypes);
-    auto nodeType = LogicalType::NODE(std::move(nodeExtraInfo));
-    auto relExtraInfo = std::make_unique<StructTypeInfo>(relFieldNames, relFieldTypes);
-    auto relType = LogicalType::REL(std::move(relExtraInfo));
+    auto nodeType = LogicalType::NODE(std::move(nodeFields));
+    auto relType = LogicalType::REL(std::move(relFields));
     auto uniqueName = getUniqueExpressionName(pathName);
     return std::make_shared<PathExpression>(getRecursiveRelLogicalType(nodeType, relType),
         uniqueName, pathName, std::move(nodeType), std::move(relType), children);
@@ -139,7 +129,7 @@ static std::vector<std::string> getPropertyNames(const std::vector<TableCatalogE
     return result;
 }
 
-static std::unique_ptr<Expression> createPropertyExpression(const std::string& propertyName,
+static std::shared_ptr<PropertyExpression> createPropertyExpression(const std::string& propertyName,
     const std::string& uniqueVariableName, const std::string& rawVariableName,
     const std::vector<TableCatalogEntry*>& entries) {
     table_id_map_t<SingleLabelPropertyInfo> infos;
@@ -159,25 +149,11 @@ static std::unique_ptr<Expression> createPropertyExpression(const std::string& p
         auto info = SingleLabelPropertyInfo(exists, isPrimaryKey);
         infos.insert({entry->getTableID(), std::move(info)});
     }
-    // Validate property under the same name has the same type.
-    KU_ASSERT(!dataTypes.empty());
-    for (const auto& type : dataTypes) {
-        if (dataTypes[0] != type) {
-            throw BinderException(
-                stringFormat("Expected the same data type for property {} but found {} and {}.",
-                    propertyName, type.toString(), dataTypes[0].toString()));
-        }
-    }
-    return make_unique<PropertyExpression>(std::move(dataTypes[0]), propertyName,
+    LogicalType maxType = LogicalTypeUtils::combineTypes(dataTypes);
+    return std::make_shared<PropertyExpression>(std::move(maxType), propertyName,
         uniqueVariableName, rawVariableName, std::move(infos));
 }
 
-static std::unique_ptr<Expression> createPropertyExpression(const std::string& propertyName,
-    const Expression& pattern, const std::vector<TableCatalogEntry*>& entries) {
-    return createPropertyExpression(propertyName, pattern.getUniqueName(), pattern.toString(),
-        entries);
-}
-
 static void checkRelDirectionTypeAgainstStorageDirection(const RelExpression* rel) {
     switch (rel->getDirectionType()) {
     case RelDirectionType::SINGLE:
@@ -215,7 +191,7 @@ std::shared_ptr<RelExpression> Binder::bindQueryRel(const RelPattern& relPattern
         throw BinderException("Bind relationship " + parsedName +
                               " to relationship with same name is not supported.");
     }
-    auto entries = bindRelTableEntries(relPattern.getTableNames());
+    auto entries = bindRelGroupEntries(relPattern.getTableNames());
     // bind src & dst node
     RelDirectionType directionType = RelDirectionType::UNKNOWN;
     std::shared_ptr<NodeExpression> srcNode;
@@ -267,33 +243,72 @@ std::shared_ptr<RelExpression> Binder::bindQueryRel(const RelPattern& relPattern
     return queryRel;
 }
 
+static std::vector<StructField> getBaseNodeStructFields() {
+    std::vector<StructField> fields;
+    fields.emplace_back(InternalKeyword::ID, LogicalType::INTERNAL_ID());
+    fields.emplace_back(InternalKeyword::LABEL, LogicalType::STRING());
+    return fields;
+}
+
+static std::vector<StructField> getBaseRelStructFields() {
+    std::vector<StructField> fields;
+    fields.emplace_back(InternalKeyword::SRC, LogicalType::INTERNAL_ID());
+    fields.emplace_back(InternalKeyword::DST, LogicalType::INTERNAL_ID());
+    fields.emplace_back(InternalKeyword::LABEL, LogicalType::STRING());
+    return fields;
+}
+
+static std::shared_ptr<PropertyExpression> construct(LogicalType type,
+    const std::string& propertyName, const Expression& child) {
+    KU_ASSERT(child.expressionType == ExpressionType::PATTERN);
+    auto& patternExpr = child.constCast<NodeOrRelExpression>();
+    auto variableName = patternExpr.getVariableName();
+    auto uniqueName = patternExpr.getUniqueName();
+    // Assign an invalid property id for virtual property.
+    table_id_map_t<SingleLabelPropertyInfo> infos;
+    for (auto& entry : patternExpr.getEntries()) {
+        infos.insert({entry->getTableID(),
+            SingleLabelPropertyInfo(false /* exists */, false /* isPrimaryKey */)});
+    }
+    return std::make_unique<PropertyExpression>(std::move(type), propertyName, uniqueName,
+        variableName, std::move(infos));
+}
+
 std::shared_ptr<RelExpression> Binder::createNonRecursiveQueryRel(const std::string& parsedName,
     const std::vector<TableCatalogEntry*>& entries, std::shared_ptr<NodeExpression> srcNode,
     std::shared_ptr<NodeExpression> dstNode, RelDirectionType directionType) {
-    auto queryRel = make_shared<RelExpression>(LogicalType(LogicalTypeID::REL),
-        getUniqueExpressionName(parsedName), parsedName, entries, std::move(srcNode),
-        std::move(dstNode), directionType, QueryRelType::NON_RECURSIVE);
+    auto uniqueName = getUniqueExpressionName(parsedName);
+    // Bind properties
+    auto structFields = getBaseRelStructFields();
+    std::vector<std::shared_ptr<PropertyExpression>> propertyExpressions;
+    if (entries.empty()) {
+        structFields.emplace_back(InternalKeyword::ID, LogicalType::INTERNAL_ID());
+    } else {
+        for (auto& propertyName : getPropertyNames(entries)) {
+            auto property = createPropertyExpression(propertyName, uniqueName, parsedName, entries);
+            structFields.emplace_back(property->getPropertyName(), property->getDataType().copy());
+            propertyExpressions.push_back(std::move(property));
+        }
+    }
+    auto queryRel = std::make_shared<RelExpression>(LogicalType::REL(std::move(structFields)),
+        uniqueName, parsedName, entries, std::move(srcNode), std::move(dstNode), directionType,
+        QueryRelType::NON_RECURSIVE);
+    queryRel->setAlias(parsedName);
+    if (entries.empty()) {
+        queryRel->addPropertyExpression(
+            construct(LogicalType::INTERNAL_ID(), InternalKeyword::ID, *queryRel));
+    } else {
+        for (auto& property : propertyExpressions) {
+            queryRel->addPropertyExpression(property);
+        }
+    }
+    // Bind internal expressions.
     if (directionType == RelDirectionType::BOTH) {
         queryRel->setDirectionExpr(expressionBinder.createVariableExpression(LogicalType::BOOL(),
             queryRel->getUniqueName() + InternalKeyword::DIRECTION));
     }
-    queryRel->setAlias(parsedName);
-    bindQueryRelProperties(*queryRel);
-    std::vector<StructField> fields;
-    fields.emplace_back(InternalKeyword::SRC, LogicalType::INTERNAL_ID());
-    fields.emplace_back(InternalKeyword::DST, LogicalType::INTERNAL_ID());
-    // Bind internal expressions.
     auto input = function::RewriteFunctionBindInput(clientContext, &expressionBinder, {queryRel});
     queryRel->setLabelExpression(function::LabelFunction::rewriteFunc(input));
-    fields.emplace_back(InternalKeyword::LABEL,
-        queryRel->getLabelExpression()->getDataType().copy());
-    // Bind properties.
-    for (auto& expression : queryRel->getPropertyExprsRef()) {
-        auto& property = expression->constCast<PropertyExpression>();
-        fields.emplace_back(property.getPropertyName(), property.getDataType().copy());
-    }
-    auto extraInfo = std::make_unique<StructTypeInfo>(std::move(fields));
-    queryRel->setExtraTypeInfo(std::move(extraInfo));
     return queryRel;
 }
 
@@ -334,44 +349,39 @@ std::shared_ptr<RelExpression> Binder::createRecursiveQueryRel(const parser::Rel
     std::shared_ptr<NodeExpression> dstNode, RelDirectionType directionType) {
     auto catalog = clientContext->getCatalog();
     auto transaction = clientContext->getTransaction();
-    table_catalog_entry_set_t entrySet;
+    table_catalog_entry_set_t nodeEntrySet;
     for (auto entry : entries) {
-        auto& relTableEntry = entry->constCast<RelTableCatalogEntry>();
-        entrySet.insert(catalog->getTableCatalogEntry(transaction, relTableEntry.getSrcTableID()));
-        entrySet.insert(catalog->getTableCatalogEntry(transaction, relTableEntry.getDstTableID()));
+        auto& relGroupEntry = entry->constCast<RelGroupCatalogEntry>();
+        for (auto id : relGroupEntry.getSrcNodeTableIDSet()) {
+            nodeEntrySet.insert(catalog->getTableCatalogEntry(transaction, id));
+        }
+        for (auto id : relGroupEntry.getDstNodeTableIDSet()) {
+            nodeEntrySet.insert(catalog->getTableCatalogEntry(transaction, id));
+        }
     }
+    auto nodeEntries = std::vector<TableCatalogEntry*>{nodeEntrySet.begin(), nodeEntrySet.end()};
     auto recursivePatternInfo = relPattern.getRecursiveInfo();
     auto prevScope = saveScope();
     scope.clear();
     // Bind intermediate node.
-    auto node = createQueryNode(recursivePatternInfo->nodeName,
-        std::vector<TableCatalogEntry*>{entrySet.begin(), entrySet.end()});
+    auto node = createQueryNode(recursivePatternInfo->nodeName, nodeEntries);
     addToScope(node->toString(), node);
-    std::vector<StructField> nodeFields;
-    nodeFields.emplace_back(InternalKeyword::ID, node->getInternalID()->getDataType().copy());
-    nodeFields.emplace_back(InternalKeyword::LABEL,
-        node->getLabelExpression()->getDataType().copy());
+    auto nodeFields = getBaseNodeStructFields();
     auto nodeProjectionList = bindRecursivePatternNodeProjectionList(*recursivePatternInfo, *node);
     bindProjectionListAsStructField(nodeProjectionList, nodeFields);
-    auto nodeExtraInfo = std::make_unique<StructTypeInfo>(std::move(nodeFields));
-    node->setExtraTypeInfo(std::move(nodeExtraInfo));
-    auto nodeCopy = createQueryNode(recursivePatternInfo->nodeName,
-        std::vector<TableCatalogEntry*>{entrySet.begin(), entrySet.end()});
+    node->setDataType(LogicalType::NODE(std::move(nodeFields)));
+    auto nodeCopy = createQueryNode(recursivePatternInfo->nodeName, nodeEntries);
     // Bind intermediate rel
     auto rel = createNonRecursiveQueryRel(recursivePatternInfo->relName, entries,
         nullptr /* srcNode */, nullptr /* dstNode */, directionType);
     addToScope(rel->toString(), rel);
     auto relProjectionList = bindRecursivePatternRelProjectionList(*recursivePatternInfo, *rel);
-    std::vector<StructField> relFields;
-    relFields.emplace_back(InternalKeyword::SRC, LogicalType::INTERNAL_ID());
-    relFields.emplace_back(InternalKeyword::DST, LogicalType::INTERNAL_ID());
-    relFields.emplace_back(InternalKeyword::LABEL, rel->getLabelExpression()->getDataType().copy());
+    auto relFields = getBaseRelStructFields();
     relFields.emplace_back(InternalKeyword::ID, LogicalType::INTERNAL_ID());
     bindProjectionListAsStructField(relProjectionList, relFields);
-    auto relExtraInfo = std::make_unique<StructTypeInfo>(std::move(relFields));
-    rel->setExtraTypeInfo(std::move(relExtraInfo));
+    rel->setDataType(LogicalType::REL(std::move(relFields)));
     // Bind predicates in {}, e.g. [e* {date=1999-01-01}]
-    std::shared_ptr<Expression> relPredicate;
+    std::shared_ptr<Expression> relPredicate = nullptr;
     for (auto& [propertyName, rhs] : relPattern.getPropertyKeyVals()) {
         auto boundLhs = expressionBinder.bindNodeOrRelPropertyExpression(*rel, propertyName);
         auto boundRhs = expressionBinder.bindExpression(*rhs);
@@ -381,21 +391,22 @@ std::shared_ptr<RelExpression> Binder::createRecursiveQueryRel(const parser::Rel
             predicate);
     }
     // Bind predicates in (r, n | WHERE )
-    std::shared_ptr<Expression> nodePredicate;
     bool emptyRecursivePattern = false;
+    std::shared_ptr<Expression> nodePredicate = nullptr;
     if (recursivePatternInfo->whereExpression != nullptr) {
+        expressionBinder.config.disableLabelFunctionLiteralRewrite = true;
         auto wherePredicate = bindWhereExpression(*recursivePatternInfo->whereExpression);
+        expressionBinder.config.disableLabelFunctionLiteralRewrite = false;
         for (auto& predicate : wherePredicate->splitOnAND()) {
             auto collector = DependentVarNameCollector();
             collector.visit(predicate);
             auto dependentVariableNames = collector.getVarNames();
             auto dependOnNode = dependentVariableNames.contains(node->getUniqueName());
             auto dependOnRel = dependentVariableNames.contains(rel->getUniqueName());
-            auto canNotEvaluatePredicateMessage =
-                stringFormat("Cannot evaluate {} in recursive pattern {}.", predicate->toString(),
-                    relPattern.getVariableName());
             if (dependOnNode && dependOnRel) {
-                throw BinderException(canNotEvaluatePredicateMessage);
+                throw BinderException(
+                    stringFormat("Cannot evaluate {} because it depends on both {} and {}.",
+                        predicate->toString(), node->toString(), rel->toString()));
             } else if (dependOnNode) {
                 nodePredicate = expressionBinder.combineBooleanExpressions(ExpressionType::AND,
                     nodePredicate, predicate);
@@ -403,69 +414,77 @@ std::shared_ptr<RelExpression> Binder::createRecursiveQueryRel(const parser::Rel
                 relPredicate = expressionBinder.combineBooleanExpressions(ExpressionType::AND,
                     relPredicate, predicate);
             } else {
-                if (predicate->expressionType != ExpressionType::LITERAL ||
-                    predicate->dataType != LogicalType::BOOL()) {
-                    throw BinderException(canNotEvaluatePredicateMessage);
+                if (!ExpressionUtil::isBoolLiteral(*predicate)) {
+                    throw BinderException(stringFormat(
+                        "Cannot evaluate {} because it does not depend on {} or {}. Treating it as "
+                        "a node or relationship predicate is ambiguous.",
+                        predicate->toString(), node->toString(), rel->toString()));
                 }
                 // If predicate is true literal, we ignore.
                 // If predicate is false literal, we mark this recursive relationship as empty
                 // and later in planner we replace it with EmptyResult.
-                if (!predicate->constCast<LiteralExpression>().getValue().getValue<bool>()) {
+                if (!ExpressionUtil::getLiteralValue<bool>(*predicate)) {
                     emptyRecursivePattern = true;
                 }
             }
         }
     }
-    auto nodePredicateExecutionFlag = expressionBinder.createVariableExpression(LogicalType::BOOL(),
-        std::string(InternalKeyword::ANONYMOUS));
     // Bind rel
     restoreScope(std::move(prevScope));
     auto parsedName = relPattern.getVariableName();
-    auto tmpRelEntries = entries;
+    auto prunedRelEntries = entries;
     if (emptyRecursivePattern) {
-        tmpRelEntries.clear();
+        prunedRelEntries.clear();
     }
     auto queryRel = std::make_shared<RelExpression>(
         getRecursiveRelLogicalType(node->getDataType(), rel->getDataType()),
-        getUniqueExpressionName(parsedName), parsedName, tmpRelEntries, std::move(srcNode),
+        getUniqueExpressionName(parsedName), parsedName, prunedRelEntries, std::move(srcNode),
         std::move(dstNode), directionType, relPattern.getRelType());
-    auto lengthExpression =
-        PropertyExpression::construct(LogicalType::INT64(), InternalKeyword::LENGTH, *queryRel);
-    auto [lowerBound, upperBound] = bindVariableLengthRelBound(relPattern);
-    auto recursiveInfo = std::make_unique<RecursiveInfo>();
-    recursiveInfo->lowerBound = lowerBound;
-    recursiveInfo->upperBound = upperBound;
-    recursiveInfo->node = node;
-    recursiveInfo->nodeCopy = nodeCopy;
-    recursiveInfo->rel = rel;
-    recursiveInfo->lengthExpression = std::move(lengthExpression);
-    recursiveInfo->nodePredicateExecFlag = nodePredicateExecutionFlag;
-    recursiveInfo->originalNodePredicate = nodePredicate;
-    if (nodePredicate != nullptr) {
-        recursiveInfo->nodePredicate = expressionBinder.combineBooleanExpressions(
-            ExpressionType::OR, nodePredicate, nodePredicateExecutionFlag);
+    // Bind graph entry.
+    auto graphEntry = graph::NativeGraphEntry();
+    for (auto nodeEntry : node->getEntries()) {
+        graphEntry.nodeInfos.emplace_back(nodeEntry);
     }
-
-    recursiveInfo->relPredicate = std::move(relPredicate);
-    recursiveInfo->nodeProjectionList = std::move(nodeProjectionList);
-    recursiveInfo->relProjectionList = std::move(relProjectionList);
-
-    recursiveInfo->pathNodeIDsExpr =
+    for (auto relEntry : rel->getEntries()) {
+        graphEntry.relInfos.emplace_back(relEntry, rel, relPredicate);
+    }
+    auto bindData = std::make_unique<function::RJBindData>(graphEntry.copy());
+    // Bind lower upper bound.
+    auto [lowerBound, upperBound] = bindVariableLengthRelBound(relPattern);
+    bindData->lowerBound = lowerBound;
+    bindData->upperBound = upperBound;
+    // Bind semantic.
+    bindData->semantic = QueryRelTypeUtils::getPathSemantic(queryRel->getRelType());
+    // Bind path related expressions.
+    bindData->lengthExpr = construct(LogicalType::INT64(), InternalKeyword::LENGTH, *queryRel);
+    bindData->pathNodeIDsExpr =
         createInvisibleVariable("pathNodeIDs", LogicalType::LIST(LogicalType::INTERNAL_ID()));
-    recursiveInfo->pathEdgeIDsExpr =
+    bindData->pathEdgeIDsExpr =
         createInvisibleVariable("pathEdgeIDs", LogicalType::LIST(LogicalType::INTERNAL_ID()));
-    recursiveInfo->pathEdgeDirectionsExpr =
-        createInvisibleVariable("pathEdgeDirections", LogicalType::LIST(LogicalType::BOOL()));
-
-    if (relPattern.getRelType() == QueryRelType::WEIGHTED_SHORTEST) {
+    if (queryRel->getDirectionType() == RelDirectionType::BOTH) {
+        bindData->directionExpr =
+            createInvisibleVariable("pathEdgeDirections", LogicalType::LIST(LogicalType::BOOL()));
+    }
+    // Bind weighted path related expressions.
+    if (QueryRelTypeUtils::isWeighted(queryRel->getRelType())) {
         auto propertyExpr = expressionBinder.bindNodeOrRelPropertyExpression(*rel,
             recursivePatternInfo->weightPropertyName);
         checkWeightedShortestPathSupportedType(propertyExpr->getDataType());
-        recursiveInfo->weightPropertyExpr = propertyExpr;
-        recursiveInfo->weightOutputExpr =
-            createVariable(parsedName + "_weight", LogicalType::DOUBLE());
+        bindData->weightPropertyExpr = propertyExpr;
+        bindData->weightOutputExpr =
+            createInvisibleVariable(parsedName + "_cost", LogicalType::DOUBLE());
     }
 
+    auto recursiveInfo = std::make_unique<RecursiveInfo>();
+    recursiveInfo->node = node;
+    recursiveInfo->nodeCopy = nodeCopy;
+    recursiveInfo->rel = rel;
+    recursiveInfo->nodePredicate = std::move(nodePredicate);
+    recursiveInfo->relPredicate = std::move(relPredicate);
+    recursiveInfo->nodeProjectionList = std::move(nodeProjectionList);
+    recursiveInfo->relProjectionList = std::move(relProjectionList);
+    recursiveInfo->function = QueryRelTypeUtils::getFunction(queryRel->getRelType());
+    recursiveInfo->bindData = std::move(bindData);
     queryRel->setRecursiveInfo(std::move(recursiveInfo));
     return queryRel;
 }
@@ -474,8 +493,8 @@ expression_vector Binder::bindRecursivePatternNodeProjectionList(
     const RecursiveRelPatternInfo& info, const NodeOrRelExpression& expr) {
     expression_vector result;
     if (!info.hasProjection) {
-        for (auto& expression : expr.getPropertyExprsRef()) {
-            result.push_back(expression->copy());
+        for (auto& expression : expr.getPropertyExpressions()) {
+            result.push_back(expression);
         }
     } else {
         for (auto& expression : info.nodeProjectionList) {
@@ -489,11 +508,11 @@ expression_vector Binder::bindRecursivePatternRelProjectionList(const RecursiveR
     const NodeOrRelExpression& expr) {
     expression_vector result;
     if (!info.hasProjection) {
-        for (auto& expression : expr.getPropertyExprsRef()) {
-            if (expression->constCast<PropertyExpression>().isInternalID()) {
+        for (auto& property : expr.getPropertyExpressions()) {
+            if (property->isInternalID()) {
                 continue;
             }
-            result.push_back(expression->copy());
+            result.push_back(property);
         }
     } else {
         for (auto& expression : info.relProjectionList) {
@@ -503,8 +522,7 @@ expression_vector Binder::bindRecursivePatternRelProjectionList(const RecursiveR
     return result;
 }
 
-std::pair<uint64_t, uint64_t> Binder::bindVariableLengthRelBound(
-    const kuzu::parser::RelPattern& relPattern) {
+std::pair<uint64_t, uint64_t> Binder::bindVariableLengthRelBound(const RelPattern& relPattern) {
     auto recursiveInfo = relPattern.getRecursiveInfo();
     uint32_t lowerBound = 0;
     function::CastString::operation(
@@ -533,21 +551,6 @@ std::pair<uint64_t, uint64_t> Binder::bindVariableLengthRelBound(
     return std::make_pair(lowerBound, upperBound);
 }
 
-void Binder::bindQueryRelProperties(RelExpression& rel) {
-    if (rel.isEmpty()) {
-        auto internalID =
-            PropertyExpression::construct(LogicalType::INTERNAL_ID(), InternalKeyword::ID, rel);
-        rel.addPropertyExpression(InternalKeyword::ID, std::move(internalID));
-        return;
-    }
-    auto entries = rel.getEntries();
-    auto propertyNames = getPropertyNames(entries);
-    for (auto& propertyName : propertyNames) {
-        auto property = createPropertyExpression(propertyName, rel, entries);
-        rel.addPropertyExpression(propertyName, std::move(property));
-    }
-}
-
 std::shared_ptr<NodeExpression> Binder::bindQueryNode(const NodePattern& nodePattern,
     QueryGraph& queryGraph) {
     auto parsedName = nodePattern.getVariableName();
@@ -592,41 +595,29 @@ std::shared_ptr<NodeExpression> Binder::createQueryNode(const NodePattern& nodeP
 
 std::shared_ptr<NodeExpression> Binder::createQueryNode(const std::string& parsedName,
     const std::vector<TableCatalogEntry*>& entries) {
-    auto queryNode = make_shared<NodeExpression>(LogicalType(LogicalTypeID::NODE),
-        getUniqueExpressionName(parsedName), parsedName, entries);
+    auto uniqueName = getUniqueExpressionName(parsedName);
+    // Bind properties.
+    auto structFields = getBaseNodeStructFields();
+    std::vector<std::shared_ptr<PropertyExpression>> propertyExpressions;
+    for (auto& propertyName : getPropertyNames(entries)) {
+        auto property = createPropertyExpression(propertyName, uniqueName, parsedName, entries);
+        structFields.emplace_back(property->getPropertyName(), property->getDataType().copy());
+        propertyExpressions.push_back(std::move(property));
+    }
+    auto queryNode = std::make_shared<NodeExpression>(LogicalType::NODE(std::move(structFields)),
+        uniqueName, parsedName, entries);
     queryNode->setAlias(parsedName);
-    std::vector<std::string> fieldNames;
-    std::vector<LogicalType> fieldTypes;
+    for (auto& property : propertyExpressions) {
+        queryNode->addPropertyExpression(property);
+    }
     // Bind internal expressions
     queryNode->setInternalID(
-        PropertyExpression::construct(LogicalType::INTERNAL_ID(), InternalKeyword::ID, *queryNode));
+        construct(LogicalType::INTERNAL_ID(), InternalKeyword::ID, *queryNode));
     auto input = function::RewriteFunctionBindInput(clientContext, &expressionBinder, {queryNode});
     queryNode->setLabelExpression(function::LabelFunction::rewriteFunc(input));
-    fieldNames.emplace_back(InternalKeyword::ID);
-    fieldNames.emplace_back(InternalKeyword::LABEL);
-    fieldTypes.push_back(queryNode->getInternalID()->getDataType().copy());
-    fieldTypes.push_back(queryNode->getLabelExpression()->getDataType().copy());
-    // Bind properties.
-    bindQueryNodeProperties(*queryNode);
-    for (auto& expression : queryNode->getPropertyExprsRef()) {
-        auto property = ku_dynamic_cast<PropertyExpression*>(expression.get());
-        fieldNames.emplace_back(property->getPropertyName());
-        fieldTypes.emplace_back(property->dataType.copy());
-    }
-    auto extraInfo = std::make_unique<StructTypeInfo>(fieldNames, fieldTypes);
-    queryNode->setExtraTypeInfo(std::move(extraInfo));
     return queryNode;
 }
 
-void Binder::bindQueryNodeProperties(NodeExpression& node) {
-    auto entries = node.getEntries();
-    auto propertyNames = getPropertyNames(entries);
-    for (auto& propertyName : propertyNames) {
-        auto property = createPropertyExpression(propertyName, node, entries);
-        node.addPropertyExpression(propertyName, std::move(property));
-    }
-}
-
 static std::vector<TableCatalogEntry*> sortEntries(const table_catalog_entry_set_t& set) {
     std::vector<TableCatalogEntry*> entries;
     for (auto entry : set) {
@@ -645,7 +636,7 @@ std::vector<TableCatalogEntry*> Binder::bindNodeTableEntries(
     auto catalog = clientContext->getCatalog();
     auto useInternal = clientContext->useInternalCatalogEntry();
     table_catalog_entry_set_t entrySet;
-    if (tableNames.empty()) {
+    if (tableNames.empty()) { // Rewrite as all node tables in database.
         for (auto entry : catalog->getNodeTableEntries(transaction, useInternal)) {
             entrySet.insert(entry);
         }
@@ -672,27 +663,21 @@ TableCatalogEntry* Binder::bindNodeTableEntry(const std::string& name) const {
     return catalog->getTableCatalogEntry(transaction, name, useInternal);
 }
 
-std::vector<TableCatalogEntry*> Binder::bindRelTableEntries(
+std::vector<TableCatalogEntry*> Binder::bindRelGroupEntries(
     const std::vector<std::string>& tableNames) const {
     auto transaction = clientContext->getTransaction();
     auto catalog = clientContext->getCatalog();
     auto useInternal = clientContext->useInternalCatalogEntry();
     table_catalog_entry_set_t entrySet;
-    if (tableNames.empty()) {
-        for (auto& entry : catalog->getRelTableEntries(transaction, useInternal)) {
+    if (tableNames.empty()) { // Rewrite as all rel groups in database.
+        for (auto entry : catalog->getRelGroupEntries(transaction, useInternal)) {
             entrySet.insert(entry);
         }
     } else {
         for (auto& name : tableNames) {
-            if (catalog->containsRelGroup(transaction, name)) {
-                auto groupEntry = catalog->getRelGroupEntry(transaction, name);
-                for (auto& id : groupEntry->getRelTableIDs()) {
-                    auto relEntry = catalog->getTableCatalogEntry(transaction, id);
-                    entrySet.insert(relEntry);
-                }
-            } else if (catalog->containsTable(transaction, name)) {
+            if (catalog->containsTable(transaction, name)) {
                 auto entry = catalog->getTableCatalogEntry(transaction, name, useInternal);
-                if (entry->getType() != CatalogEntryType::REL_TABLE_ENTRY) {
+                if (entry->getType() != CatalogEntryType::REL_GROUP_ENTRY) {
                     throw BinderException(stringFormat(
                         "Cannot bind {} as a relationship pattern label.", entry->getName()));
                 }
