@@ -2,6 +2,7 @@
 #include "binder/bound_standalone_call_function.h"
 #include "catalog/catalog.h"
 #include "common/exception/binder.h"
+#include "main/client_context.h"
 #include "parser/expression/parsed_function_expression.h"
 #include "parser/standalone_call_function.h"
 
@@ -23,8 +24,7 @@ std::unique_ptr<BoundStatement> Binder::bindStandaloneCallFunction(
         throw BinderException(
             "Only standalone table functions can be called without return statement.");
     }
-    expression_vector columns;
-    auto boundTableFunction = bindTableFunc(funcName, funcExpr, columns, {} /* yieldVariables */);
+    auto boundTableFunction = bindTableFunc(funcName, funcExpr, {} /* yieldVariables */);
     return std::make_unique<BoundStandaloneCallFunction>(std::move(boundTableFunction));
 }
 
