@@ -2,10 +2,13 @@
 
 #include <algorithm>
 
+#include "binder/binder.h"
 #include "binder/expression/literal_expression.h"
 #include "binder/expression/node_rel_expression.h"
 #include "binder/expression/parameter_expression.h"
 #include "common/exception/binder.h"
+#include "common/exception/runtime.h"
+#include "common/type_utils.h"
 #include "common/types/value/nested.h"
 
 using namespace kuzu::common;
@@ -162,6 +165,13 @@ bool ExpressionUtil::isNullLiteral(const Expression& expression) {
     return expression.constCast<LiteralExpression>().getValue().isNull();
 }
 
+bool ExpressionUtil::isBoolLiteral(const Expression& expression) {
+    if (expression.expressionType != ExpressionType::LITERAL) {
+        return false;
+    }
+    return expression.dataType == LogicalType::BOOL();
+}
+
 bool ExpressionUtil::isFalseLiteral(const Expression& expression) {
     if (expression.expressionType != ExpressionType::LITERAL) {
         return false;
@@ -170,10 +180,17 @@ bool ExpressionUtil::isFalseLiteral(const Expression& expression) {
 }
 
 bool ExpressionUtil::isEmptyList(const Expression& expression) {
-    if (expression.expressionType != ExpressionType::LITERAL) {
+    auto val = Value::createNullValue();
+    switch (expression.expressionType) {
+    case ExpressionType::LITERAL: {
+        val = expression.constCast<LiteralExpression>().getValue();
+    } break;
+    case ExpressionType::PARAMETER: {
+        val = expression.constCast<ParameterExpression>().getValue();
+    } break;
+    default:
         return false;
     }
-    auto val = expression.constCast<LiteralExpression>().getValue();
     if (val.getDataType().getLogicalTypeID() != LogicalTypeID::LIST) {
         return false;
     }
@@ -269,6 +286,9 @@ double ExpressionUtil::getLiteralValue(const Expression& expr) {
 // E.g. [NULL] is compatible with [1,2]
 // E.g. {a: NULL, b: NULL} is compatible with {a: [1,2], b: ['c']}
 static bool compatible(const LogicalType& type, const LogicalType& target) {
+    if (!type.isInternalType()) {
+        return false;
+    }
     if (type.getLogicalTypeID() == LogicalTypeID::ANY) {
         return true;
     }
@@ -409,9 +429,14 @@ bool ExpressionUtil::canCastStatically(const Expression& expr, const LogicalType
 }
 
 bool ExpressionUtil::canEvaluateAsLiteral(const Expression& expr) {
-    return (expr.expressionType == ExpressionType::LITERAL ||
-            (expr.expressionType == ExpressionType::PARAMETER &&
-                expr.getDataType().getLogicalTypeID() != LogicalTypeID::ANY));
+    switch (expr.expressionType) {
+    case ExpressionType::LITERAL:
+        return true;
+    case ExpressionType::PARAMETER:
+        return expr.getDataType().getLogicalTypeID() != LogicalTypeID::ANY;
+    default:
+        return false;
+    }
 }
 
 Value ExpressionUtil::evaluateAsLiteralValue(const Expression& expr) {
@@ -430,5 +455,97 @@ Value ExpressionUtil::evaluateAsLiteralValue(const Expression& expr) {
     return value;
 }
 
+uint64_t ExpressionUtil::evaluateAsSkipLimit(const Expression& expr) {
+    auto value = evaluateAsLiteralValue(expr);
+    auto errorMsg = "The number of rows to skip/limit must be a non-negative integer.";
+    uint64_t number = INVALID_LIMIT;
+    TypeUtils::visit(
+        value.getDataType(),
+        [&]<IntegerTypes T>(T) {
+            if (value.getValue<T>() < 0) {
+                throw RuntimeException{errorMsg};
+            }
+            number = (uint64_t)value.getValue<T>();
+        },
+        [&](auto) { throw RuntimeException{errorMsg}; });
+    return number;
+}
+
+template<typename T>
+T ExpressionUtil::getExpressionVal(const Expression& expr, const Value& value,
+    const LogicalType& targetType, validate_param_func<T> validateParamFunc) {
+    if (value.getDataType() != targetType) {
+        throw RuntimeException{common::stringFormat("Parameter: {} must be a {} literal.",
+            expr.getAlias(), targetType.toString())};
+    }
+    T val = value.getValue<T>();
+    if (validateParamFunc != nullptr) {
+        validateParamFunc(val);
+    }
+    return val;
+}
+
+template<typename T>
+T ExpressionUtil::evaluateLiteral(const Expression& expression, const common::LogicalType& type,
+    validate_param_func<T> validateParamFunc) {
+    if (!canEvaluateAsLiteral(expression)) {
+        std::string errMsg;
+        switch (expression.expressionType) {
+        case ExpressionType::PARAMETER: {
+            errMsg = "The query is a parameter expression. Please assign it a value.";
+        } break;
+        default: {
+            errMsg = "The query must be a parameter/literal expression.";
+        } break;
+        }
+        throw RuntimeException{errMsg};
+    }
+    auto value = evaluateAsLiteralValue(expression);
+    return getExpressionVal(expression, value, type, validateParamFunc);
+}
+
+std::shared_ptr<Expression> ExpressionUtil::applyImplicitCastingIfNecessary(
+    main::ClientContext* context, std::shared_ptr<Expression> expr,
+    common::LogicalType targetType) {
+    if (expr->getDataType() != targetType) {
+        binder::Binder binder{context};
+        expr = binder.getExpressionBinder()->implicitCastIfNecessary(expr, targetType);
+        expr = binder.getExpressionBinder()->foldExpression(expr);
+    }
+    return expr;
+}
+
+template KUZU_API std::string ExpressionUtil::getExpressionVal(const Expression& expr,
+    const common::Value& value, const common::LogicalType& targetType,
+    validate_param_func<std::string> validateParamFunc);
+
+template KUZU_API double ExpressionUtil::getExpressionVal(const Expression& expr,
+    const common::Value& value, const common::LogicalType& targetType,
+    validate_param_func<double> validateParamFunc);
+
+template KUZU_API int64_t ExpressionUtil::getExpressionVal(const Expression& expr,
+    const common::Value& value, const common::LogicalType& targetType,
+    validate_param_func<int64_t> validateParamFunc);
+
+template KUZU_API bool ExpressionUtil::getExpressionVal(const Expression& expr,
+    const common::Value& value, const common::LogicalType& targetType,
+    validate_param_func<bool> validateParamFunc);
+
+template KUZU_API std::string ExpressionUtil::evaluateLiteral<std::string>(
+    const Expression& expression, const LogicalType& type,
+    validate_param_func<std::string> validateParamFunc);
+
+template KUZU_API double ExpressionUtil::evaluateLiteral<double>(const Expression& expression,
+    const LogicalType& type, validate_param_func<double> validateParamFunc);
+
+template KUZU_API int64_t ExpressionUtil::evaluateLiteral<int64_t>(const Expression& expression,
+    const LogicalType& type, validate_param_func<int64_t> validateParamFunc);
+
+template KUZU_API bool ExpressionUtil::evaluateLiteral<bool>(const Expression& expression,
+    const LogicalType& type, validate_param_func<bool> validateParamFunc);
+
+template KUZU_API uint64_t ExpressionUtil::evaluateLiteral<uint64_t>(const Expression& expression,
+    const LogicalType& type, validate_param_func<uint64_t> validateParamFunc);
+
 } // namespace binder
 } // namespace kuzu
