@@ -548,13 +548,6 @@ void ArrowRowBatch::templateCopyNullValue<LogicalTypeID::LIST>(ArrowVector* vect
     vector->numNulls++;
 }
 
-template<>
-void ArrowRowBatch::templateCopyNullValue<LogicalTypeID::ARRAY>(ArrowVector* vector,
-    std::int64_t pos) {
-    setBitToZero(vector->validity.data(), pos);
-    vector->numNulls++;
-}
-
 template<>
 void ArrowRowBatch::templateCopyNullValue<LogicalTypeID::MAP>(ArrowVector* vector,
     std::int64_t pos) {
@@ -577,6 +570,13 @@ void ArrowRowBatch::copyNullValueUnion(ArrowVector* vector, Value* value, std::i
     vector->numNulls++;
 }
 
+static void copyArrowArray(ArrowVector* vector, std::int64_t pos, uint64_t numElements) {
+    setBitToZero(vector->validity.data(), pos);
+    vector->numNulls++;
+    auto& child = vector->childData[0];
+    child->numValues += numElements;
+}
+
 void ArrowRowBatch::copyNullValue(ArrowVector* vector, Value* value, std::int64_t pos) {
     switch (value->dataType.getLogicalTypeID()) {
     case LogicalTypeID::BOOL: {
@@ -647,7 +647,7 @@ void ArrowRowBatch::copyNullValue(ArrowVector* vector, Value* value, std::int64_
         templateCopyNullValue<LogicalTypeID::LIST>(vector, pos);
     } break;
     case LogicalTypeID::ARRAY: {
-        templateCopyNullValue<LogicalTypeID::ARRAY>(vector, pos);
+        copyArrowArray(vector, pos, ArrayType::getNumElements(value->dataType));
     } break;
     case LogicalTypeID::MAP: {
         templateCopyNullValue<LogicalTypeID::MAP>(vector, pos);
@@ -803,7 +803,7 @@ ArrowArray* ArrowRowBatch::templateCreateArray<LogicalTypeID::UNION>(ArrowVector
     vector.array->offset = 0;
     vector.array->dictionary = nullptr;
     vector.array->buffers = vector.buffers.data();
-    vector.array->null_count = vector.numNulls;
+    vector.array->null_count = 0;
     vector.array->length = vector.numValues;
     vector.array->n_buffers = 2;
     vector.array->buffers[0] = vector.data.data();
