@@ -5,6 +5,7 @@
 
 #include "common/system_config.h"
 #include "common/types/types.h"
+#include "common/vector/value_vector.h"
 
 namespace kuzu {
 namespace common {
@@ -17,18 +18,31 @@ static const std::array<sel_t, DEFAULT_VECTOR_CAPACITY> INCREMENTAL_SELECTED_POS
         return selectedPos;
     }();
 
+SelectionView::SelectionView(sel_t selectedSize)
+    : selectedPositions{INCREMENTAL_SELECTED_POS.data()}, selectedSize{selectedSize},
+      state{State::STATIC} {}
+
 SelectionVector::SelectionVector() : SelectionVector{DEFAULT_VECTOR_CAPACITY} {}
 
 void SelectionVector::setToUnfiltered() {
-    selectedPositions = const_cast<sel_t*>(INCREMENTAL_SELECTED_POS.data());
+    selectedPositions = INCREMENTAL_SELECTED_POS.data();
     state = State::STATIC;
 }
 void SelectionVector::setToUnfiltered(sel_t size) {
     KU_ASSERT(size <= capacity);
-    selectedPositions = const_cast<sel_t*>(INCREMENTAL_SELECTED_POS.data());
+    selectedPositions = INCREMENTAL_SELECTED_POS.data();
     selectedSize = size;
     state = State::STATIC;
 }
 
+std::vector<SelectionVector*> SelectionVector::fromValueVectors(
+    const std::vector<std::shared_ptr<ValueVector>>& vec) {
+    std::vector<SelectionVector*> ret(vec.size());
+    for (size_t i = 0; i < vec.size(); ++i) {
+        ret[i] = vec[i]->getSelVectorPtr();
+    }
+    return ret;
+}
+
 } // namespace common
 } // namespace kuzu
