@@ -17,7 +17,7 @@ std::string RelDirectionUtils::relDirectionToString(RelDataDirection direction)
     return directionStrs[relDirectionToKeyIdx(direction)];
 }
 
-common::idx_t RelDirectionUtils::relDirectionToKeyIdx(RelDataDirection direction) {
+idx_t RelDirectionUtils::relDirectionToKeyIdx(RelDataDirection direction) {
     switch (direction) {
     case RelDataDirection::FWD:
         return 0;
@@ -28,5 +28,17 @@ common::idx_t RelDirectionUtils::relDirectionToKeyIdx(RelDataDirection direction
     }
 }
 
+table_id_t RelDirectionUtils::getNbrTableID(RelDataDirection direction, table_id_t srcTableID,
+    table_id_t dstTableID) {
+    switch (direction) {
+    case RelDataDirection::FWD:
+        return dstTableID;
+    case RelDataDirection::BWD:
+        return srcTableID;
+    default:
+        KU_UNREACHABLE;
+    }
+}
+
 } // namespace common
 } // namespace kuzu
