@@ -1,5 +1,7 @@
 #include "common/file_system/file_system.h"
 
+#include "common/string_utils.h"
+
 namespace kuzu {
 namespace common {
 
@@ -15,7 +17,7 @@ void FileSystem::createDir(const std::string& /*dir*/) const {
     KU_UNREACHABLE;
 }
 
-void FileSystem::removeFileIfExists(const std::string& /*path*/) {
+void FileSystem::removeFileIfExists(const std::string&, const main::ClientContext* /*context*/) {
     KU_UNREACHABLE;
 }
 
@@ -33,7 +35,15 @@ std::string FileSystem::joinPath(const std::string& base, const std::string& par
 }
 
 std::string FileSystem::getFileExtension(const std::filesystem::path& path) {
-    return path.extension().string();
+    auto extension = path.extension();
+    if (isCompressedFile(path)) {
+        extension = path.stem().extension();
+    }
+    return extension.string();
+}
+
+bool FileSystem::isCompressedFile(const std::filesystem::path& path) {
+    return isGZIPCompressed(path);
 }
 
 std::string FileSystem::getFileName(const std::filesystem::path& path) {
@@ -49,5 +59,14 @@ void FileSystem::truncate(FileInfo& /*fileInfo*/, uint64_t /*size*/) const {
     KU_UNREACHABLE;
 }
 
+void FileSystem::reset(FileInfo& fileInfo) {
+    fileInfo.seek(0, SEEK_SET);
+}
+
+bool FileSystem::isGZIPCompressed(const std::filesystem::path& path) {
+    auto extensionLowerCase = StringUtils::getLower(path.extension().string());
+    return extensionLowerCase == ".gz" || extensionLowerCase == ".gzip";
+}
+
 } // namespace common
 } // namespace kuzu
