@@ -23,6 +23,8 @@
 
 #include <cstring>
 
+#include "storage/storage_utils.h"
+
 namespace kuzu {
 namespace common {
 
@@ -53,14 +55,15 @@ static void validateFileFlags(uint8_t flags) {
                 flags & FileFlags::CREATE_AND_TRUNCATE_IF_EXISTS));
 }
 
-std::unique_ptr<FileInfo> LocalFileSystem::openFile(const std::string& path, int flags,
-    main::ClientContext* context, FileLockType lock_type) {
+std::unique_ptr<FileInfo> LocalFileSystem::openFile(const std::string& path, FileOpenFlags flags,
+    main::ClientContext* context) {
     auto fullPath = expandPath(context, path);
-    validateFileFlags(flags);
+    auto fileFlags = flags.flags;
+    validateFileFlags(fileFlags);
 
     int openFlags = 0;
-    bool readMode = flags & FileFlags::READ_ONLY;
-    bool writeMode = flags & FileFlags::WRITE;
+    bool readMode = fileFlags & FileFlags::READ_ONLY;
+    bool writeMode = fileFlags & FileFlags::WRITE;
     if (readMode && writeMode) {
         openFlags = O_RDWR;
     } else if (readMode) {
@@ -73,10 +76,10 @@ std::unique_ptr<FileInfo> LocalFileSystem::openFile(const std::string& path, int
         // LCOV_EXCL_STOP
     }
     if (writeMode) {
-        KU_ASSERT(flags & FileFlags::WRITE);
-        if (flags & FileFlags::CREATE_IF_NOT_EXISTS) {
+        KU_ASSERT(fileFlags & FileFlags::WRITE);
+        if (fileFlags & FileFlags::CREATE_IF_NOT_EXISTS) {
             openFlags |= O_CREAT;
-        } else if (flags & FileFlags::CREATE_AND_TRUNCATE_IF_EXISTS) {
+        } else if (fileFlags & FileFlags::CREATE_AND_TRUNCATE_IF_EXISTS) {
             openFlags |= O_CREAT | O_TRUNC;
         }
     }
@@ -84,9 +87,9 @@ std::unique_ptr<FileInfo> LocalFileSystem::openFile(const std::string& path, int
 #if defined(_WIN32)
     auto dwDesiredAccess = 0ul;
     int dwCreationDisposition;
-    if (flags & FileFlags::CREATE_IF_NOT_EXISTS) {
+    if (fileFlags & FileFlags::CREATE_IF_NOT_EXISTS) {
         dwCreationDisposition = OPEN_ALWAYS;
-    } else if (flags & FileFlags::CREATE_AND_TRUNCATE_IF_EXISTS) {
+    } else if (fileFlags & FileFlags::CREATE_AND_TRUNCATE_IF_EXISTS) {
         dwCreationDisposition = CREATE_ALWAYS;
     } else {
         dwCreationDisposition = OPEN_EXISTING;
@@ -109,8 +112,8 @@ std::unique_ptr<FileInfo> LocalFileSystem::openFile(const std::string& path, int
         throw IOException(stringFormat("Cannot open file. path: {} - Error {}: {}", fullPath,
             GetLastError(), std::system_category().message(GetLastError())));
     }
-    if (lock_type != FileLockType::NO_LOCK) {
-        DWORD dwFlags = lock_type == FileLockType::READ_LOCK ?
+    if (flags.lockType != FileLockType::NO_LOCK) {
+        DWORD dwFlags = flags.lockType == FileLockType::READ_LOCK ?
                             LOCKFILE_FAIL_IMMEDIATELY :
                             LOCKFILE_FAIL_IMMEDIATELY | LOCKFILE_EXCLUSIVE_LOCK;
         OVERLAPPED overlapped = {0};
@@ -129,10 +132,10 @@ std::unique_ptr<FileInfo> LocalFileSystem::openFile(const std::string& path, int
     if (fd == -1) {
         throw IOException(stringFormat("Cannot open file {}: {}", fullPath, posixErrMessage()));
     }
-    if (lock_type != FileLockType::NO_LOCK) {
+    if (flags.lockType != FileLockType::NO_LOCK) {
         struct flock fl {};
         memset(&fl, 0, sizeof fl);
-        fl.l_type = lock_type == FileLockType::READ_LOCK ? F_RDLCK : F_WRLCK;
+        fl.l_type = flags.lockType == FileLockType::READ_LOCK ? F_RDLCK : F_WRLCK;
         fl.l_whence = SEEK_SET;
         fl.l_start = 0;
         fl.l_len = 0;
@@ -172,9 +175,9 @@ std::vector<std::string> LocalFileSystem::glob(main::ClientContext* context,
             auto fileSearchPath = context->getCurrentSetting(main::FileSearchPathSetting::name)
                                       .getValue<std::string>();
             if (fileSearchPath != "") {
-                auto searchPaths = common::StringUtils::split(fileSearchPath, ",");
+                auto searchPaths = StringUtils::split(fileSearchPath, ",");
                 for (auto& searchPath : searchPaths) {
-                    pathsToGlob.push_back(common::stringFormat("{}/{}", searchPath, path));
+                    pathsToGlob.push_back(stringFormat("{}/{}", searchPath, path));
                 }
             }
         }
@@ -189,8 +192,9 @@ std::vector<std::string> LocalFileSystem::glob(main::ClientContext* context,
 }
 
 void LocalFileSystem::overwriteFile(const std::string& from, const std::string& to) {
-    if (!fileOrPathExists(from) || !fileOrPathExists(to))
+    if (!fileOrPathExists(from) || !fileOrPathExists(to)) {
         return;
+    }
     std::error_code errorCode;
     if (!std::filesystem::copy_file(from, to, std::filesystem::copy_options::overwrite_existing,
             errorCode)) {
@@ -202,8 +206,9 @@ void LocalFileSystem::overwriteFile(const std::string& from, const std::string&
 }
 
 void LocalFileSystem::copyFile(const std::string& from, const std::string& to) {
-    if (!fileOrPathExists(from))
+    if (!fileOrPathExists(from)) {
         return;
+    }
     std::error_code errorCode;
     if (!std::filesystem::copy_file(from, to, std::filesystem::copy_options::none, errorCode)) {
         // LCOV_EXCL_START
@@ -252,33 +257,36 @@ void LocalFileSystem::createDir(const std::string& dir) const {
     }
 }
 
-bool isSubdirectory(const std::filesystem::path& base, const std::filesystem::path& sub) {
-    try {
-        // Resolve paths to their canonical form
-        auto canonicalBase = std::filesystem::canonical(base);
-        auto canonicalSub = std::filesystem::canonical(sub);
-
-        std::string relative = std::filesystem::relative(canonicalSub, canonicalBase).string();
-        // Size check for a "." result.
-        // If the path starts with "..", it's not a subdirectory.
-        return !relative.empty() && !(relative.starts_with(".."));
-
-    } catch (const std::filesystem::filesystem_error& e) {
-        // Handle errors, e.g., if paths don't exist
-        std::cerr << "Filesystem error: " << e.what() << std::endl;
-        return false;
-    }
-
-    return false;
+static std::unordered_set<std::string> getDatabaseFileSet(const std::string& path) {
+    std::unordered_set<std::string> result;
+    result.insert(storage::StorageUtils::getWALFilePath(path));
+    result.insert(storage::StorageUtils::getShadowFilePath(path));
+    result.insert(storage::StorageUtils::getTmpFilePath(path));
+    return result;
 }
 
-void LocalFileSystem::removeFileIfExists(const std::string& path) {
+static bool isExtensionFile(const main::ClientContext* context, const std::string& path) {
+    if (context == nullptr) {
+        return false;
+    }
+    auto extensionDir = context->getExtensionDir();
+    std::filesystem::path rel = std::filesystem::relative(path, extensionDir);
+    for (const auto& part : rel) {
+        if (part == "..") {
+            return false;
+        }
+    }
+    return true;
+}
+
+void LocalFileSystem::removeFileIfExists(const std::string& path,
+    const main::ClientContext* context) {
     if (!fileOrPathExists(path)) {
         return;
     }
-    if (!isSubdirectory(homeDir, path)) {
-        throw IOException(stringFormat("Error: Path {} is not within the allowed home directory {}",
-            path, homeDir));
+    if (!getDatabaseFileSet(dbPath).contains(path) && !isExtensionFile(context, path)) {
+        throw IOException(stringFormat(
+            "Error: Path {} is not within the allowed list of files to be removed.", path));
     }
     std::error_code errCode;
     bool success = false;
@@ -342,12 +350,14 @@ std::string LocalFileSystem::expandPath(main::ClientContext* context,
 bool LocalFileSystem::isLocalPath(const std::string& path) {
     return path.rfind("s3://", 0) != 0 && path.rfind("gs://", 0) != 0 &&
            path.rfind("gcs://", 0) != 0 && path.rfind("http://", 0) != 0 &&
-           path.rfind("https://", 0) != 0;
+           path.rfind("https://", 0) != 0 && path.rfind("az://", 0) != 0 &&
+           path.rfind("abfss://", 0) != 0;
 }
 
 void LocalFileSystem::readFromFile(FileInfo& fileInfo, void* buffer, uint64_t numBytes,
     uint64_t position) const {
     auto localFileInfo = fileInfo.constPtrCast<LocalFileInfo>();
+    KU_ASSERT(localFileInfo->getFileSize() >= position + numBytes);
 #if defined(_WIN32)
     DWORD numBytesRead;
     OVERLAPPED overlapped{0, 0, 0, 0};
@@ -368,7 +378,7 @@ void LocalFileSystem::readFromFile(FileInfo& fileInfo, void* buffer, uint64_t nu
     }
 #else
     auto numBytesRead = pread(localFileInfo->fd, buffer, numBytes, position);
-    if ((uint64_t)numBytesRead != numBytes &&
+    if (static_cast<uint64_t>(numBytesRead) != numBytes &&
         localFileInfo->getFileSize() != position + numBytesRead) {
         // LCOV_EXCL_START
         throw IOException(stringFormat("Cannot read from file: {} fileDescriptor: {} "
@@ -417,7 +427,7 @@ void LocalFileSystem::writeFile(FileInfo& fileInfo, const uint8_t* buffer, uint6
 #else
         auto numBytesWritten =
             pwrite(localFileInfo->fd, buffer + bufferOffset, numBytesToWrite, offset);
-        if (numBytesWritten != (int64_t)numBytesToWrite) {
+        if (numBytesWritten != static_cast<int64_t>(numBytesToWrite)) {
             // LCOV_EXCL_START
             throw IOException(
                 stringFormat("Cannot write to file. path: {} fileDescriptor: {} offsetToWrite: {} "
