@@ -3,10 +3,6 @@
 namespace kuzu {
 namespace common {
 
-Task::Task(uint64_t maxNumThreads)
-    : parent{nullptr}, maxNumThreads{maxNumThreads}, numThreadsFinished{0}, numThreadsRegistered{0},
-      exceptionsPtr{nullptr}, ID{UINT64_MAX} {}
-
 bool Task::registerThread() {
     lock_t lck{taskMtx};
     if (!hasExceptionNoLock() && canRegisterNoLock()) {
@@ -21,7 +17,7 @@ void Task::deRegisterThreadAndFinalizeTask() {
     ++numThreadsFinished;
     if (!hasExceptionNoLock() && isCompletedNoLock()) {
         try {
-            finalizeIfNecessary();
+            finalize();
         } catch (std::exception& e) {
             setExceptionNoLock(std::current_exception());
         }
