@@ -397,7 +397,7 @@ std::unique_ptr<ValueVector> ValueVector::deSerialize(Deserializer& deSer,
     deSer.validateDebuggingInfo(key, "data_type");
     auto dataType = LogicalType::deserialize(deSer);
     auto result = std::make_unique<ValueVector>(std::move(dataType), mm);
-    result->state = dataChunkState;
+    result->setState(dataChunkState);
     deSer.validateDebuggingInfo(key, "num_values");
     sel_t numValues = 0;
     deSer.deserializeValue<sel_t>(numValues);
@@ -481,7 +481,7 @@ void StringVector::addString(ValueVector* vector, uint32_t vectorPos, const char
     }
 }
 
-void StringVector::addString(ValueVector* vector, uint32_t vectorPos, const std::string& srcStr) {
+void StringVector::addString(ValueVector* vector, uint32_t vectorPos, std::string_view srcStr) {
     addString(vector, vectorPos, srcStr.data(), srcStr.length());
 }
 
@@ -546,25 +546,24 @@ void StringVector::copyToRowData(const ValueVector* vector, uint32_t pos, uint8_
     }
 }
 
-void ListVector::copyListEntryAndBufferMetaData(ValueVector& vector, const ValueVector& other) {
-    auto& selVector = vector.state->getSelVector();
-    auto& otherSelVector = other.state->getSelVector();
+void ListVector::copyListEntryAndBufferMetaData(ValueVector& vector,
+    const SelectionVector& selVector, const ValueVector& other,
+    const SelectionVector& otherSelVector) {
     KU_ASSERT(selVector.getSelSize() == otherSelVector.getSelSize());
     // Copy list entries
     for (auto i = 0u; i < otherSelVector.getSelSize(); ++i) {
         auto pos = selVector[i];
         auto otherPos = otherSelVector[i];
-        if (other.isNull(otherPos)) {
-            vector.setNull(pos, true);
-        } else {
+        vector.setNull(pos, other.isNull(pos));
+        if (!other.isNull(otherPos)) {
             vector.setValue(pos, other.getValue<list_entry_t>(otherPos));
         }
     }
     // Copy buffer metadata
     auto& buffer = getAuxBufferUnsafe(vector);
     auto& otherBuffer = getAuxBuffer(other);
-    buffer.size = otherBuffer.size;
     buffer.capacity = otherBuffer.capacity;
+    buffer.size = otherBuffer.size;
 }
 
 void ListVector::copyFromRowData(ValueVector* vector, uint32_t pos, const uint8_t* rowData) {
