@@ -12,19 +12,17 @@ namespace kuzu {
 namespace function {
 
 void CountFunction::updateAll(uint8_t* state_, ValueVector* input, uint64_t multiplicity,
-    MemoryManager* /*memoryManager*/) {
+    InMemOverflowBuffer* /*overflowBuffer*/) {
     auto state = reinterpret_cast<CountState*>(state_);
     state->count += multiplicity * input->countNonNull();
 }
 
-void CountFunction::paramRewriteFunc(binder::expression_vector& arguments) {
+void CountFunction::paramRewriteFunc(expression_vector& arguments) {
     KU_ASSERT(arguments.size() == 1);
     if (ExpressionUtil::isNodePattern(*arguments[0])) {
-        auto node = (NodeExpression*)arguments[0].get();
-        arguments[0] = node->getInternalID();
+        arguments[0] = arguments[0]->constCast<NodeExpression>().getInternalID();
     } else if (ExpressionUtil::isRelPattern(*arguments[0])) {
-        auto rel = (RelExpression*)arguments[0].get();
-        arguments[0] = rel->getInternalIDProperty();
+        arguments[0] = arguments[0]->constCast<RelExpression>().getInternalID();
     }
 }
 
@@ -32,8 +30,10 @@ function_set CountFunction::getFunctionSet() {
     function_set result;
     for (auto& type : LogicalTypeUtils::getAllValidLogicTypeIDs()) {
         for (auto isDistinct : std::vector<bool>{true, false}) {
-            result.push_back(AggregateFunctionUtils::getAggFunc<CountFunction>(name, type,
-                LogicalTypeID::INT64, isDistinct, paramRewriteFunc));
+            auto func = AggregateFunctionUtils::getAggFunc<CountFunction>(name, type,
+                LogicalTypeID::INT64, isDistinct, paramRewriteFunc);
+            func->needToHandleNulls = true;
+            result.push_back(std::move(func));
         }
     }
     return result;
