@@ -7,7 +7,7 @@ namespace kuzu {
 namespace function {
 
 void CountStarFunction::updateAll(uint8_t* state_, ValueVector* input, uint64_t multiplicity,
-    MemoryManager* /*memoryManager*/) {
+    InMemOverflowBuffer* /*overflowBuffer*/) {
     auto state = reinterpret_cast<CountState*>(state_);
     KU_ASSERT(input == nullptr);
     (void)input;
@@ -15,7 +15,7 @@ void CountStarFunction::updateAll(uint8_t* state_, ValueVector* input, uint64_t
 }
 
 void CountStarFunction::updatePos(uint8_t* state_, ValueVector* input, uint64_t multiplicity,
-    uint32_t /*pos*/, MemoryManager* /*memoryManager*/) {
+    uint32_t /*pos*/, InMemOverflowBuffer* /*overflowBuffer*/) {
     auto state = reinterpret_cast<CountState*>(state_);
     KU_ASSERT(input == nullptr);
     (void)input;
@@ -24,8 +24,10 @@ void CountStarFunction::updatePos(uint8_t* state_, ValueVector* input, uint64_t
 
 function_set CountStarFunction::getFunctionSet() {
     function_set result;
-    result.push_back(std::make_unique<AggregateFunction>(name, std::vector<LogicalTypeID>{},
-        LogicalTypeID::INT64, initialize, updateAll, updatePos, combine, finalize, false));
+    auto aggFunc = std::make_unique<AggregateFunction>(name, std::vector<LogicalTypeID>{},
+        LogicalTypeID::INT64, initialize, updateAll, updatePos, combine, finalize, false);
+    aggFunc->needToHandleNulls = true;
+    result.push_back(std::move(aggFunc));
     return result;
 }
 
