@@ -4,7 +4,9 @@ namespace kuzu {
 namespace function {
 
 void GDSComputeState::initSource(common::nodeID_t sourceNodeID) const {
-    frontierPair->initSource(sourceNodeID);
+    frontierPair->pinNextFrontier(sourceNodeID.tableID);
+    frontierPair->addNodeToNextFrontier(sourceNodeID);
+    frontierPair->setActiveNodesForNextIter();
     auxiliaryState->initSource(sourceNodeID);
 }
 
@@ -14,5 +16,11 @@ void GDSComputeState::beginFrontierCompute(common::table_id_t currTableID,
     auxiliaryState->beginFrontierCompute(currTableID, nextTableID);
 }
 
+void GDSComputeState::switchToDense(processor::ExecutionContext* context,
+    graph::Graph* graph) const {
+    frontierPair->switchToDense(context, graph);
+    auxiliaryState->switchToDense(context, graph);
+}
+
 } // namespace function
 } // namespace kuzu
