@@ -7,16 +7,17 @@
 #include "common/types/types.h"
 #include "function/table/bind_data.h"
 #include "function/table/bind_input.h"
-#include "function/table/table_function.h"
+#include "function/table/simple_table_function.h"
+#include "processor/execution_context.h"
 #include "storage/storage_manager.h"
-#include "storage/store/list_chunk_data.h"
-#include "storage/store/list_column.h"
-#include "storage/store/node_table.h"
-#include "storage/store/rel_table.h"
-#include "storage/store/string_chunk_data.h"
-#include "storage/store/string_column.h"
-#include "storage/store/struct_chunk_data.h"
-#include "storage/store/struct_column.h"
+#include "storage/table/list_chunk_data.h"
+#include "storage/table/list_column.h"
+#include "storage/table/node_table.h"
+#include "storage/table/rel_table.h"
+#include "storage/table/string_chunk_data.h"
+#include "storage/table/string_column.h"
+#include "storage/table/struct_chunk_data.h"
+#include "storage/table/struct_column.h"
 #include <concepts>
 
 using namespace kuzu::common;
@@ -36,14 +37,14 @@ struct StorageInfoLocalState final : TableFuncLocalState {
     }
 };
 
-static void collectColumns(Column* column, std::vector<Column*>& result) {
+static void collectColumns(const Column* column, std::vector<const Column*>& result) {
     result.push_back(column);
     if (column->getNullColumn()) {
         result.push_back(column->getNullColumn());
     }
     switch (column->getDataType().getPhysicalType()) {
     case PhysicalTypeID::STRUCT: {
-        const auto structColumn = ku_dynamic_cast<StructColumn*>(column);
+        const auto structColumn = ku_dynamic_cast<const StructColumn*>(column);
         const auto numChildren = StructType::getNumFields(structColumn->getDataType());
         for (auto i = 0u; i < numChildren; i++) {
             const auto childColumn = structColumn->getChild(i);
@@ -51,14 +52,15 @@ static void collectColumns(Column* column, std::vector<Column*>& result) {
         }
     } break;
     case PhysicalTypeID::STRING: {
-        const auto stringColumn = ku_dynamic_cast<StringColumn*>(column);
+        const auto stringColumn = ku_dynamic_cast<const StringColumn*>(column);
         auto& dictionary = stringColumn->getDictionary();
+        collectColumns(stringColumn->getIndexColumn(), result);
         collectColumns(dictionary.getDataColumn(), result);
         collectColumns(dictionary.getOffsetColumn(), result);
     } break;
     case PhysicalTypeID::ARRAY:
     case PhysicalTypeID::LIST: {
-        const auto listColumn = ku_dynamic_cast<ListColumn*>(column);
+        const auto listColumn = ku_dynamic_cast<const ListColumn*>(column);
         collectColumns(listColumn->getOffsetColumn(), result);
         collectColumns(listColumn->getSizeColumn(), result);
         collectColumns(listColumn->getDataColumn(), result);
@@ -71,22 +73,21 @@ static void collectColumns(Column* column, std::vector<Column*>& result) {
 
 struct StorageInfoBindData final : TableFuncBindData {
     TableCatalogEntry* tableEntry;
-    Table* table;
     const ClientContext* context;
 
     StorageInfoBindData(binder::expression_vector columns, TableCatalogEntry* tableEntry,
-        Table* table, const ClientContext* context)
+        const ClientContext* context)
         : TableFuncBindData{std::move(columns), 1 /*maxOffset*/}, tableEntry{tableEntry},
-          table{table}, context{context} {}
+          context{context} {}
 
     std::unique_ptr<TableFuncBindData> copy() const override {
-        return std::make_unique<StorageInfoBindData>(columns, tableEntry, table, context);
+        return std::make_unique<StorageInfoBindData>(columns, tableEntry, context);
     }
 };
 
-static std::unique_ptr<TableFuncLocalState> initLocalState(const TableFunctionInitInput& /*input*/,
-    const TableFuncSharedState* /*state*/, MemoryManager* mm) {
-    return std::make_unique<StorageInfoLocalState>(mm);
+static std::unique_ptr<TableFuncLocalState> initLocalState(
+    const TableFuncInitLocalStateInput& input) {
+    return std::make_unique<StorageInfoLocalState>(input.clientContext->getMemoryManager());
 }
 
 struct StorageInfoOutputData {
@@ -94,7 +95,7 @@ struct StorageInfoOutputData {
     node_group_idx_t chunkIdx = INVALID_NODE_GROUP_IDX;
     std::string tableType;
     uint32_t columnIdx = INVALID_COLUMN_ID;
-    std::vector<Column*> columns;
+    std::vector<const Column*> columns;
 };
 
 static void resetOutputIfNecessary(const StorageInfoLocalState* localState,
@@ -132,8 +133,8 @@ static void appendStorageInfoForChunkData(StorageInfoLocalState* localState, Dat
     outputChunk.getValueVectorMutable(4).setValue(vectorPos,
         outputData.columns[outputData.columnIdx++]->getName());
     outputChunk.getValueVectorMutable(5).setValue(vectorPos, columnType.toString());
-    outputChunk.getValueVectorMutable(6).setValue<uint64_t>(vectorPos, metadata.pageIdx);
-    outputChunk.getValueVectorMutable(7).setValue<uint64_t>(vectorPos, metadata.numPages);
+    outputChunk.getValueVectorMutable(6).setValue<uint64_t>(vectorPos, metadata.getStartPageIdx());
+    outputChunk.getValueVectorMutable(7).setValue<uint64_t>(vectorPos, metadata.getNumPages());
     outputChunk.getValueVectorMutable(8).setValue<uint64_t>(vectorPos, metadata.numValues);
 
     auto customToString = [&]<typename T>(T) {
@@ -185,6 +186,8 @@ static void appendStorageInfoForChunkData(StorageInfoLocalState* localState, Dat
     case PhysicalTypeID::STRING: {
         auto& stringChunk = chunkData.cast<StringChunkData>();
         auto& dictionaryChunk = stringChunk.getDictionaryChunk();
+        appendStorageInfoForChunkData(localState, outputChunk, outputData,
+            *stringChunk.getIndexColumnChunk());
         appendStorageInfoForChunkData(localState, outputChunk, outputData,
             *dictionaryChunk.getStringDataChunk());
         appendStorageInfoForChunkData(localState, outputChunk, outputData,
@@ -219,7 +222,7 @@ static void appendStorageInfoForChunkedGroup(StorageInfoLocalState* localState,
         auto& chunkedCSRGroup = chunkedGroup->cast<ChunkedCSRNodeGroup>();
         resetOutputIfNecessary(localState, outputChunk);
         appendStorageInfoForChunkData(localState, outputChunk, outputData,
-            chunkedCSRGroup.getCSRHeader().length->getData(), true);
+            chunkedCSRGroup.getCSRHeader().offset->getData(), true);
         resetOutputIfNecessary(localState, outputChunk);
         appendStorageInfoForChunkData(localState, outputChunk, outputData,
             chunkedCSRGroup.getCSRHeader().length->getData(), true);
@@ -249,6 +252,7 @@ static offset_t tableFunc(const TableFuncInput& input, TableFuncOutput& output)
     auto& dataChunk = output.dataChunk;
     auto localState = ku_dynamic_cast<StorageInfoLocalState*>(input.localState);
     KU_ASSERT(dataChunk.state->getSelVector().isUnfiltered());
+    auto storageManager = input.context->clientContext->getStorageManager();
     while (true) {
         if (localState->currChunkIdx < localState->dataChunkCollection->getNumChunks()) {
             // Copy from local state chunk.
@@ -266,19 +270,19 @@ static offset_t tableFunc(const TableFuncInput& input, TableFuncOutput& output)
             localState->currChunkIdx++;
             return numValuesToOutput;
         }
-        auto morsel = input.sharedState->ptrCast<TableFuncSharedState>()->getMorsel();
+        auto morsel = input.sharedState->ptrCast<SimpleTableFuncSharedState>()->getMorsel();
         if (!morsel.hasMoreToOutput()) {
             return 0;
         }
         const auto bindData = input.bindData->constPtrCast<StorageInfoBindData>();
-        const auto table = bindData->table;
         StorageInfoOutputData outputData;
-        outputData.tableType = table->getTableType() == TableType::NODE ? "NODE" : "REL";
         node_group_idx_t numNodeGroups = 0;
-        switch (table->getTableType()) {
+        switch (bindData->tableEntry->getTableType()) {
         case TableType::NODE: {
+            outputData.tableType = "NODE";
+            auto table = storageManager->getTable(bindData->tableEntry->getTableID());
             auto& nodeTable = table->cast<NodeTable>();
-            std::vector<Column*> columns;
+            std::vector<const Column*> columns;
             for (auto columnID = 0u; columnID < nodeTable.getNumColumns(); columnID++) {
                 collectColumns(&nodeTable.getColumn(columnID), columns);
             }
@@ -291,25 +295,29 @@ static offset_t tableFunc(const TableFuncInput& input, TableFuncOutput& output)
             }
         } break;
         case TableType::REL: {
-            auto& relTable = table->cast<RelTable>();
-            auto appendDirectedStorageInfo = [&](RelDataDirection direction) {
-                auto directedRelTableData = relTable.getDirectedTableData(direction);
-                std::vector<Column*> columns;
-                for (auto columnID = 0u; columnID < relTable.getNumColumns(); columnID++) {
-                    collectColumns(directedRelTableData->getColumn(columnID), columns);
+            outputData.tableType = "REL";
+            for (auto innerEntryInfo :
+                bindData->tableEntry->cast<RelGroupCatalogEntry>().getRelEntryInfos()) {
+                auto& relTable = storageManager->getTable(innerEntryInfo.oid)->cast<RelTable>();
+                auto appendDirectedStorageInfo = [&](RelDataDirection direction) {
+                    auto directedRelTableData = relTable.getDirectedTableData(direction);
+                    std::vector<const Column*> columns;
+                    for (auto columnID = 0u; columnID < relTable.getNumColumns(); columnID++) {
+                        collectColumns(directedRelTableData->getColumn(columnID), columns);
+                    }
+                    columns.push_back(directedRelTableData->getCSROffsetColumn());
+                    columns.push_back(directedRelTableData->getCSRLengthColumn());
+                    outputData.columns = std::move(columns);
+                    numNodeGroups = directedRelTableData->getNumNodeGroups();
+                    for (auto i = 0ul; i < numNodeGroups; i++) {
+                        outputData.nodeGroupIdx = i;
+                        appendStorageInfoForNodeGroup(localState, dataChunk, outputData,
+                            directedRelTableData->getNodeGroup(i));
+                    }
+                };
+                for (auto direction : relTable.getStorageDirections()) {
+                    appendDirectedStorageInfo(direction);
                 }
-                columns.push_back(directedRelTableData->getCSROffsetColumn());
-                columns.push_back(directedRelTableData->getCSRLengthColumn());
-                outputData.columns = std::move(columns);
-                numNodeGroups = directedRelTableData->getNumNodeGroups();
-                for (auto i = 0ul; i < numNodeGroups; i++) {
-                    outputData.nodeGroupIdx = i;
-                    appendStorageInfoForNodeGroup(localState, dataChunk, outputData,
-                        directedRelTableData->getNodeGroup(i));
-                }
-            };
-            for (auto direction : relTable.getStorageDirections()) {
-                appendDirectedStorageInfo(direction);
             }
         } break;
         default: {
@@ -346,11 +354,9 @@ static std::unique_ptr<TableFuncBindData> bindFunc(const ClientContext* context,
         throw BinderException{"Table " + tableName + " does not exist!"};
     }
     auto tableEntry = catalog->getTableCatalogEntry(context->getTransaction(), tableName);
-    auto storageManager = context->getStorageManager();
-    auto table = storageManager->getTable(tableEntry->getTableID());
     columnNames = TableFunction::extractYieldVariables(columnNames, input->yieldVariables);
     auto columns = input->binder->createVariables(columnNames, columnTypes);
-    return std::make_unique<StorageInfoBindData>(columns, tableEntry, table, context);
+    return std::make_unique<StorageInfoBindData>(columns, tableEntry, context);
 }
 
 function_set StorageInfoFunction::getFunctionSet() {
@@ -358,7 +364,7 @@ function_set StorageInfoFunction::getFunctionSet() {
     auto function = std::make_unique<TableFunction>(name, std::vector{LogicalTypeID::STRING});
     function->tableFunc = tableFunc;
     function->bindFunc = bindFunc;
-    function->initSharedStateFunc = TableFunction::initSharedState;
+    function->initSharedStateFunc = SimpleTableFunc::initSharedState;
     function->initLocalStateFunc = initLocalState;
     functionSet.push_back(std::move(function));
     return functionSet;
