@@ -6,6 +6,7 @@
 #include "common/exception/binder.h"
 #include "common/exception/conversion.h"
 #include "function/built_in_function_utils.h"
+#include "function/cast/cast_union_bind_data.h"
 #include "function/cast/functions/cast_array.h"
 #include "function/cast/functions/cast_decimal.h"
 #include "function/cast/functions/cast_from_string_functions.h"
@@ -20,8 +21,8 @@ namespace function {
 
 struct CastChildFunctionExecutor {
     template<typename OPERAND_TYPE, typename RESULT_TYPE, typename FUNC, typename OP_WRAPPER>
-    static void executeSwitch(common::ValueVector& operand, common::ValueVector& result,
-        void* dataPtr) {
+    static void executeSwitch(common::ValueVector& operand, common::SelectionVector*,
+        common::ValueVector& result, common::SelectionVector*, void* dataPtr) {
         auto numOfEntries = reinterpret_cast<CastFunctionBindData*>(dataPtr)->numOfEntries;
         for (auto i = 0u; i < numOfEntries; i++) {
             result.setNull(i, operand.isNull(i));
@@ -91,6 +92,11 @@ static void resolveNestedVector(std::shared_ptr<ValueVector> inputVector, ValueV
         }
     }
 
+    // TODO(Sami): Remove this restriction after we support casting between nested unions.
+    if (inputType->getLogicalTypeID() == LogicalTypeID::UNION ||
+        resultType->getLogicalTypeID() == LogicalTypeID::UNION) {
+        throw common::BinderException{"Casting from/to nested union type is not supported yet."};
+    }
     // non-nested types
     if (inputType->getLogicalTypeID() != resultType->getLogicalTypeID()) {
         auto func = CastFunction::bindCastFunction<CastChildFunctionExecutor>("CAST", *inputType,
@@ -98,7 +104,8 @@ static void resolveNestedVector(std::shared_ptr<ValueVector> inputVector, ValueV
                         ->execFunc;
         std::vector<std::shared_ptr<ValueVector>> childParams{inputVector};
         dataPtr->numOfEntries = numOfEntries;
-        func(childParams, *resultVector, (void*)dataPtr);
+        func(childParams, SelectionVector::fromValueVectors(childParams), *resultVector,
+            resultVector->getSelVectorPtr(), (void*)dataPtr);
     } else {
         for (auto i = 0u; i < numOfEntries; i++) {
             resultVector->copyFromVectorData(i, inputVector.get(), i);
@@ -106,27 +113,30 @@ static void resolveNestedVector(std::shared_ptr<ValueVector> inputVector, ValueV
     }
 }
 
-static void nestedTypesCastExecFunction(const std::vector<std::shared_ptr<ValueVector>>& params,
-    ValueVector& result, void*) {
+static void nestedTypesCastExecFunction(
+    const std::vector<std::shared_ptr<common::ValueVector>>& params,
+    const std::vector<common::SelectionVector*>& paramSelVectors, common::ValueVector& result,
+    common::SelectionVector* resultSelVector, void*) {
     KU_ASSERT(params.size() == 1);
     result.resetAuxiliaryBuffer();
     const auto& inputVector = params[0];
+    const auto* inputVectorSelVector = paramSelVectors[0];
 
-    // check if all selcted list entry have the requried fixed list size
+    // check if all selected list entries have the required fixed list size
     if (CastArrayHelper::containsListToArray(inputVector->dataType, result.dataType)) {
-        for (auto i = 0u; i < inputVector->state->getSelVector().getSelSize(); i++) {
-            auto pos = inputVector->state->getSelVector()[i];
+        for (auto i = 0u; i < inputVectorSelVector->getSelSize(); i++) {
+            auto pos = (*inputVectorSelVector)[i];
             CastArrayHelper::validateListEntry(inputVector.get(), result.dataType, pos);
         }
     };
 
-    auto& selVector = inputVector->state->getSelVector();
+    auto& selVector = *inputVectorSelVector;
     auto bindData = CastFunctionBindData(result.dataType.copy());
     auto numOfEntries = selVector[selVector.getSelSize() - 1] + 1;
     resolveNestedVector(inputVector, &result, numOfEntries, &bindData);
     if (inputVector->state->isFlat()) {
-        result.state->getSelVectorUnsafe().setToFiltered();
-        result.state->getSelVectorUnsafe()[0] = inputVector->state->getSelVector()[0];
+        resultSelVector->setToFiltered();
+        (*resultSelVector)[0] = (*inputVectorSelVector)[0];
     }
 }
 
@@ -170,10 +180,21 @@ static bool hasImplicitCastStruct(const LogicalType& srcType, const LogicalType&
     return true;
 }
 
-static bool hasImplicitCastUnion(const LogicalType& /*srcType*/, const LogicalType& /*dstType*/) {
-    // todo: implement union casting function
-    // currently, there seems to be no casting functionality between union types
-    return false;
+static bool hasImplicitCastUnion(const LogicalType& srcType, const LogicalType& dstType) {
+    // srcType is either non-nested or a union
+    if (srcType.getLogicalTypeID() == LogicalTypeID::UNION) {
+        // todo
+        throw ConversionException{"Casting from UNION to UNION is not supported right now."};
+    } else {
+        auto numFields = UnionType::getNumFields(dstType);
+        for (auto i = 0u; i < numFields; ++i) {
+            const auto& fieldType = UnionType::getFieldType(dstType, i);
+            if (CastFunction::hasImplicitCast(srcType, fieldType)) {
+                return true;
+            }
+        }
+        return false;
+    }
 }
 
 static bool hasImplicitCastMap(const LogicalType& srcType, const LogicalType& dstType) {
@@ -214,6 +235,8 @@ bool CastFunction::hasImplicitCast(const LogicalType& srcType, const LogicalType
             KU_UNREACHABLE;
             // LCOV_EXCL_END
         }
+    } else if (dstType.getLogicalTypeID() == LogicalTypeID::UNION) {
+        return hasImplicitCastUnion(srcType, dstType);
     }
     if (BuiltInFunctionsUtils::getCastCost(srcType.getLogicalTypeID(),
             dstType.getLogicalTypeID()) != UNDEFINED_CAST_COST) {
@@ -500,70 +523,25 @@ static std::unique_ptr<ScalarFunction> bindCastToStringFunction(const std::strin
         std::vector<LogicalTypeID>{sourceType.getLogicalTypeID()}, LogicalTypeID::STRING, func);
 }
 
-template<typename DST_TYPE>
+template<typename DST_TYPE, CastExecutor EXECUTOR>
 static std::unique_ptr<ScalarFunction> bindCastToDecimalFunction(const std::string& functionName,
     const LogicalType& sourceType, const LogicalType& targetType) {
     scalar_func_exec_t func;
-    switch (sourceType.getLogicalTypeID()) {
-    case LogicalTypeID::INT8: {
-        func = ScalarFunction::UnaryExecNestedTypeFunction<int8_t, DST_TYPE, CastToDecimal>;
-    } break;
-    case LogicalTypeID::INT16: {
-        func = ScalarFunction::UnaryExecNestedTypeFunction<int16_t, DST_TYPE, CastToDecimal>;
-    } break;
-    case LogicalTypeID::INT32: {
-        func = ScalarFunction::UnaryExecNestedTypeFunction<int32_t, DST_TYPE, CastToDecimal>;
-    } break;
-    case LogicalTypeID::SERIAL:
-    case LogicalTypeID::INT64: {
-        func = ScalarFunction::UnaryExecNestedTypeFunction<int64_t, DST_TYPE, CastToDecimal>;
-    } break;
-    case LogicalTypeID::UINT8: {
-        func = ScalarFunction::UnaryExecNestedTypeFunction<uint8_t, DST_TYPE, CastToDecimal>;
-    } break;
-    case LogicalTypeID::UINT16: {
-        func = ScalarFunction::UnaryExecNestedTypeFunction<uint16_t, DST_TYPE, CastToDecimal>;
-    } break;
-    case LogicalTypeID::UINT32: {
-        func = ScalarFunction::UnaryExecNestedTypeFunction<uint32_t, DST_TYPE, CastToDecimal>;
-    } break;
-    case LogicalTypeID::UINT64: {
-        func = ScalarFunction::UnaryExecNestedTypeFunction<uint64_t, DST_TYPE, CastToDecimal>;
-    } break;
-    case LogicalTypeID::INT128: {
-        func = ScalarFunction::UnaryExecNestedTypeFunction<int128_t, DST_TYPE, CastToDecimal>;
-    } break;
-    case LogicalTypeID::FLOAT: {
-        func = ScalarFunction::UnaryExecNestedTypeFunction<float, DST_TYPE, CastToDecimal>;
-    } break;
-    case LogicalTypeID::DOUBLE: {
-        func = ScalarFunction::UnaryExecNestedTypeFunction<double, DST_TYPE, CastToDecimal>;
-    } break;
-    case LogicalTypeID::DECIMAL: {
-        switch (sourceType.getPhysicalType()) {
-        case PhysicalTypeID::INT16:
-            func =
-                ScalarFunction::UnaryExecNestedTypeFunction<int16_t, DST_TYPE, CastBetweenDecimal>;
-            break;
-        case PhysicalTypeID::INT32:
-            func =
-                ScalarFunction::UnaryExecNestedTypeFunction<int32_t, DST_TYPE, CastBetweenDecimal>;
-            break;
-        case PhysicalTypeID::INT64:
-            func =
-                ScalarFunction::UnaryExecNestedTypeFunction<int64_t, DST_TYPE, CastBetweenDecimal>;
-            break;
-        case PhysicalTypeID::INT128:
-            func =
-                ScalarFunction::UnaryExecNestedTypeFunction<int128_t, DST_TYPE, CastBetweenDecimal>;
-            break;
-        default:
-            KU_UNREACHABLE;
-        }
-    } break;
-    default:
-        throw ConversionException{stringFormat("Unsupported casting function from {} to {}.",
-            sourceType.toString(), targetType.toString())};
+    if (sourceType.getLogicalTypeID() == LogicalTypeID::DECIMAL) {
+        TypeUtils::visit(
+            sourceType,
+            [&]<SignedIntegerTypes T>(T) {
+                func = ScalarFunction::UnaryCastExecFunction<T, DST_TYPE, CastBetweenDecimal,
+                    EXECUTOR>;
+            },
+            [&](auto) { KU_UNREACHABLE; });
+    } else {
+        TypeUtils::visit(
+            sourceType,
+            [&]<NumericTypes T>(T) {
+                func = ScalarFunction::UnaryCastExecFunction<T, DST_TYPE, CastToDecimal, EXECUTOR>;
+            },
+            [&](auto) { KU_UNREACHABLE; });
     }
     return std::make_unique<ScalarFunction>(functionName,
         std::vector<LogicalTypeID>{sourceType.getLogicalTypeID()}, targetType.getLogicalTypeID(),
@@ -613,16 +591,20 @@ static std::unique_ptr<ScalarFunction> bindCastToNumericFunction(const std::stri
         switch (sourceType.getPhysicalType()) {
         // note: this cannot handle decimal -> decimal casting.
         case PhysicalTypeID::INT16:
-            func = ScalarFunction::UnaryExecNestedTypeFunction<int16_t, DST_TYPE, CastDecimalTo>;
+            func = ScalarFunction::UnaryExecNestedTypeFunction<int16_t, DST_TYPE, CastDecimalTo,
+                EXECUTOR>;
             break;
         case PhysicalTypeID::INT32:
-            func = ScalarFunction::UnaryExecNestedTypeFunction<int32_t, DST_TYPE, CastDecimalTo>;
+            func = ScalarFunction::UnaryExecNestedTypeFunction<int32_t, DST_TYPE, CastDecimalTo,
+                EXECUTOR>;
             break;
         case PhysicalTypeID::INT64:
-            func = ScalarFunction::UnaryExecNestedTypeFunction<int64_t, DST_TYPE, CastDecimalTo>;
+            func = ScalarFunction::UnaryExecNestedTypeFunction<int64_t, DST_TYPE, CastDecimalTo,
+                EXECUTOR>;
             break;
         case PhysicalTypeID::INT128:
-            func = ScalarFunction::UnaryExecNestedTypeFunction<int128_t, DST_TYPE, CastDecimalTo>;
+            func = ScalarFunction::UnaryExecNestedTypeFunction<int128_t, DST_TYPE, CastDecimalTo,
+                EXECUTOR>;
             break;
         default:
             KU_UNREACHABLE;
@@ -637,29 +619,84 @@ static std::unique_ptr<ScalarFunction> bindCastToNumericFunction(const std::stri
         func);
 }
 
-template<typename EXECUTOR = UnaryFunctionExecutor>
+static std::unique_ptr<ScalarFunction> bindCastToUnionFunction(const std::string& functionName,
+    const LogicalType& sourceType, const LogicalType& targetType) {
+    // source type is not nested, targetType is a union
+    uint32_t minCastCost = UNDEFINED_CAST_COST;
+    union_field_idx_t minCostTag = 0;
+    auto numFields = UnionType::getNumFields(targetType);
+    for (auto i = 0u; i < numFields; ++i) {
+        const auto& fieldType = UnionType::getFieldType(targetType, i);
+        if (CastFunction::hasImplicitCast(sourceType, fieldType)) {
+            uint32_t castCost = BuiltInFunctionsUtils::getCastCost(sourceType.getLogicalTypeID(),
+                fieldType.getLogicalTypeID());
+            if (castCost < minCastCost) {
+                minCastCost = castCost;
+                minCostTag = i;
+            }
+        }
+    }
+    if (minCastCost == UNDEFINED_CAST_COST) {
+        throw ConversionException{stringFormat("Unsupported casting function from {} to {}.",
+            sourceType.toString(), targetType.toString())};
+    }
+    const auto& innerType = common::UnionType::getFieldType(targetType, minCostTag);
+    scalar_func_exec_t execFunc;
+    TypeUtils::visit(sourceType, [&execFunc]<typename T>(T) {
+        execFunc = ScalarFunction::UnaryCastUnionExecFunction<T, union_entry_t, CastToUnion>;
+    });
+    std::shared_ptr<ScalarFunction> innerCast;
+    CastToUnionBindData::inner_func_t innerFunc;
+    if (sourceType != innerType) {
+        innerCast = CastFunction::bindCastFunction<CastChildFunctionExecutor>("CAST", sourceType,
+            innerType.copy());
+        innerFunc = [innerCast](common::ValueVector& inputVector, common::ValueVector& valVector,
+                        common::SelectionVector& selVector, CastToUnionBindData& bindData) {
+            std::vector<std::shared_ptr<common::ValueVector>> innerParams{
+                std::shared_ptr<common::ValueVector>(&inputVector, [](common::ValueVector*) {})};
+            bindData.innerBindData.numOfEntries = selVector[selVector.getSelSize() - 1] + 1;
+            innerCast->execFunc(innerParams, common::SelectionVector::fromValueVectors(innerParams),
+                valVector, valVector.getSelVectorPtr(), &bindData.innerBindData);
+        };
+    } else {
+        innerFunc = [](common::ValueVector& inputVector, common::ValueVector& valVector,
+                        common::SelectionVector& selVector, CastToUnionBindData&) {
+            for (auto& pos : selVector.getSelectedPositions()) {
+                valVector.copyFromVectorData(pos, &inputVector, pos);
+            }
+        };
+    }
+    auto castFunc = std::make_unique<ScalarFunction>(functionName,
+        std::vector<LogicalTypeID>{sourceType.getLogicalTypeID()}, targetType.getLogicalTypeID(),
+        execFunc);
+    auto pInnerType = std::make_shared<LogicalType>(innerType.copy());
+    auto pTargetType = std::make_shared<LogicalType>(targetType.copy());
+    scalar_bind_func bindFunc = [minCostTag, innerFunc, pInnerType, pTargetType](
+                                    const ScalarBindFuncInput&) {
+        return std::make_unique<CastToUnionBindData>(minCostTag, innerFunc, pInnerType->copy(),
+            pTargetType->copy());
+    };
+    castFunc->bindFunc = std::move(bindFunc);
+    return castFunc;
+}
+
 static std::unique_ptr<ScalarFunction> bindCastBetweenNested(const std::string& functionName,
     const LogicalType& sourceType, const LogicalType& targetType) {
-    switch (sourceType.getLogicalTypeID()) {
-    case LogicalTypeID::LIST:
-    case LogicalTypeID::MAP:
-    case LogicalTypeID::STRUCT:
-    case LogicalTypeID::ANY:
-    case LogicalTypeID::ARRAY: {
-        // todo: compile time checking of nested types
-        if (CastArrayHelper::checkCompatibleNestedTypes(sourceType.getLogicalTypeID(),
-                targetType.getLogicalTypeID())) {
-            return std::make_unique<ScalarFunction>(functionName,
-                std::vector<LogicalTypeID>{sourceType.getLogicalTypeID()},
-                targetType.getLogicalTypeID(), nestedTypesCastExecFunction);
-        }
-        [[fallthrough]];
+    // todo: compile time checking of nested types
+    if (CastArrayHelper::checkCompatibleNestedTypes(sourceType.getLogicalTypeID(),
+            targetType.getLogicalTypeID())) {
+        return std::make_unique<ScalarFunction>(functionName,
+            std::vector<LogicalTypeID>{sourceType.getLogicalTypeID()},
+            targetType.getLogicalTypeID(), nestedTypesCastExecFunction);
     }
-    default:
-        throw ConversionException{stringFormat("Unsupported casting function from {} to {}.",
-            LogicalTypeUtils::toString(sourceType.getLogicalTypeID()),
-            LogicalTypeUtils::toString(targetType.getLogicalTypeID()))};
+    if (targetType.getLogicalTypeID() == LogicalTypeID::UNION) {
+        throw ConversionException{
+            stringFormat("Casting from {} to UNION is not supported right now.",
+                LogicalTypeUtils::toString(sourceType.getLogicalTypeID()))};
     }
+    throw ConversionException{stringFormat("Unsupported casting function from {} to {}.",
+        LogicalTypeUtils::toString(sourceType.getLogicalTypeID()),
+        LogicalTypeUtils::toString(targetType.getLogicalTypeID()))};
 }
 
 template<typename EXECUTOR = UnaryFunctionExecutor, typename DST_TYPE>
@@ -747,7 +784,7 @@ static std::unique_ptr<ScalarFunction> bindCastBetweenDecimalFunction(
         std::vector<LogicalTypeID>{LogicalTypeID::DECIMAL}, LogicalTypeID::DECIMAL, func);
 }
 
-template<typename EXECUTOR>
+template<CastExecutor EXECUTOR>
 std::unique_ptr<ScalarFunction> CastFunction::bindCastFunction(const std::string& functionName,
     const LogicalType& sourceType, const LogicalType& targetType) {
     auto sourceTypeID = sourceType.getLogicalTypeID();
@@ -768,18 +805,15 @@ std::unique_ptr<ScalarFunction> CastFunction::bindCastFunction(const std::string
             targetType);
     }
     case LogicalTypeID::DECIMAL: {
-        switch (targetType.getPhysicalType()) {
-        case PhysicalTypeID::INT16:
-            return bindCastToDecimalFunction<int16_t>(functionName, sourceType, targetType);
-        case PhysicalTypeID::INT32:
-            return bindCastToDecimalFunction<int32_t>(functionName, sourceType, targetType);
-        case PhysicalTypeID::INT64:
-            return bindCastToDecimalFunction<int64_t>(functionName, sourceType, targetType);
-        case PhysicalTypeID::INT128:
-            return bindCastToDecimalFunction<int128_t>(functionName, sourceType, targetType);
-        default:
-            KU_UNREACHABLE;
-        }
+        std::unique_ptr<ScalarFunction> scalarFunc;
+        TypeUtils::visit(
+            targetType.getPhysicalType(),
+            [&]<IntegerTypes T>(T) {
+                scalarFunc =
+                    bindCastToDecimalFunction<T, EXECUTOR>(functionName, sourceType, targetType);
+            },
+            [](auto) { KU_UNREACHABLE; });
+        return scalarFunc;
     }
     case LogicalTypeID::INT128: {
         return bindCastToNumericFunction<int128_t, CastToInt128, EXECUTOR>(functionName, sourceType,
@@ -841,12 +875,17 @@ std::unique_ptr<ScalarFunction> CastFunction::bindCastFunction(const std::string
         return bindCastToTimestampFunction<EXECUTOR, timestamp_t>(functionName, sourceType,
             targetType);
     }
+    case LogicalTypeID::UNION: {
+        if (!LogicalTypeUtils::isNested(sourceType.getLogicalTypeID())) {
+            return bindCastToUnionFunction(functionName, sourceType, targetType);
+        }
+        [[fallthrough]];
+    }
     case LogicalTypeID::LIST:
     case LogicalTypeID::ARRAY:
     case LogicalTypeID::MAP:
-    case LogicalTypeID::STRUCT:
-    case LogicalTypeID::UNION: {
-        return bindCastBetweenNested<EXECUTOR>(functionName, sourceType, targetType);
+    case LogicalTypeID::STRUCT: {
+        return bindCastBetweenNested(functionName, sourceType, targetType);
     }
     default: {
         throw ConversionException(stringFormat("Unsupported casting function from {} to {}.",
@@ -1083,9 +1122,12 @@ static std::unique_ptr<FunctionBindData> castBindFunc(ScalarBindFuncInput input)
         return nullptr;
     }
     // TODO(Xiyang): Can we unify the binding of casting function with other scalar functions?
-    func->execFunc =
-        CastFunction::bindCastFunction(func->name, input.arguments[0]->getDataType(), targetType)
-            ->execFunc;
+    auto res =
+        CastFunction::bindCastFunction(func->name, input.arguments[0]->getDataType(), targetType);
+    func->execFunc = res->execFunc;
+    if (res->bindFunc) {
+        return res->bindFunc(input);
+    }
     return std::make_unique<function::CastFunctionBindData>(targetType.copy());
 }
 
