@@ -10,12 +10,7 @@ using namespace kuzu::catalog;
 namespace kuzu {
 namespace graph {
 
-void GraphEntryTableInfo::setPredicate(std::shared_ptr<Expression> predicate_) {
-    KU_ASSERT(predicate == nullptr);
-    predicate = std::move(predicate_);
-}
-
-GraphEntry::GraphEntry(std::vector<TableCatalogEntry*> nodeEntries,
+NativeGraphEntry::NativeGraphEntry(std::vector<TableCatalogEntry*> nodeEntries,
     std::vector<TableCatalogEntry*> relEntries) {
     for (auto& entry : nodeEntries) {
         nodeInfos.emplace_back(entry);
@@ -25,7 +20,7 @@ GraphEntry::GraphEntry(std::vector<TableCatalogEntry*> nodeEntries,
     }
 }
 
-std::vector<table_id_t> GraphEntry::getNodeTableIDs() const {
+std::vector<table_id_t> NativeGraphEntry::getNodeTableIDs() const {
     std::vector<table_id_t> result;
     for (auto& info : nodeInfos) {
         result.push_back(info.entry->getTableID());
@@ -33,15 +28,7 @@ std::vector<table_id_t> GraphEntry::getNodeTableIDs() const {
     return result;
 }
 
-std::vector<table_id_t> GraphEntry::getRelTableIDs() const {
-    std::vector<table_id_t> result;
-    for (auto& info : relInfos) {
-        result.push_back(info.entry->getTableID());
-    }
-    return result;
-}
-
-std::vector<TableCatalogEntry*> GraphEntry::getNodeEntries() const {
+std::vector<TableCatalogEntry*> NativeGraphEntry::getNodeEntries() const {
     std::vector<TableCatalogEntry*> result;
     for (auto& info : nodeInfos) {
         result.push_back(info.entry);
@@ -49,15 +36,7 @@ std::vector<TableCatalogEntry*> GraphEntry::getNodeEntries() const {
     return result;
 }
 
-std::vector<TableCatalogEntry*> GraphEntry::getRelEntries() const {
-    std::vector<TableCatalogEntry*> result;
-    for (auto& info : relInfos) {
-        result.push_back(info.entry);
-    }
-    return result;
-}
-
-const GraphEntryTableInfo& GraphEntry::getRelInfo(table_id_t tableID) const {
+const NativeGraphEntryTableInfo& NativeGraphEntry::getRelInfo(table_id_t tableID) const {
     for (auto& info : relInfos) {
         if (info.entry->getTableID() == tableID) {
             return info;
@@ -68,11 +47,5 @@ const GraphEntryTableInfo& GraphEntry::getRelInfo(table_id_t tableID) const {
     // LCOV_EXCL_STOP
 }
 
-void GraphEntry::setRelPredicate(std::shared_ptr<Expression> predicate) {
-    for (auto& info : relInfos) {
-        info.setPredicate(predicate);
-    }
-}
-
 } // namespace graph
 } // namespace kuzu
