@@ -7,16 +7,18 @@ namespace kuzu {
 namespace binder {
 
 class BoundCreateMacro final : public BoundStatement {
+    static constexpr common::StatementType type_ = common::StatementType::CREATE_MACRO;
+
 public:
     explicit BoundCreateMacro(std::string macroName,
         std::unique_ptr<function::ScalarMacroFunction> macro)
-        : BoundStatement{common::StatementType::CREATE_MACRO,
+        : BoundStatement{type_,
               BoundStatementResult::createSingleStringColumnResult("result" /* columnName */)},
           macroName{std::move(macroName)}, macro{std::move(macro)} {}
 
-    inline std::string getMacroName() const { return macroName; }
+    std::string getMacroName() const { return macroName; }
 
-    inline std::unique_ptr<function::ScalarMacroFunction> getMacro() const { return macro->copy(); }
+    std::unique_ptr<function::ScalarMacroFunction> getMacro() const { return macro->copy(); }
 
 private:
     std::string macroName;
