@@ -8,11 +8,12 @@ namespace kuzu {
 namespace binder {
 
 class BoundStandaloneCall final : public BoundStatement {
+    static constexpr common::StatementType type_ = common::StatementType::STANDALONE_CALL;
+
 public:
     BoundStandaloneCall(const main::Option* option, std::shared_ptr<Expression> optionValue)
-        : BoundStatement{common::StatementType::STANDALONE_CALL,
-              BoundStatementResult::createEmptyResult()},
-          option{option}, optionValue{std::move(optionValue)} {}
+        : BoundStatement{type_, BoundStatementResult::createEmptyResult()}, option{option},
+          optionValue{std::move(optionValue)} {}
 
     const main::Option* getOption() const { return option; }
 
