@@ -7,10 +7,12 @@ namespace kuzu {
 namespace binder {
 
 class BoundCopyTo final : public BoundStatement {
+    static constexpr common::StatementType type_ = common::StatementType::COPY_TO;
+
 public:
     BoundCopyTo(std::unique_ptr<function::ExportFuncBindData> bindData,
         function::ExportFunction exportFunc, std::unique_ptr<BoundStatement> query)
-        : BoundStatement{common::StatementType::COPY_TO, BoundStatementResult::createEmptyResult()},
+        : BoundStatement{type_, BoundStatementResult::createEmptyResult()},
           bindData{std::move(bindData)}, exportFunc{std::move(exportFunc)},
           query{std::move(query)} {}
 
