@@ -6,13 +6,14 @@ namespace kuzu {
 namespace binder {
 
 class BoundCreateSequence final : public BoundStatement {
+    static constexpr common::StatementType type_ = common::StatementType::CREATE_SEQUENCE;
+
 public:
     explicit BoundCreateSequence(BoundCreateSequenceInfo info)
-        : BoundStatement{common::StatementType::CREATE_SEQUENCE,
-              BoundStatementResult::createSingleStringColumnResult()},
+        : BoundStatement{type_, BoundStatementResult::createSingleStringColumnResult()},
           info{std::move(info)} {}
 
-    inline const BoundCreateSequenceInfo* getInfo() const { return &info; }
+    const BoundCreateSequenceInfo& getInfo() const { return info; }
 
 private:
     BoundCreateSequenceInfo info;
