@@ -7,10 +7,11 @@ namespace kuzu {
 namespace binder {
 
 class KUZU_API LiteralExpression final : public Expression {
+    static constexpr common::ExpressionType type_ = common::ExpressionType::LITERAL;
+
 public:
     LiteralExpression(common::Value value, const std::string& uniqueName)
-        : Expression{common::ExpressionType::LITERAL, value.getDataType().copy(), uniqueName},
-          value{std::move(value)} {}
+        : Expression{type_, value.getDataType().copy(), uniqueName}, value{std::move(value)} {}
 
     bool isNull() const { return value.isNull(); }
 
@@ -20,10 +21,6 @@ public:
 
     std::string toStringInternal() const override { return value.toString(); }
 
-    std::unique_ptr<Expression> copy() const override {
-        return std::make_unique<LiteralExpression>(value, uniqueName);
-    }
-
 public:
     common::Value value;
 };
