@@ -12,19 +12,21 @@ public:
         : NodeOrRelExpression{std::move(dataType), std::move(uniqueName), std::move(variableName),
               std::move(entries)} {}
 
-    void setInternalID(std::unique_ptr<Expression> expression) {
-        internalID = std::move(expression);
-    }
-    std::shared_ptr<Expression> getInternalID() const {
+    ~NodeExpression() override;
+
+    bool isMultiLabeled() const override { return entries.size() > 1; }
+
+    void setInternalID(std::shared_ptr<PropertyExpression> expr) { internalID = std::move(expr); }
+    std::shared_ptr<PropertyExpression> getInternalID() const override {
         KU_ASSERT(internalID != nullptr);
-        return internalID->copy();
+        return internalID;
     }
 
-    // Get primary key property expression for a given table ID.
+    // Get the primary key property expression for a given table ID.
     std::shared_ptr<Expression> getPrimaryKey(common::table_id_t tableID) const;
 
 private:
-    std::unique_ptr<Expression> internalID;
+    std::shared_ptr<PropertyExpression> internalID;
 };
 
 } // namespace binder
