@@ -14,11 +14,11 @@ public:
           variableName{std::move(variableName)}, nodeType{std::move(nodeType)},
           relType{std::move(relType)} {}
 
-    inline std::string getVariableName() const { return variableName; }
-    inline const common::LogicalType& getNodeType() const { return nodeType; }
-    inline const common::LogicalType& getRelType() const { return relType; }
+    std::string getVariableName() const { return variableName; }
+    const common::LogicalType& getNodeType() const { return nodeType; }
+    const common::LogicalType& getRelType() const { return relType; }
 
-    inline std::string toStringInternal() const override { return variableName; }
+    std::string toStringInternal() const override { return variableName; }
 
 private:
     std::string variableName;
