@@ -7,16 +7,16 @@ namespace kuzu {
 namespace binder {
 
 class BoundRegularQuery final : public BoundStatement {
+    static constexpr common::StatementType type_ = common::StatementType::QUERY;
 
 public:
     explicit BoundRegularQuery(std::vector<bool> isUnionAll, BoundStatementResult statementResult)
-        : BoundStatement{common::StatementType::QUERY, std::move(statementResult)},
-          isUnionAll{std::move(isUnionAll)} {}
+        : BoundStatement{type_, std::move(statementResult)}, isUnionAll{std::move(isUnionAll)} {}
 
     void addSingleQuery(NormalizedSingleQuery singleQuery) {
         singleQueries.push_back(std::move(singleQuery));
     }
-    uint64_t getNumSingleQueries() const { return singleQueries.size(); }
+    common::idx_t getNumSingleQueries() const { return singleQueries.size(); }
     NormalizedSingleQuery* getSingleQueryUnsafe(common::idx_t idx) { return &singleQueries[idx]; }
     const NormalizedSingleQuery* getSingleQuery(common::idx_t idx) const {
         return &singleQueries[idx];
