@@ -6,7 +6,7 @@
 namespace kuzu {
 namespace binder {
 
-class BoundReadingClause {
+class KUZU_API BoundReadingClause {
 public:
     explicit BoundReadingClause(common::ClauseType clauseType) : clauseType{clauseType} {}
     DELETE_COPY_DEFAULT_MOVE(BoundReadingClause);
@@ -30,6 +30,10 @@ public:
         return common::ku_dynamic_cast<const TARGET&>(*this);
     }
     template<class TARGET>
+    TARGET* ptrCast() const {
+        return common::ku_dynamic_cast<TARGET*>(this);
+    }
+    template<class TARGET>
     const TARGET* constPtrCast() const {
         return common::ku_dynamic_cast<const TARGET*>(this);
     }
