@@ -10,11 +10,11 @@ public:
     explicit BoundWithClause(BoundProjectionBody projectionBody)
         : BoundReturnClause{std::move(projectionBody)} {}
 
-    inline void setWhereExpression(std::shared_ptr<Expression> expression) {
+    void setWhereExpression(std::shared_ptr<Expression> expression) {
         whereExpression = std::move(expression);
     }
-    inline bool hasWhereExpression() const { return whereExpression != nullptr; }
-    inline std::shared_ptr<Expression> getWhereExpression() const { return whereExpression; }
+    bool hasWhereExpression() const { return whereExpression != nullptr; }
+    std::shared_ptr<Expression> getWhereExpression() const { return whereExpression; }
 
 private:
     std::shared_ptr<Expression> whereExpression;
