@@ -9,7 +9,6 @@ namespace catalog {
 enum class CatalogEntryType : uint8_t {
     // Table entries
     NODE_TABLE_ENTRY = 0,
-    REL_TABLE_ENTRY = 1,
     REL_GROUP_ENTRY = 2,
     FOREIGN_TABLE_ENTRY = 4,
     // Macro entries
@@ -19,7 +18,6 @@ enum class CatalogEntryType : uint8_t {
     SCALAR_FUNCTION_ENTRY = 21,
     REWRITE_FUNCTION_ENTRY = 22,
     TABLE_FUNCTION_ENTRY = 23,
-    GDS_FUNCTION_ENTRY = 24,
     COPY_FUNCTION_ENTRY = 25,
     STANDALONE_TABLE_FUNCTION_ENTRY = 26,
     // Sequence entries
