@@ -11,6 +11,8 @@ enum class ScanSourceType : uint8_t {
     FILE = 1,
     OBJECT = 2,
     QUERY = 3,
+    TABLE_FUNC = 4,
+    PARAM = 5,
 };
 
 class ScanSourceTypeUtils {
