@@ -3,10 +3,11 @@
 #include <cstdint>
 #include <string>
 
+#include "common/api.h"
+
 namespace kuzu {
 namespace common {
 
-// TODO(Guodong/Ziyi/Xiyang): Should we remove this and instead use `CatalogEntryType`?
 enum class TableType : uint8_t {
     UNKNOWN = 0,
     NODE = 1,
@@ -14,7 +15,7 @@ enum class TableType : uint8_t {
     FOREIGN = 5,
 };
 
-struct TableTypeUtils {
+struct KUZU_API TableTypeUtils {
     static std::string toString(TableType tableType);
 };
 
