@@ -5,6 +5,7 @@
 
 #include "common/api.h"
 #include "common/cast.h"
+#include "function/table/table_function.h"
 
 namespace kuzu {
 namespace common {
@@ -29,8 +30,14 @@ struct KUZU_API FileInfo {
 
     int64_t seek(uint64_t offset, int whence);
 
+    void reset();
+
     void truncate(uint64_t size);
 
+    bool canPerformSeek() const;
+
+    virtual function::TableFunction getHandleFunction() const { KU_UNREACHABLE; }
+
     template<class TARGET>
     TARGET* ptrCast() {
         return common::ku_dynamic_cast<TARGET*>(this);
