@@ -2,6 +2,8 @@
 
 #include <cstdint>
 
+#include "common/cast.h"
+
 namespace kuzu {
 namespace common {
 
@@ -11,6 +13,11 @@ public:
     virtual ~Reader() = default;
 
     virtual bool finished() = 0;
+
+    template<typename TARGET>
+    TARGET* cast() {
+        return common::ku_dynamic_cast<TARGET*>(this);
+    }
 };
 
 } // namespace common
