@@ -46,9 +46,9 @@ using idx_t = uint32_t;
 constexpr idx_t INVALID_IDX = UINT32_MAX;
 using block_idx_t = uint64_t;
 constexpr block_idx_t INVALID_BLOCK_IDX = UINT64_MAX;
-using struct_field_idx_t = uint8_t;
+using struct_field_idx_t = uint16_t;
 using union_field_idx_t = struct_field_idx_t;
-constexpr struct_field_idx_t INVALID_STRUCT_FIELD_IDX = UINT8_MAX;
+constexpr struct_field_idx_t INVALID_STRUCT_FIELD_IDX = UINT16_MAX;
 using row_idx_t = uint64_t;
 constexpr row_idx_t INVALID_ROW_IDX = UINT64_MAX;
 constexpr uint32_t UNDEFINED_CAST_COST = UINT32_MAX;
@@ -115,8 +115,8 @@ struct list_entry_t {
     offset_t offset;
     list_size_t size;
 
-    list_entry_t() : offset{INVALID_OFFSET}, size{UINT32_MAX} {}
-    list_entry_t(offset_t offset, list_size_t size) : offset{offset}, size{size} {}
+    constexpr list_entry_t() : offset{INVALID_OFFSET}, size{UINT32_MAX} {}
+    constexpr list_entry_t(offset_t offset, list_size_t size) : offset{offset}, size{size} {}
 };
 
 struct struct_entry_t {
@@ -135,10 +135,14 @@ struct int128_t;
 struct ku_string_t;
 
 template<typename T>
-concept IntegerTypes =
+concept SignedIntegerTypes =
     std::is_same_v<T, int8_t> || std::is_same_v<T, int16_t> || std::is_same_v<T, int32_t> ||
-    std::is_same_v<T, int64_t> || std::is_same_v<T, uint8_t> || std::is_same_v<T, uint16_t> ||
-    std::is_same_v<T, uint32_t> || std::is_same_v<T, uint64_t> || std::is_same_v<T, int128_t>;
+    std::is_same_v<T, int64_t> || std::is_same_v<T, int128_t>;
+
+template<typename T>
+concept IntegerTypes =
+    SignedIntegerTypes<T> || std::is_same_v<T, uint8_t> || std::is_same_v<T, uint16_t> ||
+    std::is_same_v<T, uint32_t> || std::is_same_v<T, uint64_t>;
 
 template<typename T>
 concept FloatingPointTypes = std::is_same_v<T, float> || std::is_same_v<T, double>;
@@ -333,11 +337,11 @@ public:
     static LogicalType POINTER() { return LogicalType(LogicalTypeID::POINTER); }
     static KUZU_API LogicalType STRUCT(std::vector<StructField>&& fields);
 
-    static KUZU_API LogicalType RECURSIVE_REL(std::unique_ptr<StructTypeInfo> typeInfo);
+    static KUZU_API LogicalType RECURSIVE_REL(std::vector<StructField>&& fields);
 
-    static KUZU_API LogicalType NODE(std::unique_ptr<StructTypeInfo> typeInfo);
+    static KUZU_API LogicalType NODE(std::vector<StructField>&& fields);
 
-    static KUZU_API LogicalType REL(std::unique_ptr<StructTypeInfo> typeInfo);
+    static KUZU_API LogicalType REL(std::vector<StructField>&& fields);
 
     static KUZU_API LogicalType UNION(std::vector<StructField>&& fields);
 
@@ -373,7 +377,7 @@ private:
     TypeCategory category = TypeCategory::INTERNAL;
 };
 
-class ExtraTypeInfo {
+class KUZU_API ExtraTypeInfo {
 public:
     virtual ~ExtraTypeInfo() = default;
 
@@ -437,7 +441,7 @@ protected:
     uint32_t precision, scale;
 };
 
-class ListTypeInfo : public ExtraTypeInfo {
+class KUZU_API ListTypeInfo : public ExtraTypeInfo {
 public:
     ListTypeInfo() = default;
     explicit ListTypeInfo(LogicalType childType) : childType{std::move(childType)} {}
@@ -459,7 +463,7 @@ protected:
     LogicalType childType;
 };
 
-class ArrayTypeInfo final : public ListTypeInfo {
+class KUZU_API ArrayTypeInfo final : public ListTypeInfo {
 public:
     ArrayTypeInfo() : numElements{0} {};
     explicit ArrayTypeInfo(LogicalType childType, uint64_t numElements)
@@ -579,8 +583,6 @@ struct KUZU_API StructType {
     static const StructField& getField(const LogicalType& type, const std::string& key);
 
     static struct_field_idx_t getFieldIdx(const LogicalType& type, const std::string& key);
-
-    static LogicalType getNodeType(const catalog::NodeTableCatalogEntry& entry);
 };
 
 struct KUZU_API MapType {
@@ -592,7 +594,7 @@ struct KUZU_API MapType {
 struct KUZU_API UnionType {
     static constexpr union_field_idx_t TAG_FIELD_IDX = 0;
 
-    static constexpr auto TAG_FIELD_TYPE = LogicalTypeID::INT8;
+    static constexpr auto TAG_FIELD_TYPE = LogicalTypeID::UINT16;
 
     static constexpr char TAG_FIELD_NAME[] = "tag";
 
@@ -643,6 +645,7 @@ struct KUZU_API LogicalTypeUtils {
     // Also combines structs by the union of their fields. As such, currently, it is not guaranteed
     // for casting to work from input types to resulting types. Ideally this changes
     static LogicalType combineTypes(const LogicalType& left, const LogicalType& right);
+    static LogicalType combineTypes(const std::vector<LogicalType>& types);
 
     // makes a copy of the type with any occurences of ANY replaced with replacement
     static LogicalType purgeAny(const LogicalType& type, const LogicalType& replacement);
