@@ -9,7 +9,7 @@ struct BaseCountFunction {
 
     struct CountState : public AggregateState {
         inline uint32_t getStateSize() const override { return sizeof(*this); }
-        inline void moveResultToVector(common::ValueVector* outputVector, uint64_t pos) override {
+        inline void writeToVector(common::ValueVector* outputVector, uint64_t pos) override {
             memcpy(outputVector->getData() + pos * outputVector->getNumBytesPerValue(),
                 reinterpret_cast<uint8_t*>(&count), outputVector->getNumBytesPerValue());
         }
@@ -19,12 +19,11 @@ struct BaseCountFunction {
 
     static std::unique_ptr<AggregateState> initialize() {
         auto state = std::make_unique<CountState>();
-        state->isNull = false;
         return state;
     }
 
     static void combine(uint8_t* state_, uint8_t* otherState_,
-        storage::MemoryManager* /*memoryManager*/) {
+        common::InMemOverflowBuffer* /*overflowBuffer*/) {
         auto state = reinterpret_cast<CountState*>(state_);
         auto otherState = reinterpret_cast<CountState*>(otherState_);
         state->count += otherState->count;
