@@ -75,7 +75,18 @@ struct BinaryUDFFunctionWrapper {
     }
 };
 
+struct BinarySelectWithBindDataWrapper {
+    template<typename LEFT_TYPE, typename RIGHT_TYPE, typename OP>
+    static void operation(LEFT_TYPE& left, RIGHT_TYPE& right, uint8_t& result,
+        common::ValueVector* leftValueVector, common::ValueVector* rightValueVector,
+        void* dataPtr) {
+        OP::operation(left, right, result, *leftValueVector, *rightValueVector, *leftValueVector,
+            dataPtr);
+    }
+};
+
 struct BinaryFunctionExecutor {
+
     template<typename LEFT_TYPE, typename RIGHT_TYPE, typename RESULT_TYPE, typename FUNC,
         typename OP_WRAPPER>
     static inline void executeOnValue(common::ValueVector& left, common::ValueVector& right,
@@ -87,168 +98,78 @@ struct BinaryFunctionExecutor {
             resPos, dataPtr);
     }
 
-    template<typename LEFT_TYPE, typename RIGHT_TYPE, typename RESULT_TYPE, typename FUNC,
-        typename OP_WRAPPER>
-    static void executeBothFlat(common::ValueVector& left, common::ValueVector& right,
-        common::ValueVector& result, void* dataPtr) {
-        auto lPos = left.state->getSelVector()[0];
-        auto rPos = right.state->getSelVector()[0];
-        auto resPos = result.state->getSelVector()[0];
-        result.setNull(resPos, left.isNull(lPos) || right.isNull(rPos));
-        if (!result.isNull(resPos)) {
-            executeOnValue<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, OP_WRAPPER>(left, right,
-                result, lPos, rPos, resPos, dataPtr);
-        }
+    static inline std::tuple<common::sel_t, common::sel_t, common::sel_t> getSelectedPositions(
+        common::SelectionVector* leftSelVector, common::SelectionVector* rightSelVector,
+        common::SelectionVector* resultSelVector, common::sel_t selPos, bool leftFlat,
+        bool rightFlat) {
+        common::sel_t lPos = (*leftSelVector)[leftFlat ? 0 : selPos];
+        common::sel_t rPos = (*rightSelVector)[rightFlat ? 0 : selPos];
+        common::sel_t resPos = (*resultSelVector)[leftFlat && rightFlat ? 0 : selPos];
+        return {lPos, rPos, resPos};
     }
 
     template<typename LEFT_TYPE, typename RIGHT_TYPE, typename RESULT_TYPE, typename FUNC,
         typename OP_WRAPPER>
-    static void executeFlatUnFlat(common::ValueVector& left, common::ValueVector& right,
-        common::ValueVector& result, void* dataPtr) {
-        auto lPos = left.state->getSelVector()[0];
-        auto& rightSelVector = right.state->getSelVector();
-        if (left.isNull(lPos)) {
+    static void executeOnSelectedValues(common::ValueVector& left,
+        common::SelectionVector* leftSelVector, common::ValueVector& right,
+        common::SelectionVector* rightSelVector, common::ValueVector& result,
+        common::SelectionVector* resultSelVector, void* dataPtr) {
+        const bool leftFlat = left.state->isFlat();
+        const bool rightFlat = right.state->isFlat();
+
+        const bool allNullsGuaranteed = (rightFlat && right.isNull((*rightSelVector)[0])) ||
+                                        (leftFlat && left.isNull((*leftSelVector)[0]));
+        if (allNullsGuaranteed) {
             result.setAllNull();
-        } else if (right.hasNoNullsGuarantee()) {
-            result.setAllNonNull();
-            rightSelVector.forEach([&](auto i) {
-                executeOnValue<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, OP_WRAPPER>(left, right,
-                    result, lPos, i, i, dataPtr);
-            });
         } else {
-            rightSelVector.forEach([&](auto i) {
-                result.setNull(i, right.isNull(i)); // left is always not null
-                if (!result.isNull(i)) {
+            const bool noNullsGuaranteed = (leftFlat || left.hasNoNullsGuarantee()) &&
+                                           (rightFlat || right.hasNoNullsGuarantee());
+            if (noNullsGuaranteed) {
+                result.setAllNonNull();
+            }
+
+            const auto numSelectedValues =
+                leftFlat ? rightSelVector->getSelSize() : leftSelVector->getSelSize();
+            for (common::sel_t selPos = 0; selPos < numSelectedValues; ++selPos) {
+                auto [lPos, rPos, resPos] = getSelectedPositions(leftSelVector, rightSelVector,
+                    resultSelVector, selPos, leftFlat, rightFlat);
+                if (noNullsGuaranteed) {
                     executeOnValue<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, OP_WRAPPER>(left,
-                        right, result, lPos, i, i, dataPtr);
+                        right, result, lPos, rPos, resPos, dataPtr);
+                } else {
+                    result.setNull(resPos, left.isNull(lPos) || right.isNull(rPos));
+                    if (!result.isNull(resPos)) {
+                        executeOnValue<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, OP_WRAPPER>(left,
+                            right, result, lPos, rPos, resPos, dataPtr);
+                    }
                 }
-            });
+            }
         }
     }
 
     template<typename LEFT_TYPE, typename RIGHT_TYPE, typename RESULT_TYPE, typename FUNC,
         typename OP_WRAPPER>
-    static void executeUnFlatFlat(common::ValueVector& left, common::ValueVector& right,
-        common::ValueVector& result, void* dataPtr) {
-        auto rPos = right.state->getSelVector()[0];
-        auto& leftSelVector = left.state->getSelVector();
-        if (right.isNull(rPos)) {
-            result.setAllNull();
-        } else if (left.hasNoNullsGuarantee()) {
-            result.setAllNonNull();
-            leftSelVector.forEach([&](auto i) {
-                executeOnValue<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, OP_WRAPPER>(left, right,
-                    result, i, rPos, i, dataPtr);
-            });
-        } else {
-            leftSelVector.forEach([&](auto i) {
-                result.setNull(i, left.isNull(i)); // right is always not null
-                if (!result.isNull(i)) {
-                    executeOnValue<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, OP_WRAPPER>(left,
-                        right, result, i, rPos, i, dataPtr);
-                }
-            });
-        }
-    }
-
-    template<typename LEFT_TYPE, typename RIGHT_TYPE, typename RESULT_TYPE, typename FUNC,
-        typename OP_WRAPPER>
-    static void executeBothUnFlat(common::ValueVector& left, common::ValueVector& right,
-        common::ValueVector& result, void* dataPtr) {
-        KU_ASSERT(left.state == right.state);
-        auto& resultSelVector = result.state->getSelVector();
-        if (left.hasNoNullsGuarantee() && right.hasNoNullsGuarantee()) {
-            result.setAllNonNull();
-            resultSelVector.forEach([&](auto i) {
-                executeOnValue<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, OP_WRAPPER>(left, right,
-                    result, i, i, i, dataPtr);
-            });
-        } else {
-            resultSelVector.forEach([&](auto i) {
-                result.setNull(i, left.isNull(i) || right.isNull(i));
-                if (!result.isNull(i)) {
-                    executeOnValue<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, OP_WRAPPER>(left,
-                        right, result, i, i, i, dataPtr);
-                }
-            });
-        }
-    }
-
-    template<typename LEFT_TYPE, typename RIGHT_TYPE, typename RESULT_TYPE, typename FUNC,
-        typename OP_WRAPPER>
-    static void executeSwitch(common::ValueVector& left, common::ValueVector& right,
-        common::ValueVector& result, void* dataPtr) {
+    static void executeSwitch(common::ValueVector& left, common::SelectionVector* leftSelVector,
+        common::ValueVector& right, common::SelectionVector* rightSelVector,
+        common::ValueVector& result, common::SelectionVector* resultSelVector, void* dataPtr) {
         result.resetAuxiliaryBuffer();
-        if (left.state->isFlat() && right.state->isFlat()) {
-            executeBothFlat<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, OP_WRAPPER>(left, right,
-                result, dataPtr);
-        } else if (left.state->isFlat() && !right.state->isFlat()) {
-            executeFlatUnFlat<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, OP_WRAPPER>(left, right,
-                result, dataPtr);
-        } else if (!left.state->isFlat() && right.state->isFlat()) {
-            executeUnFlatFlat<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, OP_WRAPPER>(left, right,
-                result, dataPtr);
-        } else if (!left.state->isFlat() && !right.state->isFlat()) {
-            executeBothUnFlat<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, OP_WRAPPER>(left, right,
-                result, dataPtr);
-        } else {
-            KU_ASSERT(false);
-        }
+        executeOnSelectedValues<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, OP_WRAPPER>(left,
+            leftSelVector, right, rightSelVector, result, resultSelVector, dataPtr);
     }
 
     template<typename LEFT_TYPE, typename RIGHT_TYPE, typename RESULT_TYPE, typename FUNC>
-    static void execute(common::ValueVector& left, common::ValueVector& right,
-        common::ValueVector& result) {
-        executeSwitch<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, BinaryFunctionWrapper>(left, right,
-            result, nullptr /* dataPtr */);
-    }
-
-    template<typename LEFT_TYPE, typename RIGHT_TYPE, typename RESULT_TYPE, typename FUNC>
-    static void executeString(common::ValueVector& left, common::ValueVector& right,
-        common::ValueVector& result) {
-        executeSwitch<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, BinaryStringFunctionWrapper>(left,
-            right, result, nullptr /* dataPtr */);
-    }
-
-    template<typename LEFT_TYPE, typename RIGHT_TYPE, typename RESULT_TYPE, typename FUNC>
-    static void executeListStruct(common::ValueVector& left, common::ValueVector& right,
-        common::ValueVector& result) {
-        executeSwitch<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, BinaryListStructFunctionWrapper>(
-            left, right, result, nullptr /* dataPtr */);
-    }
-
-    template<typename LEFT_TYPE, typename RIGHT_TYPE, typename RESULT_TYPE, typename FUNC>
-    static void executeMapCreation(common::ValueVector& left, common::ValueVector& right,
-        common::ValueVector& result, void* dataPtr) {
-        executeSwitch<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, BinaryMapCreationFunctionWrapper>(
-            left, right, result, dataPtr);
-    }
-
-    template<typename LEFT_TYPE, typename RIGHT_TYPE, typename RESULT_TYPE, typename FUNC>
-    static void executeListExtract(common::ValueVector& left, common::ValueVector& right,
-        common::ValueVector& result) {
-        executeSwitch<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, BinaryListExtractFunctionWrapper>(
-            left, right, result, nullptr /* dataPtr */);
-    }
-
-    template<typename LEFT_TYPE, typename RIGHT_TYPE, typename RESULT_TYPE, typename FUNC>
-    static void executeComparison(common::ValueVector& left, common::ValueVector& right,
-        common::ValueVector& result) {
-        executeSwitch<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, BinaryComparisonFunctionWrapper>(
-            left, right, result, nullptr /* dataPtr */);
-    }
-
-    template<typename LEFT_TYPE, typename RIGHT_TYPE, typename RESULT_TYPE, typename FUNC>
-    static void executeUDF(common::ValueVector& left, common::ValueVector& right,
-        common::ValueVector& result, void* dataPtr) {
-        executeSwitch<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, BinaryUDFFunctionWrapper>(left,
-            right, result, dataPtr);
+    static void execute(common::ValueVector& left, common::SelectionVector* leftSelVector,
+        common::ValueVector& right, common::SelectionVector* rightSelVector,
+        common::ValueVector& result, common::SelectionVector* resultSelVector) {
+        executeSwitch<LEFT_TYPE, RIGHT_TYPE, RESULT_TYPE, FUNC, BinaryFunctionWrapper>(left,
+            leftSelVector, right, rightSelVector, result, resultSelVector, nullptr /* dataPtr */);
     }
 
     struct BinarySelectWrapper {
         template<typename LEFT_TYPE, typename RIGHT_TYPE, typename OP>
         static inline void operation(LEFT_TYPE& left, RIGHT_TYPE& right, uint8_t& result,
-            common::ValueVector* /*leftValueVector*/, common::ValueVector* /*rightValueVector*/) {
+            common::ValueVector* /*leftValueVector*/, common::ValueVector* /*rightValueVector*/,
+            void* /*dataPtr*/) {
             OP::operation(left, right, result);
         }
     };
@@ -256,7 +177,8 @@ struct BinaryFunctionExecutor {
     struct BinaryComparisonSelectWrapper {
         template<typename LEFT_TYPE, typename RIGHT_TYPE, typename OP>
         static inline void operation(LEFT_TYPE& left, RIGHT_TYPE& right, uint8_t& result,
-            common::ValueVector* leftValueVector, common::ValueVector* rightValueVector) {
+            common::ValueVector* leftValueVector, common::ValueVector* rightValueVector,
+            void* /*dataPtr*/) {
             OP::operation(left, right, result, leftValueVector, rightValueVector);
         }
     };
@@ -264,31 +186,32 @@ struct BinaryFunctionExecutor {
     template<class LEFT_TYPE, class RIGHT_TYPE, class FUNC, typename SELECT_WRAPPER>
     static void selectOnValue(common::ValueVector& left, common::ValueVector& right, uint64_t lPos,
         uint64_t rPos, uint64_t resPos, uint64_t& numSelectedValues,
-        std::span<common::sel_t> selectedPositionsBuffer) {
+        std::span<common::sel_t> selectedPositionsBuffer, void* dataPtr) {
         uint8_t resultValue = 0;
         SELECT_WRAPPER::template operation<LEFT_TYPE, RIGHT_TYPE, FUNC>(
             ((LEFT_TYPE*)left.getData())[lPos], ((RIGHT_TYPE*)right.getData())[rPos], resultValue,
-            &left, &right);
+            &left, &right, dataPtr);
         selectedPositionsBuffer[numSelectedValues] = resPos;
         numSelectedValues += (resultValue == true);
     }
 
     template<class LEFT_TYPE, class RIGHT_TYPE, class FUNC, typename SELECT_WRAPPER>
-    static uint64_t selectBothFlat(common::ValueVector& left, common::ValueVector& right) {
+    static uint64_t selectBothFlat(common::ValueVector& left, common::ValueVector& right,
+        void* dataPtr) {
         auto lPos = left.state->getSelVector()[0];
         auto rPos = right.state->getSelVector()[0];
         uint8_t resultValue = 0;
         if (!left.isNull(lPos) && !right.isNull(rPos)) {
             SELECT_WRAPPER::template operation<LEFT_TYPE, RIGHT_TYPE, FUNC>(
                 ((LEFT_TYPE*)left.getData())[lPos], ((RIGHT_TYPE*)right.getData())[rPos],
-                resultValue, &left, &right);
+                resultValue, &left, &right, dataPtr);
         }
         return resultValue == true;
     }
 
     template<typename LEFT_TYPE, typename RIGHT_TYPE, typename FUNC, typename SELECT_WRAPPER>
     static bool selectFlatUnFlat(common::ValueVector& left, common::ValueVector& right,
-        common::SelectionVector& selVector) {
+        common::SelectionVector& selVector, void* dataPtr) {
         auto lPos = left.state->getSelVector()[0];
         uint64_t numSelectedValues = 0;
         auto selectedPositionsBuffer = selVector.getMutableBuffer();
@@ -298,13 +221,13 @@ struct BinaryFunctionExecutor {
         } else if (right.hasNoNullsGuarantee()) {
             rightSelVector.forEach([&](auto i) {
                 selectOnValue<LEFT_TYPE, RIGHT_TYPE, FUNC, SELECT_WRAPPER>(left, right, lPos, i, i,
-                    numSelectedValues, selectedPositionsBuffer);
+                    numSelectedValues, selectedPositionsBuffer, dataPtr);
             });
         } else {
             rightSelVector.forEach([&](auto i) {
                 if (!right.isNull(i)) {
                     selectOnValue<LEFT_TYPE, RIGHT_TYPE, FUNC, SELECT_WRAPPER>(left, right, lPos, i,
-                        i, numSelectedValues, selectedPositionsBuffer);
+                        i, numSelectedValues, selectedPositionsBuffer, dataPtr);
                 }
             });
         }
@@ -314,7 +237,7 @@ struct BinaryFunctionExecutor {
 
     template<typename LEFT_TYPE, typename RIGHT_TYPE, typename FUNC, typename SELECT_WRAPPER>
     static bool selectUnFlatFlat(common::ValueVector& left, common::ValueVector& right,
-        common::SelectionVector& selVector) {
+        common::SelectionVector& selVector, void* dataPtr) {
         auto rPos = right.state->getSelVector()[0];
         uint64_t numSelectedValues = 0;
         auto selectedPositionsBuffer = selVector.getMutableBuffer();
@@ -324,13 +247,13 @@ struct BinaryFunctionExecutor {
         } else if (left.hasNoNullsGuarantee()) {
             leftSelVector.forEach([&](auto i) {
                 selectOnValue<LEFT_TYPE, RIGHT_TYPE, FUNC, SELECT_WRAPPER>(left, right, i, rPos, i,
-                    numSelectedValues, selectedPositionsBuffer);
+                    numSelectedValues, selectedPositionsBuffer, dataPtr);
             });
         } else {
             leftSelVector.forEach([&](auto i) {
                 if (!left.isNull(i)) {
                     selectOnValue<LEFT_TYPE, RIGHT_TYPE, FUNC, SELECT_WRAPPER>(left, right, i, rPos,
-                        i, numSelectedValues, selectedPositionsBuffer);
+                        i, numSelectedValues, selectedPositionsBuffer, dataPtr);
                 }
             });
         }
@@ -341,21 +264,21 @@ struct BinaryFunctionExecutor {
     // Right, left, and result vectors share the same selectedPositions.
     template<class LEFT_TYPE, class RIGHT_TYPE, class FUNC, typename SELECT_WRAPPER>
     static bool selectBothUnFlat(common::ValueVector& left, common::ValueVector& right,
-        common::SelectionVector& selVector) {
+        common::SelectionVector& selVector, void* dataPtr) {
         uint64_t numSelectedValues = 0;
         auto selectedPositionsBuffer = selVector.getMutableBuffer();
         auto& leftSelVector = left.state->getSelVector();
         if (left.hasNoNullsGuarantee() && right.hasNoNullsGuarantee()) {
             leftSelVector.forEach([&](auto i) {
                 selectOnValue<LEFT_TYPE, RIGHT_TYPE, FUNC, SELECT_WRAPPER>(left, right, i, i, i,
-                    numSelectedValues, selectedPositionsBuffer);
+                    numSelectedValues, selectedPositionsBuffer, dataPtr);
             });
         } else {
             leftSelVector.forEach([&](auto i) {
                 auto isNull = left.isNull(i) || right.isNull(i);
                 if (!isNull) {
                     selectOnValue<LEFT_TYPE, RIGHT_TYPE, FUNC, SELECT_WRAPPER>(left, right, i, i, i,
-                        numSelectedValues, selectedPositionsBuffer);
+                        numSelectedValues, selectedPositionsBuffer, dataPtr);
                 }
             });
         }
@@ -364,39 +287,40 @@ struct BinaryFunctionExecutor {
     }
 
     // BOOLEAN (AND, OR, XOR)
-    template<class LEFT_TYPE, class RIGHT_TYPE, class FUNC>
+    template<class LEFT_TYPE, class RIGHT_TYPE, class FUNC,
+        typename OP_WRAPPER = BinarySelectWrapper>
     static bool select(common::ValueVector& left, common::ValueVector& right,
-        common::SelectionVector& selVector) {
+        common::SelectionVector& selVector, void* dataPtr) {
         if (left.state->isFlat() && right.state->isFlat()) {
-            return selectBothFlat<LEFT_TYPE, RIGHT_TYPE, FUNC, BinarySelectWrapper>(left, right);
+            return selectBothFlat<LEFT_TYPE, RIGHT_TYPE, FUNC, OP_WRAPPER>(left, right, dataPtr);
         } else if (left.state->isFlat() && !right.state->isFlat()) {
-            return selectFlatUnFlat<LEFT_TYPE, RIGHT_TYPE, FUNC, BinarySelectWrapper>(left, right,
-                selVector);
+            return selectFlatUnFlat<LEFT_TYPE, RIGHT_TYPE, FUNC, OP_WRAPPER>(left, right, selVector,
+                dataPtr);
         } else if (!left.state->isFlat() && right.state->isFlat()) {
-            return selectUnFlatFlat<LEFT_TYPE, RIGHT_TYPE, FUNC, BinarySelectWrapper>(left, right,
-                selVector);
+            return selectUnFlatFlat<LEFT_TYPE, RIGHT_TYPE, FUNC, OP_WRAPPER>(left, right, selVector,
+                dataPtr);
         } else {
-            return selectBothUnFlat<LEFT_TYPE, RIGHT_TYPE, FUNC, BinarySelectWrapper>(left, right,
-                selVector);
+            return selectBothUnFlat<LEFT_TYPE, RIGHT_TYPE, FUNC, OP_WRAPPER>(left, right, selVector,
+                dataPtr);
         }
     }
 
     // COMPARISON (GT, GTE, LT, LTE, EQ, NEQ)
     template<class LEFT_TYPE, class RIGHT_TYPE, class FUNC>
     static bool selectComparison(common::ValueVector& left, common::ValueVector& right,
-        common::SelectionVector& selVector) {
+        common::SelectionVector& selVector, void* dataPtr) {
         if (left.state->isFlat() && right.state->isFlat()) {
             return selectBothFlat<LEFT_TYPE, RIGHT_TYPE, FUNC, BinaryComparisonSelectWrapper>(left,
-                right);
+                right, dataPtr);
         } else if (left.state->isFlat() && !right.state->isFlat()) {
             return selectFlatUnFlat<LEFT_TYPE, RIGHT_TYPE, FUNC, BinaryComparisonSelectWrapper>(
-                left, right, selVector);
+                left, right, selVector, dataPtr);
         } else if (!left.state->isFlat() && right.state->isFlat()) {
             return selectUnFlatFlat<LEFT_TYPE, RIGHT_TYPE, FUNC, BinaryComparisonSelectWrapper>(
-                left, right, selVector);
+                left, right, selVector, dataPtr);
         } else {
             return selectBothUnFlat<LEFT_TYPE, RIGHT_TYPE, FUNC, BinaryComparisonSelectWrapper>(
-                left, right, selVector);
+                left, right, selVector, dataPtr);
         }
     }
 };
