@@ -8,10 +8,10 @@ namespace function {
 struct ConstFunctionExecutor {
 
     template<typename RESULT_TYPE, typename OP>
-    static void execute(common::ValueVector& result) {
+    static void execute(common::ValueVector& result, common::SelectionVector& sel) {
         KU_ASSERT(result.state->isFlat());
         auto resultValues = (RESULT_TYPE*)result.getData();
-        auto idx = result.state->getSelVector()[0];
+        auto idx = sel[0];
         KU_ASSERT(idx == 0);
         OP::operation(resultValues[idx]);
     }
