@@ -18,17 +18,18 @@ private:
     template<typename FUNC>
     static void UnaryNullExecFunction(
         const std::vector<std::shared_ptr<common::ValueVector>>& params,
-        common::ValueVector& result, void* /*dataPtr*/ = nullptr) {
+        const std::vector<common::SelectionVector*>& paramSelVectors, common::ValueVector& result,
+        common::SelectionVector*, void* /*dataPtr*/ = nullptr) {
         KU_ASSERT(params.size() == 1);
-        NullOperationExecutor::execute<FUNC>(*params[0], result);
+        NullOperationExecutor::execute<FUNC>(*params[0], *paramSelVectors[0], result);
     }
 
     template<typename FUNC>
     static bool UnaryNullSelectFunction(
         const std::vector<std::shared_ptr<common::ValueVector>>& params,
-        common::SelectionVector& selVector) {
+        common::SelectionVector& selVector, void* dataPtr) {
         KU_ASSERT(params.size() == 1);
-        return NullOperationExecutor::select<FUNC>(*params[0], selVector);
+        return NullOperationExecutor::select<FUNC>(*params[0], selVector, dataPtr);
     }
 };
 
