@@ -38,5 +38,17 @@ struct LabelFunction {
     static std::shared_ptr<binder::Expression> rewriteFunc(const RewriteFunctionBindInput& input);
 };
 
+struct LabelsFunction {
+    using alias = LabelFunction;
+
+    static constexpr const char* name = "LABELS";
+};
+
+struct CostFunction {
+    static constexpr const char* name = "COST";
+
+    static function_set getFunctionSet();
+};
+
 } // namespace function
 } // namespace kuzu
