@@ -6,6 +6,10 @@
 #include "extension/catalog_extension.h"
 #include "transaction/transaction_manager.h"
 
+namespace duckdb {
+class MaterializedQueryResult;
+}
+
 namespace kuzu {
 namespace storage {
 class StorageManager;
@@ -27,15 +31,22 @@ public:
 
     catalog::Catalog* getCatalog() { return catalog.get(); }
 
+    std::unique_ptr<duckdb::MaterializedQueryResult> executeQuery(const std::string& query);
+
     void invalidateCache();
 
+    template<class TARGET>
+    const TARGET& constCast() const {
+        return common::ku_dynamic_cast<const TARGET&>(*this);
+    }
+
 protected:
     std::string dbName;
     std::string dbType;
     std::unique_ptr<catalog::Catalog> catalog;
 };
 
-class AttachedKuzuDatabase : public AttachedDatabase {
+class AttachedKuzuDatabase final : public AttachedDatabase {
 public:
     AttachedKuzuDatabase(std::string dbPath, std::string dbName, std::string dbType,
         ClientContext* clientContext);
@@ -44,9 +55,6 @@ public:
 
     transaction::TransactionManager* getTransactionManager() { return transactionManager.get(); }
 
-private:
-    void initCatalog(const std::string& path, ClientContext* context);
-
 private:
     std::unique_ptr<storage::StorageManager> storageManager;
     std::unique_ptr<transaction::TransactionManager> transactionManager;
