@@ -12,7 +12,7 @@ public:
     void collect(planner::LogicalOperator* op);
 
     bool hasOperators() const { return !ops.empty(); }
-    std::vector<planner::LogicalOperator*> getOperators() const { return ops; }
+    const std::vector<planner::LogicalOperator*>& getOperators() const { return ops; }
 
 protected:
     std::vector<planner::LogicalOperator*> ops;
@@ -44,10 +44,5 @@ protected:
     void visitRecursiveExtend(planner::LogicalOperator* op) override { ops.push_back(op); }
 };
 
-class LogicalGDSCallCollector final : public LogicalOperatorCollector {
-protected:
-    void visitGDSCall(planner::LogicalOperator* op) override { ops.push_back(op); }
-};
-
 } // namespace optimizer
 } // namespace kuzu
