@@ -1,8 +1,7 @@
 #pragma once
 
 #include "parser/ddl/drop_info.h"
-#include "planner/operator/ddl/logical_ddl.h"
-#include "planner/operator/logical_operator.h"
+#include "planner/operator/simple/logical_simple.h"
 
 namespace kuzu {
 namespace planner {
@@ -15,12 +14,14 @@ struct LogicalDropPrintInfo : OPPrintInfo {
     std::string toString() const override { return name; }
 };
 
-class LogicalDrop : public LogicalDDL {
+class LogicalDrop : public LogicalSimple {
+    static constexpr LogicalOperatorType type_ = LogicalOperatorType::DROP;
+
 public:
-    LogicalDrop(const parser::DropInfo& dropInfo,
-        std::shared_ptr<binder::Expression> outputExpression)
-        : LogicalDDL{LogicalOperatorType::DROP, dropInfo.name, std::move(outputExpression)},
-          dropInfo{dropInfo} {}
+    explicit LogicalDrop(parser::DropInfo dropInfo)
+        : LogicalSimple{type_}, dropInfo{std::move(dropInfo)} {}
+
+    std::string getExpressionsForPrinting() const override { return dropInfo.name; }
 
     const parser::DropInfo& getDropInfo() const { return dropInfo; }
 
@@ -29,7 +30,7 @@ public:
     }
 
     std::unique_ptr<LogicalOperator> copy() override {
-        return make_unique<LogicalDrop>(dropInfo, outputExpression);
+        return std::make_unique<LogicalDrop>(dropInfo);
     }
 
 private:
