@@ -7,12 +7,14 @@ namespace planner {
 
 using cardinality_t = uint64_t;
 
-class LogicalPlan {
+class KUZU_API LogicalPlan {
     friend class CardinalityEstimator;
     friend class CostModel;
 
 public:
     LogicalPlan() : cost{0} {}
+    LogicalPlan(const LogicalPlan& other) : lastOperator{other.lastOperator}, cost{other.cost} {}
+    EXPLICIT_COPY_DEFAULT_MOVE(LogicalPlan);
 
     void setLastOperator(std::shared_ptr<LogicalOperator> op) { lastOperator = std::move(op); }
 
@@ -38,10 +40,6 @@ public:
     bool isProfile() const;
     bool hasUpdate() const;
 
-    std::unique_ptr<LogicalPlan> shallowCopy() const;
-
-    std::unique_ptr<LogicalPlan> deepCopy() const;
-
 private:
     std::shared_ptr<LogicalOperator> lastOperator;
     uint64_t cost;
