@@ -1,5 +1,6 @@
 #pragma once
 
+// TODO(Guodong): Change these includes into forward declaration.
 #include "common/profiler.h"
 #include "main/client_context.h"
 
@@ -8,7 +9,7 @@ namespace processor {
 
 class FactorizedTable;
 
-struct ExecutionContext {
+struct KUZU_API ExecutionContext {
     uint64_t queryID;
     common::Profiler* profiler;
     main::ClientContext* clientContext;
