@@ -9,17 +9,9 @@ namespace processor {
 class HashAggregateScan final : public BaseAggregateScan {
 public:
     HashAggregateScan(std::shared_ptr<HashAggregateSharedState> sharedState,
-        std::vector<DataPos> groupByKeyVectorsPos, std::vector<DataPos> aggregatesPos,
-        std::unique_ptr<PhysicalOperator> child, uint32_t id,
+        std::vector<DataPos> groupByKeyVectorsPos, AggregateScanInfo scanInfo, uint32_t id,
         std::unique_ptr<OPPrintInfo> printInfo)
-        : BaseAggregateScan{std::move(aggregatesPos), std::move(child), id, std::move(printInfo)},
-          groupByKeyVectorsPos{std::move(groupByKeyVectorsPos)},
-          sharedState{std::move(sharedState)} {}
-
-    HashAggregateScan(std::shared_ptr<HashAggregateSharedState> sharedState,
-        std::vector<DataPos> groupByKeyVectorsPos, std::vector<DataPos> aggregatesPos, uint32_t id,
-        std::unique_ptr<OPPrintInfo> printInfo)
-        : BaseAggregateScan{std::move(aggregatesPos), id, std::move(printInfo)},
+        : BaseAggregateScan{std::move(scanInfo), id, std::move(printInfo)},
           groupByKeyVectorsPos{std::move(groupByKeyVectorsPos)},
           sharedState{std::move(sharedState)} {}
 
@@ -30,8 +22,8 @@ public:
     bool getNextTuplesInternal(ExecutionContext* context) override;
 
     std::unique_ptr<PhysicalOperator> copy() override {
-        return std::make_unique<HashAggregateScan>(sharedState, groupByKeyVectorsPos, aggregatesPos,
-            id, printInfo->copy());
+        return std::make_unique<HashAggregateScan>(sharedState, groupByKeyVectorsPos, scanInfo, id,
+            printInfo->copy());
     }
 
     double getProgress(ExecutionContext* context) const override;
