@@ -9,23 +9,16 @@ namespace processor {
 class SimpleAggregateScan final : public BaseAggregateScan {
 public:
     SimpleAggregateScan(std::shared_ptr<SimpleAggregateSharedState> sharedState,
-        std::vector<DataPos> aggregatesPos, std::unique_ptr<PhysicalOperator> child, uint32_t id,
-        std::unique_ptr<OPPrintInfo> printInfo)
-        : BaseAggregateScan{std::move(aggregatesPos), std::move(child), id, std::move(printInfo)},
-          sharedState{std::move(sharedState)}, outDataChunk{nullptr} {}
-
-    SimpleAggregateScan(std::shared_ptr<SimpleAggregateSharedState> sharedState,
-        std::vector<DataPos> aggregatesPos, uint32_t id, std::unique_ptr<OPPrintInfo> printInfo)
-        : BaseAggregateScan{std::move(aggregatesPos), id, std::move(printInfo)},
+        AggregateScanInfo scanInfo, uint32_t id, std::unique_ptr<OPPrintInfo> printInfo)
+        : BaseAggregateScan{std::move(scanInfo), id, std::move(printInfo)},
           sharedState{std::move(sharedState)}, outDataChunk{nullptr} {}
 
     void initLocalStateInternal(ResultSet* resultSet, ExecutionContext* context) override;
 
     bool getNextTuplesInternal(ExecutionContext* context) override;
 
-    // SimpleAggregateScan is the source operator of a pipeline, so it should not clone its child.
     std::unique_ptr<PhysicalOperator> copy() override {
-        return make_unique<SimpleAggregateScan>(sharedState, aggregatesPos, id, printInfo->copy());
+        return make_unique<SimpleAggregateScan>(sharedState, scanInfo, id, printInfo->copy());
     }
 
 private:
