@@ -1,7 +1,6 @@
-
 #pragma once
 
-#include "processor/operator/ddl/ddl.h"
+#include "processor/operator/sink.h"
 
 namespace kuzu {
 namespace processor {
@@ -24,21 +23,20 @@ private:
         : OPPrintInfo{other}, typeName{other.typeName}, type{other.type} {}
 };
 
-class CreateType final : public DDL {
+class CreateType final : public SimpleSink {
     static constexpr PhysicalOperatorType type_ = PhysicalOperatorType::CREATE_TYPE;
 
 public:
-    CreateType(std::string name, common::LogicalType type, DataPos outputPos, uint32_t id,
+    CreateType(std::string name, common::LogicalType type,
+        std::shared_ptr<FactorizedTable> messageTable, physical_op_id id,
         std::unique_ptr<OPPrintInfo> printInfo)
-        : DDL{type_, outputPos, id, std::move(printInfo)}, name{std::move(name)},
-          type{std::move(type)} {}
+        : SimpleSink{type_, std::move(messageTable), id, std::move(printInfo)},
+          name{std::move(name)}, type{std::move(type)} {}
 
-    void executeDDLInternal(ExecutionContext* context) override;
-
-    std::string getOutputMsg() override;
+    void executeInternal(ExecutionContext* context) override;
 
     std::unique_ptr<PhysicalOperator> copy() override {
-        return std::make_unique<CreateType>(name, type.copy(), outputPos, id, printInfo->copy());
+        return std::make_unique<CreateType>(name, type.copy(), messageTable, id, printInfo->copy());
     }
 
 private:
