@@ -10,7 +10,11 @@ namespace processor {
 class QueryProcessor {
 
 public:
+#if defined(__APPLE__)
+    explicit QueryProcessor(uint64_t numThreads, uint32_t threadQos);
+#else
     explicit QueryProcessor(uint64_t numThreads);
+#endif
 
     inline common::TaskScheduler* getTaskScheduler() { return taskScheduler.get(); }
 
