@@ -17,11 +17,12 @@ public:
     // because the current QueryProcessor::execute requires the last operator in the physical plan
     // must be ResultCollector. We should remove this class after we remove the assumption that the
     // last operator in the pipeline must be resultCollector.
-    static void appendStringToTable(FactorizedTable* factorizedTable, std::string& outputMsg,
+    static void appendStringToTable(FactorizedTable* factorizedTable, const std::string& outputMsg,
         storage::MemoryManager* memoryManager);
-    static std::shared_ptr<FactorizedTable> getFactorizedTableForOutputMsg(std::string& outputMsg,
-        storage::MemoryManager* memoryManager);
-    static std::shared_ptr<FactorizedTable> getSingleStringColumnFTable(storage::MemoryManager* mm);
+    static std::shared_ptr<FactorizedTable> getFactorizedTableForOutputMsg(
+        const std::string& outputMsg, storage::MemoryManager* memoryManager);
+    static KUZU_API std::shared_ptr<FactorizedTable> getSingleStringColumnFTable(
+        storage::MemoryManager* mm);
 };
 
 } // namespace processor
