@@ -1,7 +1,7 @@
-#include "processor/operator/aggregate/aggregate_hash_table.h"
-
 #pragma once
 
+#include "processor/operator/aggregate/aggregate_hash_table.h"
+
 namespace kuzu {
 namespace processor {
 
@@ -25,9 +25,8 @@ public:
 
     PatternCreationInfo getPatternCreationInfo(const std::vector<common::ValueVector*>& keyVectors);
 
-    uint64_t matchFTEntries(std::span<const common::ValueVector*> flatKeyVectors,
-        std::span<const common::ValueVector*> unFlatKeyVectors, uint64_t numMayMatches,
-        uint64_t numNoMatches) override;
+    uint64_t matchFTEntries(std::span<const common::ValueVector*> keyVectors,
+        uint64_t numMayMatches, uint64_t numNoMatches) override;
 
 private:
     uint8_t* tuple;
