@@ -1,7 +1,6 @@
 #pragma once
 
-#include <cstdint>
-
+#include "storage/buffer_manager/memory_manager.h"
 #include "storage/file_handle.h"
 
 namespace kuzu {
@@ -20,17 +19,18 @@ public:
     Spiller(std::string tmpFilePath, BufferManager& bufferManager, common::VirtualFileSystem* vfs);
     void addUnusedChunk(ChunkedNodeGroup* nodeGroup);
     void clearUnusedChunk(ChunkedNodeGroup* nodeGroup);
-    uint64_t spillToDisk(ColumnChunkData& chunk) const;
+    SpillResult spillToDisk(ColumnChunkData& chunk) const;
     void loadFromDisk(ColumnChunkData& chunk) const;
     // reclaims memory from the next full partitioner group in the set
     // and returns the amount of memory reclaimed
     // If the set is empty, returns zero
-    uint64_t claimNextGroup();
+    SpillResult claimNextGroup();
     // Must only be used once all chunks have been loaded from disk.
     void clearFile();
     ~Spiller();
 
 private:
+    FileHandle* getOrCreateDataFH() const;
     FileHandle* getDataFH() const;
 
 private:
@@ -38,7 +38,7 @@ private:
     BufferManager& bufferManager;
     common::VirtualFileSystem* vfs;
     std::unordered_set<ChunkedNodeGroup*> fullPartitionerGroups;
-    FileHandle* dataFH;
+    std::atomic<FileHandle*> dataFH;
     std::mutex partitionerGroupsMtx;
     mutable std::mutex fileCreationMutex;
 };
