@@ -4,12 +4,13 @@
 #include "common/exception/checkpoint.h"
 #include "common/exception/connection.h"
 #include "common/exception/runtime.h"
+#include "common/file_system/virtual_file_system.h"
 #include "common/random_engine.h"
 #include "common/string_utils.h"
 #include "common/task_system/progress_bar.h"
 #include "extension/extension.h"
 #include "extension/extension_manager.h"
-#include "graph/graph_entry.h"
+#include "graph/graph_entry_set.h"
 #include "main/attached_database.h"
 #include "main/database.h"
 #include "main/database_manager.h"
@@ -54,11 +55,7 @@ ClientContext::ClientContext(Database* database)
     randomEngine = std::make_unique<RandomEngine>();
     remoteDatabase = nullptr;
     graphEntrySet = std::make_unique<graph::GraphEntrySet>();
-#if defined(_WIN32)
-    clientConfig.homeDirectory = getEnvVariable("USERPROFILE");
-#else
-    clientConfig.homeDirectory = getEnvVariable("HOME");
-#endif
+    clientConfig.homeDirectory = getUserHomeDir();
     clientConfig.fileSearchPath = "";
     clientConfig.enableSemiMask = ClientConfigDefault::ENABLE_SEMI_MASK;
     clientConfig.enableZoneMap = ClientConfigDefault::ENABLE_ZONE_MAP;
@@ -75,7 +72,14 @@ ClientContext::ClientContext(Database* database)
     progressBar = std::make_unique<ProgressBar>(clientConfig.enableProgressBar);
 }
 
-ClientContext::~ClientContext() = default;
+ClientContext::~ClientContext() {
+    if (preventTransactionRollbackOnDestruction) {
+        return;
+    }
+    if (getTransaction()) {
+        getDatabase()->transactionManager->rollback(*this, getTransaction());
+    }
+}
 
 uint64_t ClientContext::getTimeoutRemainingInMS() const {
     KU_ASSERT(hasTimeout());
@@ -100,6 +104,9 @@ uint64_t ClientContext::getQueryTimeOut() const {
 
 void ClientContext::setMaxNumThreadForExec(uint64_t numThreads) {
     lock_t lck{mtx};
+    if (numThreads == 0) {
+        numThreads = localDatabase->dbConfig.maxNumThreads;
+    }
     clientConfig.numThreads = numThreads;
 }
 
@@ -143,10 +150,23 @@ void ClientContext::addScanReplace(function::ScanReplacement scanReplacement) {
     scanReplacements.push_back(std::move(scanReplacement));
 }
 
-std::unique_ptr<function::ScanReplacementData> ClientContext::tryReplace(
+std::unique_ptr<function::ScanReplacementData> ClientContext::tryReplaceByName(
     const std::string& objectName) const {
     for (auto& scanReplacement : scanReplacements) {
-        auto replaceData = scanReplacement.replaceFunc(objectName);
+        auto replaceHandles = scanReplacement.lookupFunc(objectName);
+        if (replaceHandles.empty()) {
+            continue; // Fail to replace.
+        }
+        return scanReplacement.replaceFunc(std::span(replaceHandles.begin(), replaceHandles.end()));
+    }
+    return {};
+}
+
+std::unique_ptr<function::ScanReplacementData> ClientContext::tryReplaceByHandle(
+    function::scan_replace_handle_t handle) const {
+    auto handleSpan = std::span{&handle, 1};
+    for (auto& scanReplacement : scanReplacements) {
+        auto replaceData = scanReplacement.replaceFunc(handleSpan);
         if (replaceData == nullptr) {
             continue; // Fail to replace.
         }
@@ -226,6 +246,14 @@ RandomEngine* ClientContext::getRandomEngine() const {
     return randomEngine.get();
 }
 
+bool ClientContext::isInMemory() const {
+    if (remoteDatabase != nullptr) {
+        // If we are connected to a remote database, we assume it is not in memory.
+        return false;
+    }
+    return localDatabase->storageManager->isInMemory();
+}
+
 std::string ClientContext::getEnvVariable(const std::string& name) {
 #if defined(_WIN32)
     auto envValue = WindowsUtils::utf8ToUnicode(name.c_str());
@@ -243,6 +271,14 @@ std::string ClientContext::getEnvVariable(const std::string& name) {
 #endif
 }
 
+std::string ClientContext::getUserHomeDir() {
+#if defined(_WIN32)
+    return getEnvVariable("USERPROFILE");
+#else
+    return getEnvVariable("HOME");
+#endif
+}
+
 void ClientContext::setDefaultDatabase(AttachedKuzuDatabase* defaultDatabase_) {
     remoteDatabase = defaultDatabase_;
 }
@@ -282,11 +318,16 @@ graph::GraphEntrySet& ClientContext::getGraphEntrySetUnsafe() {
     return *graphEntrySet;
 }
 
+const graph::GraphEntrySet& ClientContext::getGraphEntrySet() const {
+    return *graphEntrySet;
+}
+
 void ClientContext::cleanUp() {
     getVFSUnsafe()->cleanUP(this);
 }
 
-std::unique_ptr<PreparedStatement> ClientContext::prepare(std::string_view query) {
+std::unique_ptr<PreparedStatement> ClientContext::prepareWithParams(std::string_view query,
+    std::unordered_map<std::string, std::unique_ptr<Value>> inputParams) {
     std::unique_lock lck{mtx};
     auto parsedStatements = std::vector<std::shared_ptr<Statement>>();
     try {
@@ -298,7 +339,15 @@ std::unique_ptr<PreparedStatement> ClientContext::prepare(std::string_view query
         return preparedStatementWithError(
             "Connection Exception: We do not support prepare multiple statements.");
     }
-    auto result = prepareNoLock(parsedStatements[0], true /*shouldCommitNewTransaction*/);
+
+    // The binder deals with the parameter values as shared ptrs
+    // Copy the params to a new map that matches the format that the binder expects
+    std::unordered_map<std::string, std::shared_ptr<Value>> inputParamsTmp;
+    for (auto& [key, value] : inputParams) {
+        inputParamsTmp.insert(std::make_pair(key, std::make_shared<Value>(*value)));
+    }
+    auto result = prepareNoLock(parsedStatements[0], true /*shouldCommitNewTransaction*/,
+        std::move(inputParamsTmp));
     useInternalCatalogEntry_ = false;
     return result;
 }
@@ -356,8 +405,8 @@ std::unique_ptr<QueryResult> ClientContext::queryNoLock(std::string_view query,
         if (statement->isInternal()) {
             // The result of internal statements should be invisible to end users. Skip chaining the
             // result of internal statements to the final result to end users.
-            internalCompilingTime = currentQuerySummary->getCompilingTime();
-            internalExecutionTime = currentQuerySummary->getExecutionTime();
+            internalCompilingTime += currentQuerySummary->getCompilingTime();
+            internalExecutionTime += currentQuerySummary->getExecutionTime();
             continue;
         }
         currentQuerySummary->incrementCompilingTime(internalCompilingTime);
@@ -399,6 +448,7 @@ void ClientContext::bindParametersNoLock(const PreparedStatement* preparedStatem
         if (!parameterMap.contains(name)) {
             throw Exception("Parameter " + name + " not found.");
         }
+        preparedStatement->validateExecuteParam(name, value.get());
         auto expectParam = parameterMap.at(name);
         // The much more natural `parameterMap.at(name) = std::move(v)` fails.
         // The reason is that other parts of the code rely on the existing Value object to be
@@ -417,7 +467,7 @@ std::vector<std::shared_ptr<Statement>> ClientContext::parseQuery(std::string_vi
     auto parsedStatements = Parser::parseQuery(query);
     parserTimer.stop();
     const auto avgParsingTime = parserTimer.getElapsedTimeMS() / parsedStatements.size() / 1.0;
-    StandaloneCallRewriter standaloneCallAnalyzer{this};
+    StandaloneCallRewriter standaloneCallAnalyzer{this, parsedStatements.size() == 1};
     for (auto i = 0u; i < parsedStatements.size(); i++) {
         auto rewriteQuery = standaloneCallAnalyzer.getRewriteQuery(*parsedStatements[i]);
         if (rewriteQuery.empty()) {
@@ -447,11 +497,6 @@ void ClientContext::validateTransaction(const PreparedStatement& preparedStateme
     if (!canExecuteWriteQuery() && !preparedStatement.isReadOnly()) {
         throw ConnectionException("Cannot execute write operations in a read-only database!");
     }
-    // TODO(Guodong): Remove this check after we support COPY FROM in manual transactions.
-    if (preparedStatement.getStatementType() == StatementType::COPY_FROM &&
-        !transactionContext->isAutoTransaction()) {
-        throw ConnectionException("COPY FROM is only supported in auto transaction mode.");
-    }
     if (preparedStatement.parsedStatement->requireTransaction() &&
         transactionContext->hasActiveTransaction()) {
         KU_ASSERT(!transactionContext->isAutoTransaction());
@@ -467,7 +512,12 @@ std::unique_ptr<PreparedStatement> ClientContext::prepareNoLock(
     prepareTimer.start();
     try {
         preparedStatement->preparedSummary.statementType = parsedStatement->getStatementType();
-        preparedStatement->readOnly = StatementReadWriteAnalyzer().isReadOnly(*parsedStatement);
+        auto readWriteAnalyzer = StatementReadWriteAnalyzer(this);
+        TransactionHelper::runFuncInTransaction(
+            *transactionContext, [&]() -> void { readWriteAnalyzer.visit(*parsedStatement); },
+            true /* readOnly */, false /* */,
+            TransactionHelper::TransactionCommitAction::COMMIT_IF_NEW);
+        preparedStatement->readOnly = readWriteAnalyzer.isReadOnly();
         preparedStatement->parsedStatement = std::move(parsedStatement);
         validateTransaction(*preparedStatement);
 
@@ -479,16 +529,16 @@ std::unique_ptr<PreparedStatement> ClientContext::prepareNoLock(
                     binder.setInputParameters(*inputParams);
                 }
                 const auto boundStatement = binder.bind(*preparedStatement->parsedStatement);
+                binder.validateAllInputParametersParsed();
                 preparedStatement->parameterMap = binder.getParameterMap();
                 preparedStatement->statementResult = std::make_unique<BoundStatementResult>(
                     boundStatement->getStatementResult()->copy());
                 // planning
                 auto planner = Planner(this);
-                auto bestPlan = planner.getBestPlan(*boundStatement);
+                auto bestPlan = planner.planStatement(*boundStatement);
                 // optimizing
-                optimizer::Optimizer::optimize(bestPlan.get(), this,
-                    planner.getCardinalityEstimator());
-                preparedStatement->logicalPlan = std::move(bestPlan);
+                optimizer::Optimizer::optimize(&bestPlan, this, planner.getCardinalityEstimator());
+                preparedStatement->logicalPlan = std::make_unique<LogicalPlan>(bestPlan);
             },
             preparedStatement->isReadOnly(), preparedStatement->isTransactionStatement(),
             TransactionHelper::getAction(shouldCommitNewTransaction,
@@ -514,17 +564,12 @@ std::unique_ptr<QueryResult> ClientContext::executeNoLock(PreparedStatement* pre
     this->startTimer();
     auto executingTimer = TimeMetric(true /* enable */);
     executingTimer.start();
-    auto mapper = PlanMapper(this);
     std::shared_ptr<FactorizedTable> resultFT;
     std::unique_ptr<QueryResult> queryResult;
     try {
         TransactionHelper::runFuncInTransaction(
             *transactionContext,
             [&]() -> void {
-                const auto physicalPlan =
-                    mapper.mapLogicalPlanToPhysical(preparedStatement->logicalPlan.get(),
-                        preparedStatement->statementResult->getColumns());
-                queryResult = std::make_unique<QueryResult>(preparedStatement->preparedSummary);
                 const auto profiler = std::make_unique<Profiler>();
                 profiler->enabled = preparedStatement->isProfile();
                 if (!queryID) {
@@ -532,11 +577,19 @@ std::unique_ptr<QueryResult> ClientContext::executeNoLock(PreparedStatement* pre
                 }
                 const auto executionContext =
                     std::make_unique<ExecutionContext>(profiler.get(), this, *queryID);
+                auto mapper = PlanMapper(executionContext.get());
+                const auto physicalPlan =
+                    mapper.mapLogicalPlanToPhysical(preparedStatement->logicalPlan.get(),
+                        preparedStatement->statementResult->getColumns());
+                queryResult = std::make_unique<QueryResult>(preparedStatement->preparedSummary);
                 if (preparedStatement->isTransactionStatement()) {
                     resultFT = localDatabase->queryProcessor->execute(physicalPlan.get(),
                         executionContext.get());
                 } else {
-                    getTransaction()->checkForceCheckpoint(preparedStatement->getStatementType());
+                    if (preparedStatement->getStatementType() == StatementType::COPY_FROM) {
+                        // Note: We always force checkpoint for COPY_FROM statement.
+                        getTransaction()->setForceCheckpoint();
+                    }
                     resultFT = localDatabase->queryProcessor->execute(physicalPlan.get(),
                         executionContext.get());
                 }
@@ -582,7 +635,7 @@ ClientContext::TransactionHelper::getAction(bool commitIfNew, bool commitIfAuto)
     return TransactionCommitAction::NOT_COMMIT;
 }
 
-// If there is an active transaction in the context, we execute the function in current active
+// If there is an active transaction in the context, we execute the function in the current active
 // transaction. If there is no active transaction, we start an auto commit transaction.
 void ClientContext::TransactionHelper::runFuncInTransaction(TransactionContext& context,
     const std::function<void()>& fun, bool readOnlyStatement, bool isTransactionStatement,
@@ -616,7 +669,7 @@ bool ClientContext::canExecuteWriteQuery() const {
     // remote kuzu database can be attached.
     const auto dbManager = localDatabase->databaseManager.get();
     for (const auto& attachedDB : dbManager->getAttachedDatabases()) {
-        if (attachedDB->getDBType() == common::ATTACHED_KUZU_DB_TYPE) {
+        if (attachedDB->getDBType() == ATTACHED_KUZU_DB_TYPE) {
             return false;
         }
     }
