@@ -1,10 +1,10 @@
 #include "main/settings.h"
 
 #include "common/exception/runtime.h"
-#include "common/file_system/virtual_file_system.h"
 #include "main/client_context.h"
 #include "storage/buffer_manager/buffer_manager.h"
 #include "storage/buffer_manager/memory_manager.h"
+#include "storage/storage_utils.h"
 
 namespace kuzu {
 namespace main {
@@ -15,7 +15,7 @@ void SpillToDiskSetting::setContext(ClientContext* context, const common::Value&
     const auto& dbConfig = *context->getDBConfig();
     std::string spillPath;
     if (dbConfig.enableSpillingToDisk) {
-        if (dbConfig.isDBPathInMemory(context->getDatabasePath())) {
+        if (context->isInMemory()) {
             throw common::RuntimeException(
                 "Cannot set spill_to_disk to true for an in-memory database!");
         }
@@ -23,8 +23,7 @@ void SpillToDiskSetting::setContext(ClientContext* context, const common::Value&
             throw common::RuntimeException(
                 "Cannot set spill_to_disk to true for a read only database!");
         }
-        spillPath = context->getVFSUnsafe()->joinPath(context->getDatabasePath(),
-            common::StorageConstants::TEMP_SPILLING_FILE_NAME);
+        spillPath = storage::StorageUtils::getTmpFilePath(context->getDatabasePath());
     } else {
         // Set path to empty will disable spiller.
         spillPath = "";
