@@ -14,12 +14,17 @@ std::unique_ptr<ParsedFunctionExpression> ParsedFunctionExpression::deserialize(
     deserializer.deserializeValue(isDistinct);
     std::string functionName;
     deserializer.deserializeValue(functionName);
-    return std::make_unique<ParsedFunctionExpression>(std::move(functionName), isDistinct);
+    std::vector<std::string> optionalArguments;
+    deserializer.deserializeVector(optionalArguments);
+    auto result = std::make_unique<ParsedFunctionExpression>(std::move(functionName), isDistinct);
+    result->setOptionalArguments(std::move(optionalArguments));
+    return result;
 }
 
 void ParsedFunctionExpression::serializeInternal(Serializer& serializer) const {
     serializer.serializeValue(isDistinct);
     serializer.serializeValue(functionName);
+    serializer.serializeVector(optionalArguments);
 }
 
 } // namespace parser
