@@ -215,8 +215,8 @@ std::unique_ptr<ParsedExpression> Transformer::transformMultiplyDivideModuloExpr
 std::unique_ptr<ParsedExpression> Transformer::transformPowerOfExpression(
     CypherParser::OC_PowerOfExpressionContext& ctx) {
     std::unique_ptr<ParsedExpression> expression;
-    for (auto& unaryAddOrSubtractExpression : ctx.oC_UnaryAddSubtractOrFactorialExpression()) {
-        auto next = transformUnaryAddSubtractOrFactorialExpression(*unaryAddOrSubtractExpression);
+    for (auto& stringListNullOperatorExpression : ctx.oC_StringListNullOperatorExpression()) {
+        auto next = transformStringListNullOperatorExpression(*stringListNullOperatorExpression);
         if (!expression) {
             expression = std::move(next);
         } else {
@@ -230,9 +230,8 @@ std::unique_ptr<ParsedExpression> Transformer::transformPowerOfExpression(
 
 std::unique_ptr<ParsedExpression> Transformer::transformUnaryAddSubtractOrFactorialExpression(
     CypherParser::OC_UnaryAddSubtractOrFactorialExpressionContext& ctx) {
-    auto result =
-        transformStringListNullOperatorExpression(*ctx.oC_StringListNullOperatorExpression());
-    if (ctx.FACTORIAL()) { // Factorial has a higher
+    auto result = transformPropertyOrLabelsExpression(*ctx.oC_PropertyOrLabelsExpression());
+    if (ctx.FACTORIAL()) { // Factorial has a higher precedence
         auto raw = result->toString() + "!";
         result = std::make_unique<ParsedFunctionExpression>(FactorialFunction::name,
             std::move(result), std::move(raw));
@@ -249,15 +248,15 @@ std::unique_ptr<ParsedExpression> Transformer::transformUnaryAddSubtractOrFactor
 
 std::unique_ptr<ParsedExpression> Transformer::transformStringListNullOperatorExpression(
     CypherParser::OC_StringListNullOperatorExpressionContext& ctx) {
-    auto propertyExpression =
-        transformPropertyOrLabelsExpression(*ctx.oC_PropertyOrLabelsExpression());
+    auto unaryAddSubtractOrFactorialExpression = transformUnaryAddSubtractOrFactorialExpression(
+        *ctx.oC_UnaryAddSubtractOrFactorialExpression());
     if (ctx.oC_NullOperatorExpression()) {
         return transformNullOperatorExpression(*ctx.oC_NullOperatorExpression(),
-            std::move(propertyExpression));
+            std::move(unaryAddSubtractOrFactorialExpression));
     }
     if (!ctx.oC_ListOperatorExpression().empty()) {
         auto result = transformListOperatorExpression(*ctx.oC_ListOperatorExpression(0),
-            std::move(propertyExpression));
+            std::move(unaryAddSubtractOrFactorialExpression));
         for (auto i = 1u; i < ctx.oC_ListOperatorExpression().size(); ++i) {
             result = transformListOperatorExpression(*ctx.oC_ListOperatorExpression(i),
                 std::move(result));
@@ -266,9 +265,9 @@ std::unique_ptr<ParsedExpression> Transformer::transformStringListNullOperatorEx
     }
     if (ctx.oC_StringOperatorExpression()) {
         return transformStringOperatorExpression(*ctx.oC_StringOperatorExpression(),
-            std::move(propertyExpression));
+            std::move(unaryAddSubtractOrFactorialExpression));
     }
-    return propertyExpression;
+    return unaryAddSubtractOrFactorialExpression;
 }
 
 std::unique_ptr<ParsedExpression> Transformer::transformStringOperatorExpression(
@@ -313,15 +312,15 @@ std::unique_ptr<ParsedExpression> Transformer::transformListOperatorExpression(
             left = transformExpression(*ctx.oC_Expression(0));
             right = transformExpression(*ctx.oC_Expression(1));
         } else if (ctx.oC_Expression().size() == 0) { // [:]
-            left = std::make_unique<ParsedLiteralExpression>(Value(0), "0");
-            right = std::make_unique<ParsedLiteralExpression>(Value(0), "0");
+            left = std::make_unique<ParsedLiteralExpression>(Value(1), "1");
+            right = std::make_unique<ParsedLiteralExpression>(Value(-1), "-1");
         } else {
             if (ctx.children[1]->getText() == ":") { // [:right]
-                left = std::make_unique<ParsedLiteralExpression>(Value(0), "0");
+                left = std::make_unique<ParsedLiteralExpression>(Value(1), "1");
                 right = transformExpression(*ctx.oC_Expression(0));
             } else { // [left:]
                 left = transformExpression(*ctx.oC_Expression(0));
-                right = std::make_unique<ParsedLiteralExpression>(Value(0), "0");
+                right = std::make_unique<ParsedLiteralExpression>(Value(-1), "-1");
             }
         }
         listSlice->addChild(std::move(left));
@@ -376,10 +375,8 @@ std::unique_ptr<ParsedExpression> Transformer::transformAtom(CypherParser::OC_At
         return transformFunctionInvocation(*ctx.oC_FunctionInvocation());
     } else if (ctx.oC_PathPatterns()) {
         return transformPathPattern(*ctx.oC_PathPatterns());
-    } else if (ctx.oC_ExistSubquery()) {
-        return transformExistSubquery(*ctx.oC_ExistSubquery());
-    } else if (ctx.kU_CountSubquery()) {
-        return transformCountSubquery(*ctx.kU_CountSubquery());
+    } else if (ctx.oC_ExistCountSubquery()) {
+        return transformExistCountSubquery(*ctx.oC_ExistCountSubquery());
     } else if (ctx.oC_Quantifier()) {
         return transformOcQuantifier(*ctx.oC_Quantifier());
     } else {
@@ -553,22 +550,16 @@ std::unique_ptr<ParsedExpression> Transformer::transformPathPattern(
     return subquery;
 }
 
-std::unique_ptr<ParsedExpression> Transformer::transformExistSubquery(
-    CypherParser::OC_ExistSubqueryContext& ctx) {
-    auto subquery = std::make_unique<ParsedSubqueryExpression>(SubqueryType::EXISTS, ctx.getText());
+std::unique_ptr<ParsedExpression> Transformer::transformExistCountSubquery(
+    CypherParser::OC_ExistCountSubqueryContext& ctx) {
+    auto type = ctx.EXISTS() ? SubqueryType::EXISTS : SubqueryType::COUNT;
+    auto subquery = std::make_unique<ParsedSubqueryExpression>(type, ctx.getText());
     subquery->setPatternElements(transformPattern(*ctx.oC_Pattern()));
     if (ctx.oC_Where()) {
         subquery->setWhereClause(transformWhere(*ctx.oC_Where()));
     }
-    return subquery;
-}
-
-std::unique_ptr<ParsedExpression> Transformer::transformCountSubquery(
-    CypherParser::KU_CountSubqueryContext& ctx) {
-    auto subquery = std::make_unique<ParsedSubqueryExpression>(SubqueryType::COUNT, ctx.getText());
-    subquery->setPatternElements(transformPattern(*ctx.oC_Pattern()));
-    if (ctx.oC_Where()) {
-        subquery->setWhereClause(transformWhere(*ctx.oC_Where()));
+    if (ctx.kU_Hint()) {
+        subquery->setHint(transformJoinHint(*ctx.kU_Hint()->kU_JoinNode()));
     }
     return subquery;
 }
