@@ -2,6 +2,10 @@
 
 #include "binder/binder.h"
 #include "binder/bound_standalone_call_function.h"
+#include "catalog/catalog.h"
+#include "common/exception/parser.h"
+#include "main/client_context.h"
+#include "parser/expression/parsed_function_expression.h"
 #include "parser/standalone_call_function.h"
 
 namespace kuzu {
@@ -17,6 +21,14 @@ void StandaloneCallRewriter::visitStandaloneCallFunction(const Statement& statem
     main::ClientContext::TransactionHelper::runFuncInTransaction(
         *context->getTransactionContext(),
         [&]() -> void {
+            auto funcName = standaloneCallFunc.getFunctionExpression()
+                                ->constPtrCast<ParsedFunctionExpression>()
+                                ->getFunctionName();
+            if (!context->getCatalog()->containsFunction(context->getTransaction(), funcName) &&
+                !singleStatement) {
+                throw common::ParserException{
+                    funcName + " must be called in a query which doesn't have other statements."};
+            }
             binder::Binder binder{context};
             const auto boundStatement = binder.bind(standaloneCallFunc);
             auto& boundStandaloneCall =
