@@ -1,5 +1,7 @@
 #include "parser/visitor/statement_read_write_analyzer.h"
 
+#include "main/client_context.h"
+#include "main/db_config.h"
 #include "parser/expression/parsed_expression_visitor.h"
 #include "parser/query/reading_clause/reading_clause.h"
 #include "parser/query/return_with_clause/with_clause.h"
@@ -7,20 +9,18 @@
 namespace kuzu {
 namespace parser {
 
-bool StatementReadWriteAnalyzer::isReadOnly(const Statement& statement) {
-    visit(statement);
-    return readOnly;
-}
-
-static bool hasSequenceUpdate(const ParsedExpression* expr) {
-    auto collector = ParsedSequenceFunctionCollector();
-    collector.visit(expr);
-    return collector.hasSeqUpdate();
+void StatementReadWriteAnalyzer::visitExtension(const Statement& /*statement*/) {
+    // We allow LOAD EXTENSION to run in read-only mode.
+    if (context->getDBConfig()->readOnly) {
+        readOnly = true;
+    } else {
+        readOnly = false;
+    }
 }
 
 void StatementReadWriteAnalyzer::visitReadingClause(const ReadingClause* readingClause) {
     if (readingClause->hasWherePredicate()) {
-        if (hasSequenceUpdate(readingClause->getWherePredicate())) {
+        if (!isExprReadOnly(readingClause->getWherePredicate())) {
             readOnly = false;
         }
     }
@@ -28,7 +28,7 @@ void StatementReadWriteAnalyzer::visitReadingClause(const ReadingClause* reading
 
 void StatementReadWriteAnalyzer::visitWithClause(const WithClause* withClause) {
     for (auto& expr : withClause->getProjectionBody()->getProjectionExpressions()) {
-        if (hasSequenceUpdate(expr.get())) {
+        if (!isExprReadOnly(expr.get())) {
             readOnly = false;
             return;
         }
@@ -37,12 +37,18 @@ void StatementReadWriteAnalyzer::visitWithClause(const WithClause* withClause) {
 
 void StatementReadWriteAnalyzer::visitReturnClause(const ReturnClause* returnClause) {
     for (auto& expr : returnClause->getProjectionBody()->getProjectionExpressions()) {
-        if (hasSequenceUpdate(expr.get())) {
+        if (!isExprReadOnly(expr.get())) {
             readOnly = false;
             return;
         }
     }
 }
 
+bool StatementReadWriteAnalyzer::isExprReadOnly(const ParsedExpression* expr) {
+    auto analyzer = ReadWriteExprAnalyzer(context);
+    analyzer.visit(expr);
+    return analyzer.isReadOnly();
+}
+
 } // namespace parser
 } // namespace kuzu
