@@ -22,11 +22,6 @@ void LogicalScanNodeTable::computeFactorizedSchema() {
         schema->insertToGroupAndScope(property, groupPos);
     }
     switch (scanType) {
-    case LogicalScanNodeTableType::OFFSET_SCAN: {
-        schema->setGroupAsSingleState(groupPos);
-        auto recursiveJoinInfo = extraInfo->constCast<RecursiveJoinScanInfo>();
-        schema->insertToGroupAndScope(recursiveJoinInfo.nodePredicateExecFlag, groupPos);
-    } break;
     case LogicalScanNodeTableType::PRIMARY_KEY_SCAN: {
         schema->setGroupAsSingleState(groupPos);
     } break;
@@ -42,10 +37,6 @@ void LogicalScanNodeTable::computeFlatSchema() {
     for (auto& property : properties) {
         schema->insertToGroupAndScope(property, 0);
     }
-    if (scanType == LogicalScanNodeTableType::OFFSET_SCAN) {
-        auto recursiveJoinInfo = extraInfo->constCast<RecursiveJoinScanInfo>();
-        schema->insertToGroupAndScope(recursiveJoinInfo.nodePredicateExecFlag, 0);
-    }
 }
 
 std::unique_ptr<LogicalOperator> LogicalScanNodeTable::copy() {
