@@ -12,7 +12,9 @@ std::unique_ptr<PhysicalOperator> PlanMapper::mapTableFunctionCall(
     auto& call = logicalOperator->constCast<LogicalTableFunctionCall>();
     auto getPhysicalPlanFunc = call.getTableFunc().getPhysicalPlanFunc;
     KU_ASSERT(getPhysicalPlanFunc);
-    return getPhysicalPlanFunc(clientContext, this, logicalOperator);
+    auto res = getPhysicalPlanFunc(this, logicalOperator);
+    logicalOpToPhysicalOpMap.insert({logicalOperator, res.get()});
+    return res;
 }
 
 } // namespace processor
