@@ -5,10 +5,10 @@ namespace processor {
 
 void SimpleAggregateScan::initLocalStateInternal(ResultSet* resultSet, ExecutionContext* context) {
     BaseAggregateScan::initLocalStateInternal(resultSet, context);
-    KU_ASSERT(!aggregatesPos.empty());
-    auto outDataChunkPos = aggregatesPos[0].dataChunkPos;
+    KU_ASSERT(!scanInfo.aggregatesPos.empty());
+    auto outDataChunkPos = scanInfo.aggregatesPos[0].dataChunkPos;
     RUNTIME_CHECK({
-        for (auto& dataPos : aggregatesPos) {
+        for (auto& dataPos : scanInfo.aggregatesPos) {
             KU_ASSERT(dataPos.dataChunkPos == outDataChunkPos);
         }
     });
@@ -23,10 +23,10 @@ bool SimpleAggregateScan::getNextTuplesInternal(ExecutionContext* /*context*/) {
     // Output of simple aggregate is guaranteed to be a single value for each aggregate.
     KU_ASSERT(startOffset == 0 && endOffset == 1);
     for (auto i = 0u; i < aggregateVectors.size(); i++) {
-        writeAggregateResultToVector(*aggregateVectors[i], 0 /* position to write */,
+        scanInfo.moveAggResultToVectorFuncs[i](*aggregateVectors[i], 0 /* position to write */,
             sharedState->getAggregateState(i));
     }
-    KU_ASSERT(!aggregatesPos.empty());
+    KU_ASSERT(!scanInfo.aggregatesPos.empty());
     outDataChunk->state->initOriginalAndSelectedSize(1);
     metrics->numOutputTuple.increase(outDataChunk->state->getSelVector().getSelSize());
     return true;
