@@ -13,13 +13,11 @@ std::string CreateTypePrintInfo::toString() const {
     return typeName + " AS " + type;
 }
 
-void CreateType::executeDDLInternal(ExecutionContext* context) {
-    auto catalog = context->clientContext->getCatalog();
-    catalog->createType(context->clientContext->getTransaction(), name, type.copy());
-}
-
-std::string CreateType::getOutputMsg() {
-    return stringFormat("Type {}({}) has been created.", name, type.toString());
+void CreateType::executeInternal(ExecutionContext* context) {
+    auto clientContext = context->clientContext;
+    clientContext->getCatalog()->createType(clientContext->getTransaction(), name, type.copy());
+    appendMessage(stringFormat("Type {}({}) has been created.", name, type.toString()),
+        clientContext->getMemoryManager());
 }
 
 } // namespace processor
