@@ -48,11 +48,11 @@ BaseCSVReader::BaseCSVReader(const std::string& filePath, common::idx_t fileIdx,
       bufferIdx(0), bufferSize{0}, position{0}, lineContext(), osFileOffset{0}, fileIdx(fileIdx),
       errorHandler(errorHandler), rowEmpty{false} {
     fileInfo = context->getVFSUnsafe()->openFile(filePath,
-        FileFlags::READ_ONLY
+        FileOpenFlags(FileFlags::READ_ONLY
 #ifdef _WIN32
-            | FileFlags::BINARY
+                      | FileFlags::BINARY
 #endif
-        ,
+            ),
         context);
 }
 
@@ -201,13 +201,20 @@ bool BaseCSVReader::readBuffer(uint64_t* start) {
     return readCount > 0;
 }
 
-std::string BaseCSVReader::reconstructLine(uint64_t startPosition, uint64_t endPosition) {
+std::string BaseCSVReader::reconstructLine(uint64_t startPosition, uint64_t endPosition,
+    bool completeLine) {
     KU_ASSERT(endPosition >= startPosition);
 
     std::string res;
-    res.resize(endPosition - startPosition);
+    // For cases where we cannot perform a seek (e.g. compressed file system) we just return an
+    // empty string
+    if (fileInfo->canPerformSeek()) {
+        res.resize(endPosition - startPosition);
+        fileInfo->readFromFile(res.data(), res.size(), startPosition);
 
-    (void)fileInfo->readFromFile(res.data(), res.size(), startPosition);
+        const char* incompleteLineSuffix = completeLine ? "" : "...";
+        res += incompleteLineSuffix;
+    }
 
     return StringUtils::ltrimNewlines(StringUtils::rtrimNewlines(res));
 }
@@ -275,7 +282,7 @@ BaseCSVReader::parse_result_t BaseCSVReader::parseCSV(Driver& driver) {
 
     while (true) {
         column_id_t column = 0;
-        auto start = position;
+        auto start = position.load();
         bool hasQuotes = false;
         std::vector<uint64_t> escapePositions;
         lineContext.setNewLine(getFileOffset());
@@ -545,16 +552,14 @@ column_id_t BaseCSVReader::appendWarningDataColumns(std::vector<std::string>& re
 
 PopulatedCopyFromError BaseCSVReader::basePopulateErrorFunc(CopyFromFileError error,
     const SharedFileErrorHandler* sharedErrorHandler, BaseCSVReader* reader, std::string filePath) {
-    const char* incompleteLineSuffix = error.completedLine ? "" : "...";
     const auto warningData = CSVWarningSourceData::constructFrom(error.warningData);
     const auto lineNumber =
         sharedErrorHandler->getLineNumber(warningData.blockIdx, warningData.offsetInBlock);
     return PopulatedCopyFromError{
         .message = std::move(error.message),
         .filePath = std::move(filePath),
-        .skippedLineOrRecord =
-            reader->reconstructLine(warningData.startByteOffset, warningData.endByteOffset) +
-            incompleteLineSuffix,
+        .skippedLineOrRecord = reader->reconstructLine(warningData.startByteOffset,
+            warningData.endByteOffset, error.completedLine),
         .lineNumber = lineNumber,
     };
 }
