@@ -3,6 +3,7 @@
 #include "binder/binder.h"
 #include "function/table/bind_data.h"
 #include "function/table/table_function.h"
+#include "main/client_context.h"
 #include "processor/execution_context.h"
 #include "processor/operator/persistent/reader/csv/driver.h"
 #include "processor/operator/persistent/reader/reader_bind_utils.h"
@@ -73,7 +74,7 @@ uint64_t SerialCSVReader::parseBlock(block_idx_t blockIdx, DataChunk& resultChun
     }
     SerialParsingDriver driver(resultChunk, this);
     const auto [numRowsRead, numErrors] = parseCSV(driver);
-    errorHandler->reportFinishedBlock(blockIdx, numRowsRead);
+    errorHandler->reportFinishedBlock(blockIdx, numRowsRead + numErrors);
     resultChunk.state->getSelVectorUnsafe().setSelSize(numRowsRead);
     increaseNumRowsInCurrentBlock(numRowsRead, numErrors);
     return numRowsRead;
@@ -81,9 +82,10 @@ uint64_t SerialCSVReader::parseBlock(block_idx_t blockIdx, DataChunk& resultChun
 
 SerialCSVScanSharedState::SerialCSVScanSharedState(FileScanInfo fileScanInfo, uint64_t numRows,
     main::ClientContext* context, CSVOption csvOption, CSVColumnInfo columnInfo, uint64_t queryID)
-    : ScanFileSharedState{std::move(fileScanInfo), numRows, context},
+    : ScanFileWithProgressSharedState{std::move(fileScanInfo), numRows, context},
       csvOption{std::move(csvOption)}, columnInfo{std::move(columnInfo)}, totalReadSizeByFile{0},
       queryID(queryID), populateErrorFunc(constructPopulateFunc()) {
+    std::lock_guard lck{mtx};
     initReader(context);
 }
 
@@ -95,7 +97,7 @@ populate_func_t SerialCSVScanSharedState::constructPopulateFunc() const {
 }
 
 void SerialCSVScanSharedState::read(DataChunk& outputChunk) {
-    std::lock_guard<std::mutex> mtx{lock};
+    std::lock_guard<std::mutex> lck{mtx};
     do {
         if (fileIdx >= fileScanInfo.getNumFiles()) {
             return;
@@ -232,18 +234,19 @@ static std::unique_ptr<TableFuncBindData> bindFunc(main::ClientContext* context,
     for (auto& column : warningColumns) {
         resultColumns.push_back(column);
     }
-    return std::make_unique<ScanBindData>(std::move(resultColumns), scanInput->fileScanInfo.copy(),
-        context, numWarningDataColumns, 0 /* estCardinality */);
+    return std::make_unique<ScanFileBindData>(std::move(resultColumns), 0 /* numRows */,
+        scanInput->fileScanInfo.copy(), context, numWarningDataColumns);
 }
 
-static std::unique_ptr<TableFuncSharedState> initSharedState(const TableFunctionInitInput& input) {
-    auto bindData = input.bindData->constPtrCast<ScanBindData>();
+static std::unique_ptr<TableFuncSharedState> initSharedState(
+    const TableFuncInitSharedStateInput& input) {
+    auto bindData = input.bindData->constPtrCast<ScanFileBindData>();
     auto csvOption = CSVReaderConfig::construct(bindData->fileScanInfo.options).option;
-    row_idx_t numRows = 0;
     auto columnInfo = CSVColumnInfo(bindData->getNumColumns() - bindData->numWarningDataColumns,
         bindData->getColumnSkips(), bindData->numWarningDataColumns);
-    auto sharedState = std::make_unique<SerialCSVScanSharedState>(bindData->fileScanInfo.copy(),
-        numRows, bindData->context, csvOption.copy(), columnInfo.copy(), input.queryID);
+    auto sharedState =
+        std::make_unique<SerialCSVScanSharedState>(bindData->fileScanInfo.copy(), 0 /* numRows */,
+            bindData->context, csvOption.copy(), columnInfo.copy(), input.context->queryID);
     for (idx_t i = 0; i < sharedState->fileScanInfo.filePaths.size(); ++i) {
         const auto& filePath = sharedState->fileScanInfo.filePaths[i];
         auto reader = std::make_unique<SerialCSVReader>(filePath, i, csvOption.copy(),
@@ -253,11 +256,6 @@ static std::unique_ptr<TableFuncSharedState> initSharedState(const TableFunction
     return sharedState;
 }
 
-static std::unique_ptr<TableFuncLocalState> initLocalState(const TableFunctionInitInput&,
-    const TableFuncSharedState*, storage::MemoryManager*) {
-    return std::make_unique<TableFuncLocalState>();
-}
-
 static void finalizeFunc(const ExecutionContext* ctx, TableFuncSharedState* sharedState) {
     auto state = ku_dynamic_cast<SerialCSVScanSharedState*>(sharedState);
     state->finalizeReader(ctx->clientContext);
@@ -270,6 +268,7 @@ static double progressFunc(TableFuncSharedState* sharedState) {
     } else if (state->fileIdx >= state->fileScanInfo.getNumFiles()) {
         return 1.0;
     }
+    std::lock_guard lck{state->mtx};
     uint64_t totalReadSize = state->totalReadSizeByFile + state->reader->getFileOffset();
     return static_cast<double>(totalReadSize) / state->totalSize;
 }
@@ -280,7 +279,7 @@ function_set SerialCSVScan::getFunctionSet() {
     function->tableFunc = tableFunc;
     function->bindFunc = bindFunc;
     function->initSharedStateFunc = initSharedState;
-    function->initLocalStateFunc = initLocalState;
+    function->initLocalStateFunc = TableFunction::initEmptyLocalState;
     function->progressFunc = progressFunc;
     function->finalizeFunc = finalizeFunc;
     functionSet.push_back(std::move(function));
@@ -289,12 +288,7 @@ function_set SerialCSVScan::getFunctionSet() {
 
 void SerialCSVReader::resetReaderState() {
     // Reset file position to the beginning.
-    if (-1 == fileInfo->seek(0, SEEK_SET)) {
-        handleCopyException(
-            stringFormat("Failed to seek to the beginning of the file:, errno: {}.", errno), true);
-        return;
-    }
-
+    fileInfo->reset();
     buffer.reset();
     bufferSize = 0;
     position = 0;
