@@ -35,7 +35,8 @@ void ParquetReader::initializeScan(ParquetReaderScanState& state,
     state.groupIdxList = std::move(groups_to_read);
     if (!state.fileInfo || state.fileInfo->path != filePath) {
         state.prefetchMode = false;
-        state.fileInfo = vfs->openFile(filePath, FileFlags::READ_ONLY, context);
+        state.fileInfo =
+            vfs->openFile(filePath, common::FileOpenFlags(FileFlags::READ_ONLY), context);
     }
 
     state.thriftFileProto = createThriftProtocol(state.fileInfo.get(), state.prefetchMode);
@@ -170,7 +171,8 @@ void ParquetReader::scan(processor::ParquetReaderScanState& state, DataChunk& re
 }
 
 void ParquetReader::initMetadata() {
-    auto fileInfo = context->getVFSUnsafe()->openFile(filePath, FileFlags::READ_ONLY, context);
+    auto fileInfo =
+        context->getVFSUnsafe()->openFile(filePath, FileOpenFlags(FileFlags::READ_ONLY), context);
     auto proto = createThriftProtocol(fileInfo.get(), false);
     auto& transport = ku_dynamic_cast<ThriftFileTransport&>(*proto->getTransport());
     auto fileSize = transport.GetSize();
@@ -577,11 +579,12 @@ uint64_t ParquetReader::getGroupOffset(ParquetReaderScanState& state) {
 
 ParquetScanSharedState::ParquetScanSharedState(FileScanInfo fileScanInfo, uint64_t numRows,
     main::ClientContext* context, std::vector<bool> columnSkips)
-    : ScanFileSharedState{std::move(fileScanInfo), numRows, context}, columnSkips{columnSkips} {
+    : ScanFileWithProgressSharedState{std::move(fileScanInfo), numRows, context},
+      columnSkips{columnSkips} {
     readers.push_back(std::make_unique<ParquetReader>(this->fileScanInfo.filePaths[fileIdx],
         columnSkips, context));
     totalRowsGroups = 0;
-    for (auto i = fileIdx; i < this->fileScanInfo.getNumFiles(); i++) {
+    for (auto i = fileIdx.load(); i < this->fileScanInfo.getNumFiles(); i++) {
         auto reader =
             std::make_unique<ParquetReader>(this->fileScanInfo.filePaths[i], columnSkips, context);
         totalRowsGroups += reader->getNumRowsGroups();
@@ -591,7 +594,7 @@ ParquetScanSharedState::ParquetScanSharedState(FileScanInfo fileScanInfo, uint64
 
 static bool parquetSharedStateNext(ParquetScanLocalState& localState,
     ParquetScanSharedState& sharedState) {
-    std::lock_guard<std::mutex> mtx{sharedState.lock};
+    std::lock_guard<std::mutex> mtx{sharedState.mtx};
     while (true) {
         if (sharedState.fileIdx >= sharedState.fileScanInfo.getNumFiles()) {
             return false;
@@ -662,11 +665,12 @@ static void bindColumns(const ExtraScanTableFuncBindInput* bindInput,
     }
 }
 
-static row_idx_t getNumRows(const ScanBindData* bindData) {
+static row_idx_t getNumRows(std::vector<std::string> filePaths, uint64_t numColumns,
+    main::ClientContext* context) {
+    std::vector<bool> dummyColumnSkips(false, numColumns);
     row_idx_t numRows = 0;
-    for (const auto& path : bindData->fileScanInfo.filePaths) {
-        auto reader =
-            std::make_unique<ParquetReader>(path, bindData->getColumnSkips(), bindData->context);
+    for (const auto& path : filePaths) {
+        auto reader = std::make_unique<ParquetReader>(path, dummyColumnSkips, context);
         numRows += reader->getMetadata()->num_rows;
     }
     return numRows;
@@ -692,21 +696,22 @@ static std::unique_ptr<TableFuncBindData> bindFunc(main::ClientContext* context,
     detectedColumnNames =
         TableFunction::extractYieldVariables(detectedColumnNames, input->yieldVariables);
     auto resultColumns = input->binder->createVariables(detectedColumnNames, detectedColumnTypes);
-    auto bindData = std::make_unique<ScanBindData>(std::move(resultColumns),
+    auto bindData = std::make_unique<ScanFileBindData>(std::move(resultColumns),
+        getNumRows(scanInput->fileScanInfo.filePaths, detectedColumnNames.size(), context),
         scanInput->fileScanInfo.copy(), context);
-    bindData->cardinality = getNumRows(bindData.get());
     return bindData;
 }
 
-static std::unique_ptr<TableFuncSharedState> initSharedState(const TableFunctionInitInput& input) {
-    auto bindData = input.bindData->constPtrCast<ScanBindData>();
+static std::unique_ptr<TableFuncSharedState> initSharedState(
+    const TableFuncInitSharedStateInput& input) {
+    auto bindData = input.bindData->constPtrCast<ScanFileBindData>();
     return std::make_unique<ParquetScanSharedState>(bindData->fileScanInfo.copy(),
-        getNumRows(bindData), bindData->context, bindData->getColumnSkips());
+        bindData->numRows, bindData->context, bindData->getColumnSkips());
 }
 
-static std::unique_ptr<TableFuncLocalState> initLocalState(const TableFunctionInitInput&,
-    TableFuncSharedState* state, storage::MemoryManager*) {
-    auto sharedState = state->ptrCast<ParquetScanSharedState>();
+static std::unique_ptr<TableFuncLocalState> initLocalState(
+    const TableFuncInitLocalStateInput& input) {
+    auto sharedState = input.sharedState.ptrCast<ParquetScanSharedState>();
     auto localState = std::make_unique<ParquetScanLocalState>();
     if (!parquetSharedStateNext(*localState, *sharedState)) {
         return nullptr;
