@@ -321,8 +321,8 @@ void ColumnWriter::handleDefineLevels(ColumnWriterState& state, ColumnWriterStat
     }
 }
 
-void ColumnWriter::compressPage(common::BufferedSerializer& bufferedSerializer,
-    size_t& compressedSize, uint8_t*& compressedData, std::unique_ptr<uint8_t[]>& compressedBuf) {
+void ColumnWriter::compressPage(common::BufferWriter& bufferedSerializer, size_t& compressedSize,
+    uint8_t*& compressedData, std::unique_ptr<uint8_t[]>& compressedBuf) {
     switch (writer.getCodec()) {
     case CompressionCodec::UNCOMPRESSED: {
         compressedSize = bufferedSerializer.getSize();
