@@ -12,16 +12,14 @@ std::string DetatchDatabasePrintInfo::toString() const {
 }
 
 void DetachDatabase::executeInternal(ExecutionContext* context) {
-    auto dbManager = context->clientContext->getDatabaseManager();
+    auto clientContext = context->clientContext;
+    auto dbManager = clientContext->getDatabaseManager();
     if (dbManager->hasAttachedDatabase(dbName) &&
         dbManager->getAttachedDatabase(dbName)->getDBType() == common::ATTACHED_KUZU_DB_TYPE) {
-        context->clientContext->setDefaultDatabase(nullptr /* defaultDatabase */);
+        clientContext->setDefaultDatabase(nullptr /* defaultDatabase */);
     }
     dbManager->detachDatabase(dbName);
-}
-
-std::string DetachDatabase::getOutputMsg() {
-    return "Detached database successfully.";
+    appendMessage("Detached database successfully.", clientContext->getMemoryManager());
 }
 
 } // namespace processor
