@@ -15,11 +15,10 @@ std::string LoadExtensionPrintInfo::toString() const {
 }
 
 void LoadExtension::executeInternal(ExecutionContext* context) {
-    context->clientContext->getExtensionManager()->loadExtension(path, context->clientContext);
-}
-
-std::string LoadExtension::getOutputMsg() {
-    return stringFormat("Extension: {} has been loaded.", path);
+    auto clientContext = context->clientContext;
+    clientContext->getExtensionManager()->loadExtension(path, clientContext);
+    appendMessage(stringFormat("Extension: {} has been loaded.", path),
+        clientContext->getMemoryManager());
 }
 
 } // namespace processor
