@@ -1,9 +1,11 @@
 #include "storage/buffer_manager/buffer_manager.h"
 
 #include <atomic>
+#include <chrono>
 #include <cstdint>
 #include <cstring>
 #include <memory>
+#include <thread>
 
 #include "common/assert.h"
 #include "common/constants.h"
@@ -14,7 +16,8 @@
 #include "main/db_config.h"
 #include "storage/buffer_manager/spiller.h"
 #include "storage/file_handle.h"
-#include "storage/store/column_chunk_data.h"
+#include "storage/table/column_chunk_data.h"
+#include <span>
 
 #if defined(_WIN32)
 #include <exception>
@@ -46,28 +49,9 @@ bool EvictionQueue::insert(uint32_t fileIndex, page_idx_t pageIndex) {
     return false;
 }
 
-std::atomic<EvictionCandidate>* EvictionQueue::next() {
-    std::atomic<EvictionCandidate>* candidate = nullptr;
-    do {
-        // Since the buffer pool size is a power of two (as is the page size), (UINT64_MAX + 1) %
-        // size == 0, which means no entries will be skipped when the cursor overflows
-        candidate = &data[evictionCursor.fetch_add(1, std::memory_order_relaxed) % capacity];
-    } while (candidate->load() == EMPTY && size > 0);
-    return candidate;
-}
-
-void EvictionQueue::removeCandidatesForFile(uint32_t fileIndex) {
-    if (size == 0) {
-        return;
-    }
-    for (uint64_t i = 0; i < capacity; i++) {
-        auto candidate = data[i].load();
-        if (candidate.fileIdx == fileIndex && data[i].compare_exchange_strong(candidate, EMPTY)) {
-            if (size-- == 1) {
-                return;
-            }
-        }
-    }
+std::span<std::atomic<EvictionCandidate>, EvictionQueue::BATCH_SIZE> EvictionQueue::next() {
+    return std::span<std::atomic<EvictionCandidate>, BATCH_SIZE>(
+        data.get() + ((evictionCursor += BATCH_SIZE) % capacity), BATCH_SIZE);
 }
 
 void EvictionQueue::clear(std::atomic<EvictionCandidate>& candidate) {
@@ -106,10 +90,11 @@ void BufferManager::verifySizeParams(uint64_t bufferPoolSize, uint64_t maxDBSize
         throw BufferManagerException(stringFormat(
             "The given buffer pool size should be at least {} bytes.", KUZU_PAGE_SIZE));
     }
-    if (maxDBSize < KUZU_PAGE_SIZE * StorageConstants::PAGE_GROUP_SIZE) {
+    // We require at least two page groups, one for the main data file, and one for the shadow file.
+    if (maxDBSize < 2 * KUZU_PAGE_SIZE * StorageConstants::PAGE_GROUP_SIZE) {
         throw BufferManagerException(
             "The given max db size should be at least " +
-            std::to_string(KUZU_PAGE_SIZE * StorageConstants::PAGE_GROUP_SIZE) + " bytes.");
+            std::to_string(2 * KUZU_PAGE_SIZE * StorageConstants::PAGE_GROUP_SIZE) + " bytes.");
     }
     if ((maxDBSize & (maxDBSize - 1)) != 0) {
         throw BufferManagerException("The given max db size should be a power of 2.");
@@ -200,11 +185,12 @@ void handleAccessViolation(unsigned int exceptionCode, PEXCEPTION_POINTERS excep
 // Returns true if the function completes successfully
 inline bool try_func(const std::function<void(uint8_t*)>& func, uint8_t* frame,
     const std::array<std::unique_ptr<VMRegion>, 2>& vmRegions [[maybe_unused]],
-    PageSizeClass pageSizeClass [[maybe_unused]]) {
+    PageSizeClass pageSizeClass [[maybe_unused]], [[maybe_unused]] PageState* pageState) {
 #if BM_MALLOC
     if (frame == nullptr) {
         return false;
     }
+    pageState->addReader();
 #endif
 
 #if defined(_WIN32) && !BM_MALLOC
@@ -222,6 +208,9 @@ inline bool try_func(const std::function<void(uint8_t*)>& func, uint8_t* frame,
             throw EXCEPTION_ACCESS_VIOLATION;
         }
     }
+#endif
+#if BM_MALLOC
+    pageState->removeReader();
 #endif
     return true;
 }
@@ -238,7 +227,7 @@ void BufferManager::optimisticRead(FileHandle& fileHandle, page_idx_t pageIdx,
         switch (PageState::getState(currStateAndVersion)) {
         case PageState::UNLOCKED: {
             if (!try_func(func, getFrame(fileHandle, pageIdx), vmRegions,
-                    fileHandle.getPageSizeClass())) {
+                    fileHandle.getPageSizeClass(), pageState)) {
                 continue;
             }
             if (pageState->getStateAndVersion() == currStateAndVersion) {
@@ -269,34 +258,36 @@ void BufferManager::unpin(FileHandle& fileHandle, page_idx_t pageIdx) {
 
 // evicts up to 64 pages and returns the space reclaimed
 uint64_t BufferManager::evictPages() {
-    constexpr size_t BATCH_SIZE = 64;
-    std::array<std::atomic<EvictionCandidate>*, BATCH_SIZE> evictionCandidates{};
+    std::array<std::atomic<EvictionCandidate>*, EvictionQueue::BATCH_SIZE> evictionCandidates{};
     size_t evictablePages = 0;
-    size_t pagesTried = 0;
     uint64_t claimedMemory = 0;
 
     // Try each page at least twice.
     // E.g. if the vast majority of pages are unmarked and unlocked,
     // the first pass will mark them and the second pass, if insufficient marked pages
     // are found, will evict the first batch.
-    auto failureLimit = evictionQueue.getSize() * 2;
-    while (evictablePages < BATCH_SIZE && pagesTried < failureLimit) {
-        evictionCandidates[evictablePages] = evictionQueue.next();
-        pagesTried++;
-        auto evictionCandidate = evictionCandidates[evictablePages]->load();
-        if (evictionCandidate == EvictionQueue::EMPTY) {
-            continue;
-        }
-        auto* pageState =
-            fileHandles[evictionCandidate.fileIdx]->getPageState(evictionCandidate.pageIdx);
-        auto pageStateAndVersion = pageState->getStateAndVersion();
-        if (!evictionCandidate.isEvictable(pageStateAndVersion)) {
-            if (evictionCandidate.isSecondChanceEvictable(pageStateAndVersion)) {
-                pageState->tryMark(pageStateAndVersion);
+    // Using the eviction queue's cursor means that we fail after the same number of total attempts,
+    // regardless of how many threads are trying to evict.
+    auto startCursor = evictionQueue.getEvictionCursor();
+    auto failureLimit = evictionQueue.getCapacity() * 2;
+    while (evictablePages == 0 && evictionQueue.getEvictionCursor() - startCursor < failureLimit) {
+        for (auto& candidate : evictionQueue.next()) {
+            auto evictionCandidate = candidate.load();
+            if (evictionCandidate == EvictionQueue::EMPTY) {
+                continue;
             }
-            continue;
+            KU_ASSERT(evictionCandidate.fileIdx < fileHandles.size());
+            auto* pageState =
+                fileHandles[evictionCandidate.fileIdx]->getPageState(evictionCandidate.pageIdx);
+            auto pageStateAndVersion = pageState->getStateAndVersion();
+            if (!evictionCandidate.isEvictable(pageStateAndVersion)) {
+                if (evictionCandidate.isSecondChanceEvictable(pageStateAndVersion)) {
+                    pageState->tryMark(pageStateAndVersion);
+                }
+                continue;
+            }
+            evictionCandidates[evictablePages++] = &candidate;
         }
-        evictablePages++;
     }
 
     for (size_t i = 0; i < evictablePages; i++) {
@@ -305,6 +296,25 @@ uint64_t BufferManager::evictPages() {
     return claimedMemory;
 }
 
+void BufferManager::removeEvictedCandidates() {
+    auto startCursor = evictionQueue.getEvictionCursor();
+    while (evictionQueue.getEvictionCursor() - startCursor < evictionQueue.getCapacity()) {
+        for (auto& candidate : evictionQueue.next()) {
+            auto evictionCandidate = candidate.load();
+            if (evictionCandidate == EvictionQueue::EMPTY) {
+                continue;
+            }
+            KU_ASSERT(evictionCandidate.fileIdx < fileHandles.size());
+            auto* pageState =
+                fileHandles[evictionCandidate.fileIdx]->getPageState(evictionCandidate.pageIdx);
+            auto pageStateAndVersion = pageState->getStateAndVersion();
+            if (PageState::getState(pageStateAndVersion) == PageState::EVICTED) {
+                evictionQueue.clear(candidate);
+            }
+        }
+    }
+}
+
 // This function tries to load the given page into a frame. Due to our design of mmap, each page is
 // uniquely mapped to a frame. Thus, claiming a frame is equivalent to ensuring enough physical
 // memory is available.
@@ -345,8 +355,10 @@ bool BufferManager::reserve(uint64_t sizeToReserve) {
         // space for what we're trying to reserve, then we can continue even if the current total is
         // higher than the buffer pool size as we should never actually exceed the buffer pool size.
         return sizeToReserve > totalClaimedMemory &&
-               (usedMemory - totalClaimedMemory) > bufferPoolSize.load();
+               // usedMemory - totalClaimedMemory could underflow
+               usedMemory > bufferPoolSize.load() - totalClaimedMemory;
     };
+    uint8_t failedCount = 0;
     // Evict pages if necessary until we have enough memory.
     while (needMoreMemory()) {
         uint64_t memoryClaimed = 0;
@@ -355,18 +367,28 @@ bool BufferManager::reserve(uint64_t sizeToReserve) {
         if (!spiller || usedMemory - nonEvictableMemory > bufferPoolSize / 2) {
             memoryClaimed = evictPages();
         } else {
-            memoryClaimed = spiller->claimNextGroup();
-            nonEvictableClaimedMemory += memoryClaimed;
+            auto [_memoryClaimed, nowEvictableMemory] = spiller->claimNextGroup();
+            memoryClaimed = _memoryClaimed;
+            nonEvictableClaimedMemory += _memoryClaimed;
+            nonEvictableMemory -= nowEvictableMemory;
             // If we're unable to claim anything from the spiller, fall back to evicting pages
-            if (memoryClaimed == 0) {
+            // We may also need to evict pages if the spiller just unpins BM pages
+            if (memoryClaimed == 0 || nowEvictableMemory > 0) {
                 memoryClaimed = evictPages();
             }
         }
         if (memoryClaimed == 0 && needMoreMemory()) {
-            // Cannot find more pages to be evicted. Free the memory we reserved and return false.
-            freeUsedMemory(sizeToReserve + totalClaimedMemory);
-            nonEvictableMemory -= nonEvictableClaimedMemory;
-            return false;
+            if (failedCount++ < 2) {
+                // If we failed to find any memory to free, try waiting briefly for other threads to
+                // stop using memory
+                std::this_thread::sleep_for(std::chrono::milliseconds(5));
+            } else {
+                // Cannot find more pages to be evicted. Free the memory we reserved and return
+                // false.
+                freeUsedMemory(sizeToReserve + totalClaimedMemory);
+                nonEvictableMemory -= nonEvictableClaimedMemory;
+                return false;
+            }
         }
         totalClaimedMemory += memoryClaimed;
     }
@@ -393,8 +415,14 @@ uint64_t BufferManager::tryEvictPage(std::atomic<EvictionCandidate>& _candidate)
     }
     // The pageState was locked, but another thread already evicted this candidate and unlocked it
     // before the lock occurred
-    if (_candidate.load() != candidate) {
-        pageState.unlock();
+    if (_candidate.load() != candidate
+#if BM_MALLOC
+        // When the pageState is locked, optimisticReads will wait, so at this point no new
+        // optimistic reads will begin and thus it is safe to free the buffer at this point
+        || pageState.getReaderCount() > 0
+#endif
+    ) {
+        pageState.unlockUnchanged();
         return 0;
     }
     if (fileHandles[candidate.fileIdx]->isInMemoryMode()) {
@@ -430,7 +458,6 @@ void BufferManager::cachePageIntoFrame(FileHandle& fileHandle, page_idx_t pageId
 }
 
 void BufferManager::removeFilePagesFromFrames(FileHandle& fileHandle) {
-    evictionQueue.removeCandidatesForFile(fileHandle.getFileIndex());
     for (auto pageIdx = 0u; pageIdx < fileHandle.getNumPages(); ++pageIdx) {
         removePageFromFrame(fileHandle, pageIdx, false /* do not flush */);
     }
