@@ -1,6 +1,6 @@
 #include "storage/predicate/null_predicate.h"
 
-#include "storage/store/column_chunk_stats.h"
+#include "storage/table/column_chunk_stats.h"
 
 namespace kuzu::storage {
 common::ZoneMapCheckResult ColumnNullPredicate::checkZoneMap(
