@@ -31,9 +31,6 @@ std::unique_ptr<WALRecord> WALRecord::deserialize(Deserializer& deserializer,
     case WALRecordType::COMMIT_RECORD: {
         walRecord = CommitRecord::deserialize(deserializer);
     } break;
-    case WALRecordType::ROLLBACK_RECORD: {
-        walRecord = RollbackRecord::deserialize(deserializer);
-    } break;
     case WALRecordType::CREATE_CATALOG_ENTRY_RECORD: {
         walRecord = CreateCatalogEntryRecord::deserialize(deserializer);
     } break;
@@ -49,7 +46,7 @@ std::unique_ptr<WALRecord> WALRecord::deserialize(Deserializer& deserializer,
     case WALRecordType::NODE_DELETION_RECORD: {
         walRecord = NodeDeletionRecord::deserialize(deserializer, clientContext);
     } break;
-    case WALRecordType::NODE_UDPATE_RECORD: {
+    case WALRecordType::NODE_UPDATE_RECORD: {
         walRecord = NodeUpdateRecord::deserialize(deserializer, clientContext);
     } break;
     case WALRecordType::REL_DELETION_RECORD: {
@@ -70,6 +67,9 @@ std::unique_ptr<WALRecord> WALRecord::deserialize(Deserializer& deserializer,
     case WALRecordType::UPDATE_SEQUENCE_RECORD: {
         walRecord = UpdateSequenceRecord::deserialize(deserializer);
     } break;
+    case WALRecordType::LOAD_EXTENSION_RECORD: {
+        walRecord = LoadExtensionRecord::deserialize(deserializer);
+    } break;
     case WALRecordType::INVALID_RECORD: {
         throw RuntimeException("Corrupted wal file. Read out invalid WAL record type.");
     }
@@ -97,14 +97,6 @@ std::unique_ptr<CommitRecord> CommitRecord::deserialize(Deserializer&) {
     return std::make_unique<CommitRecord>();
 }
 
-void RollbackRecord::serialize(Serializer& serializer) const {
-    WALRecord::serialize(serializer);
-}
-
-std::unique_ptr<RollbackRecord> RollbackRecord::deserialize(Deserializer&) {
-    return std::make_unique<RollbackRecord>();
-}
-
 void CheckpointRecord::serialize(Serializer& serializer) const {
     WALRecord::serialize(serializer);
 }
@@ -121,9 +113,8 @@ void CreateCatalogEntryRecord::serialize(Serializer& serializer) const {
 
 std::unique_ptr<CreateCatalogEntryRecord> CreateCatalogEntryRecord::deserialize(
     Deserializer& deserializer) {
-    auto catalogEntry = catalog::CatalogEntry::deserialize(deserializer);
     auto retVal = std::make_unique<CreateCatalogEntryRecord>();
-    retVal->ownedCatalogEntry = std::move(catalogEntry);
+    retVal->ownedCatalogEntry = catalog::CatalogEntry::deserialize(deserializer);
     bool isInternal = false;
     deserializer.deserializeValue(isInternal);
     retVal->isInternal = isInternal;
@@ -143,13 +134,15 @@ std::unique_ptr<CopyTableRecord> CopyTableRecord::deserialize(Deserializer& dese
 
 void DropCatalogEntryRecord::serialize(Serializer& serializer) const {
     WALRecord::serialize(serializer);
-    serializer.write(entryID);
+    serializer.write<oid_t>(entryID);
+    serializer.write<catalog::CatalogEntryType>(entryType);
 }
 
 std::unique_ptr<DropCatalogEntryRecord> DropCatalogEntryRecord::deserialize(
     Deserializer& deserializer) {
     auto retVal = std::make_unique<DropCatalogEntryRecord>();
     deserializer.deserializeValue(retVal->entryID);
+    deserializer.deserializeValue(retVal->entryType);
     return retVal;
 }
 
@@ -179,6 +172,12 @@ static void serializeAlterExtraInfo(Serializer& serializer, const BoundAlterInfo
         auto renameTableInfo = extraInfo->constPtrCast<BoundExtraRenameTableInfo>();
         serializer.write(renameTableInfo->newName);
     } break;
+    case AlterType::ADD_FROM_TO_CONNECTION:
+    case AlterType::DROP_FROM_TO_CONNECTION: {
+        auto connectionInfo = extraInfo->constPtrCast<BoundExtraAlterFromToConnection>();
+        serializer.write(connectionInfo->fromTableID);
+        serializer.write(connectionInfo->toTableID);
+    } break;
     default: {
         KU_UNREACHABLE;
     }
@@ -219,6 +218,14 @@ static decltype(auto) deserializeAlterRecord(Deserializer& deserializer) {
         deserializer.deserializeValue(newName);
         extraInfo = std::make_unique<BoundExtraRenameTableInfo>(std::move(newName));
     } break;
+    case AlterType::ADD_FROM_TO_CONNECTION:
+    case AlterType::DROP_FROM_TO_CONNECTION: {
+        table_id_t fromTableID = INVALID_TABLE_ID;
+        table_id_t toTableID = INVALID_TABLE_ID;
+        deserializer.deserializeValue(fromTableID);
+        deserializer.deserializeValue(toTableID);
+        extraInfo = std::make_unique<BoundExtraAlterFromToConnection>(fromTableID, toTableID);
+    } break;
     default: {
         KU_UNREACHABLE;
     }
@@ -285,7 +292,7 @@ std::unique_ptr<TableInsertionRecord> TableInsertionRecord::deserialize(Deserial
     deserializer.deserializeValue<row_idx_t>(numRows);
     deserializer.validateDebuggingInfo(key, "num_vectors");
     deserializer.deserializeValue(numVectors);
-    auto resultChunkState = std::make_shared<DataChunkState>();
+    auto resultChunkState = DataChunkState::getSingleValueDataChunkState();
     valueVectors.reserve(numVectors);
     for (auto i = 0u; i < numVectors; i++) {
         valueVectors.push_back(ValueVector::deSerialize(deserializer,
@@ -458,5 +465,19 @@ std::unique_ptr<RelUpdateRecord> RelUpdateRecord::deserialize(Deserializer& dese
         std::move(dstNodeIDVector), std::move(relIDVector), std::move(propertyVector));
 }
 
+void LoadExtensionRecord::serialize(Serializer& serializer) const {
+    WALRecord::serialize(serializer);
+    serializer.writeDebuggingInfo("path");
+    serializer.write<std::string>(path);
+}
+
+std::unique_ptr<LoadExtensionRecord> LoadExtensionRecord::deserialize(Deserializer& deserializer) {
+    std::string key;
+    deserializer.validateDebuggingInfo(key, "path");
+    std::string path;
+    deserializer.deserializeValue<std::string>(path);
+    return std::make_unique<LoadExtensionRecord>(std::move(path));
+}
+
 } // namespace storage
 } // namespace kuzu
