@@ -18,14 +18,15 @@ public class TestBase {
     static Path tempDir;
 
     @BeforeAll
-    static void getDBandConn() throws IOException, ObjectRefDestroyedException {
-        TestHelper.loadData(tempDir.toFile().getAbsolutePath());
+    static void getDBandConn() throws IOException {
+        String dbPath = tempDir.resolve("db.kz").toString();
+        TestHelper.loadData(dbPath);
         db = TestHelper.getDatabase();
         conn = TestHelper.getConnection();
     }
 
     @AfterAll
-    static void destroyDBandConn() throws ObjectRefDestroyedException {
+    static void destroyDBandConn() {
         try {
             db.close();
             conn.close();
