@@ -1,6 +1,7 @@
 use crate::ffi::ffi;
 use crate::ffi::ffi::PhysicalTypeID;
 use crate::logical_type::LogicalType;
+use rust_decimal::prelude::ToPrimitive;
 use std::cmp::Ordering;
 use std::convert::{TryFrom, TryInto};
 use std::fmt;
@@ -24,7 +25,9 @@ impl std::fmt::Display for ConversionError {
 
 impl std::fmt::Debug for ConversionError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        use ConversionError::*;
+        use ConversionError::{
+            Date, Timestamp, TimestampMs, TimestampNs, TimestampSec, TimestampTz,
+        };
         match self {
             Date(days) => write!(f, "Could not convert Kuzu date offset of UNIX_EPOCH + {days} days to time::Date"),
             Timestamp(us) => write!(f, "Could not convert Kuzu timestamp offset of UNIX_EPOCH + {us} microseconds to time::OffsetDateTime"),
@@ -42,7 +45,7 @@ impl std::error::Error for ConversionError {
     }
 }
 
-/// NodeVal represents a node in the graph and stores the nodeID, label and properties of that
+/// `NodeVal` represents a node in the graph and stores the nodeID, label and properties of that
 /// node.
 #[derive(Clone, Debug, PartialEq)]
 pub struct NodeVal {
@@ -68,7 +71,7 @@ impl NodeVal {
         &self.label
     }
 
-    /// Adds a property with the given key/value pair to the NodeVal
+    /// Adds a property with the given key/value pair to the `NodeVal`
     /// # Arguments
     /// * `key`: The name of the property
     /// * `value`: The value of the property
@@ -76,19 +79,16 @@ impl NodeVal {
         self.properties.push((key.into(), value.into()));
     }
 
-    /// Returns all properties of the NodeVal
+    /// Returns all properties of the `NodeVal`
     pub fn get_properties(&self) -> &Vec<(String, Value)> {
         &self.properties
     }
 }
 
-fn properties_display(
-    f: &mut fmt::Formatter<'_>,
-    properties: &Vec<(String, Value)>,
-) -> fmt::Result {
+fn properties_display(f: &mut fmt::Formatter<'_>, properties: &[(String, Value)]) -> fmt::Result {
     write!(f, "{{")?;
     for (index, (name, value)) in properties.iter().enumerate() {
-        write!(f, "{}:{}", name, value)?;
+        write!(f, "{name}:{value}")?;
         if index < properties.len() - 1 {
             write!(f, ",")?;
         }
@@ -104,7 +104,7 @@ impl std::fmt::Display for NodeVal {
     }
 }
 
-/// RelVal represents a relationship in the graph and stores the relID, src/dst nodes and properties of that
+/// `RelVal` represents a relationship in the graph and stores the relID, src/dst nodes and properties of that
 /// rel
 #[derive(Clone, Debug, PartialEq)]
 pub struct RelVal {
@@ -135,7 +135,7 @@ impl RelVal {
         &self.label
     }
 
-    /// Adds a property with the given key/value pair to the NodeVal
+    /// Adds a property with the given key/value pair to the `NodeVal`
     /// # Arguments
     /// * `key`: The name of the property
     /// * `value`: The value of the property
@@ -143,7 +143,7 @@ impl RelVal {
         self.properties.push((key, value));
     }
 
-    /// Returns all properties of the RelVal
+    /// Returns all properties of the `RelVal`
     pub fn get_properties(&self) -> &Vec<(String, Value)> {
         &self.properties
     }
@@ -157,7 +157,7 @@ impl std::fmt::Display for RelVal {
     }
 }
 
-/// Stores the table_id and offset of a node/rel.
+/// Stores the `table_id` and `offset` of a node/rel.
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct InternalID {
     pub offset: u64,
@@ -195,7 +195,7 @@ impl From<(u64, u64)> for InternalID {
     }
 }
 
-/// Data types supported by Kùzu
+/// Data types supported by Kuzu
 ///
 /// Also see <https://kuzudb.com/docusaurus/cypher/data-types/overview.html>
 #[derive(Clone, Debug, PartialEq)]
@@ -214,19 +214,19 @@ pub enum Value {
     Double(f64),
     Float(f32),
     /// Stored internally as the number of days since 1970-01-01 as a 32-bit signed integer, which
-    /// allows for a wider range of dates to be stored than can be represented by time::Date
+    /// allows for a wider range of dates to be stored than can be represented by `time::Date`
     ///
     /// <https://kuzudb.com/docusaurus/cypher/data-types/date.html>
     Date(time::Date),
     /// May be signed or unsigned.
     ///
-    /// Nanosecond precision of time::Duration (if available) will not be preserved when passed to
+    /// Nanosecond precision of `time::Duration` (if available) will not be preserved when passed to
     /// queries, and results will always have at most microsecond precision.
     ///
     /// <https://kuzudb.com/docusaurus/cypher/data-types/interval.html>
     Interval(time::Duration),
     /// Stored internally as the number of microseconds since 1970-01-01
-    /// Nanosecond precision of SystemTime (if available) will not be preserved when used.
+    /// Nanosecond precision of `SystemTime` (if available) will not be preserved when used.
     ///
     /// <https://kuzudb.com/docusaurus/cypher/data-types/timestamp.html>
     Timestamp(time::OffsetDateTime),
@@ -255,7 +255,7 @@ pub enum Value {
         ///
         /// Does not include the starting or ending Node.
         nodes: Vec<NodeVal>,
-        /// Sequence of Rels which make up the RecursiveRel
+        /// Sequence of Rels which make up the `RecursiveRel`
         rels: Vec<RelVal>,
     },
     /// <https://kuzudb.com/docusaurus/cypher/data-types/map>
@@ -269,10 +269,10 @@ pub enum Value {
     Decimal(rust_decimal::Decimal),
 }
 
-fn display_list<T: std::fmt::Display>(f: &mut fmt::Formatter<'_>, list: &Vec<T>) -> fmt::Result {
+fn display_list<T: std::fmt::Display>(f: &mut fmt::Formatter<'_>, list: &[T]) -> fmt::Result {
     write!(f, "[")?;
     for (i, value) in list.iter().enumerate() {
-        write!(f, "{}", value)?;
+        write!(f, "{value}")?;
         if i != list.len() - 1 {
             write!(f, ",")?;
         }
@@ -301,17 +301,17 @@ impl std::fmt::Display for Value {
             Value::List(_, x) | Value::Array(_, x) => display_list(f, x),
             // Note: These don't match kuzu's toString, but we probably don't want them to
             Value::Interval(x) => write!(f, "{x}"),
-            Value::Timestamp(x) => write!(f, "{x}"),
-            Value::TimestampTz(x) => write!(f, "{x}"),
-            Value::TimestampNs(x) => write!(f, "{x}"),
-            Value::TimestampMs(x) => write!(f, "{x}"),
-            Value::TimestampSec(x) => write!(f, "{x}"),
+            Value::Timestamp(x)
+            | Value::TimestampTz(x)
+            | Value::TimestampNs(x)
+            | Value::TimestampMs(x)
+            | Value::TimestampSec(x) => write!(f, "{x}"),
             Value::Float(x) => write!(f, "{x}"),
             Value::Double(x) => write!(f, "{x}"),
             Value::Struct(x) => {
                 write!(f, "{{")?;
                 for (i, (name, value)) in x.iter().enumerate() {
-                    write!(f, "{}: {}", name, value)?;
+                    write!(f, "{name}: {value}")?;
                     if i != x.len() - 1 {
                         write!(f, ", ")?;
                     }
@@ -321,7 +321,7 @@ impl std::fmt::Display for Value {
             Value::Map(_, x) => {
                 write!(f, "{{")?;
                 for (i, (name, value)) in x.iter().enumerate() {
-                    write!(f, "{}={}", name, value)?;
+                    write!(f, "{name}={value}")?;
                     if i != x.len() - 1 {
                         write!(f, ", ")?;
                     }
@@ -410,7 +410,7 @@ impl From<&Value> for LogicalType {
 fn get_date_from_unix_days(days: i32) -> Result<time::Date, ConversionError> {
     time::Date::from_calendar_date(1970, time::Month::January, 1)
         .unwrap()
-        .checked_add(time::Duration::days(days as i64))
+        .checked_add(time::Duration::days(i64::from(days)))
         .ok_or(ConversionError::Date(days))
 }
 
@@ -425,15 +425,16 @@ impl TryFrom<&ffi::Value> for Value {
 
     fn try_from(value: &ffi::Value) -> Result<Self, Self::Error> {
         use ffi::LogicalTypeID;
-        if value.isNull() {
-            return Ok(Value::Null(value.into()));
-        }
-
+        #[allow(clippy::cast_possible_wrap)]
         fn get_i128(value: &ffi::Value) -> i128 {
             let int128_val = ffi::value_get_int128_t(value);
             let low = int128_val[1];
             let high = int128_val[0] as i64;
-            (low as i128) + ((high as i128) << 64)
+            (i128::from(low)) + ((i128::from(high)) << 64)
+        }
+
+        if value.isNull() {
+            return Ok(Value::Null(value.into()));
         }
 
         match ffi::value_get_data_type_id(value) {
@@ -448,6 +449,7 @@ impl TryFrom<&ffi::Value> for Value {
             LogicalTypeID::UINT32 => Ok(Value::UInt32(value.get_value_u32())),
             LogicalTypeID::UINT64 => Ok(Value::UInt64(value.get_value_u64())),
             LogicalTypeID::INT128 => Ok(Value::Int128(get_i128(value))),
+            #[allow(clippy::cast_sign_loss)]
             LogicalTypeID::UUID => Ok(Value::UUID(uuid::Uuid::from_u128(
                 // values are stored as i128 and the first bit flipped so that they order as if
                 // they are u128
@@ -622,7 +624,7 @@ impl TryFrom<&ffi::Value> for Value {
                         }
                     })
                 } else {
-                    panic!("Unexpected value in RecursiveRel's rels: {}", rels)
+                    panic!("Unexpected value in RecursiveRel's rels: {rels}")
                 };
                 let rels = if let Value::List(LogicalType::Rel, rels) = rels {
                     rels.into_iter().map(|x| {
@@ -633,7 +635,7 @@ impl TryFrom<&ffi::Value> for Value {
                         }
                     })
                 } else {
-                    panic!("Unexpected value in RecursiveRel's rels: {}", rels)
+                    panic!("Unexpected value in RecursiveRel's rels: {rels}")
                 };
 
                 Ok(Value::RecursiveRel {
@@ -642,12 +644,9 @@ impl TryFrom<&ffi::Value> for Value {
                 })
             }
             LogicalTypeID::UNION => {
-                let types =
-                    if let LogicalType::Union { types } = ffi::value_get_data_type(value).into() {
-                        types
-                    } else {
-                        unreachable!()
-                    };
+                let LogicalType::Union { types } = ffi::value_get_data_type(value).into() else {
+                    unreachable!()
+                };
                 debug_assert!(ffi::value_get_children_size(value) == 1);
                 let value: Value = ffi::value_get_child(value, 0).try_into()?;
                 Ok(Value::Union {
@@ -664,10 +663,10 @@ impl TryFrom<&ffi::Value> for Value {
                 {
                     let decimal_value: i128 = match ffi::value_get_physical_type(value) {
                         PhysicalTypeID::INT128 => get_i128(value),
-                        PhysicalTypeID::INT64 => value.get_value_i64() as i128,
-                        PhysicalTypeID::INT32 => value.get_value_i32() as i128,
-                        PhysicalTypeID::INT16 => value.get_value_i16() as i128,
-                        PhysicalTypeID::INT8 => value.get_value_i8() as i128,
+                        PhysicalTypeID::INT64 => i128::from(value.get_value_i64()),
+                        PhysicalTypeID::INT32 => i128::from(value.get_value_i32()),
+                        PhysicalTypeID::INT16 => i128::from(value.get_value_i16()),
+                        PhysicalTypeID::INT8 => i128::from(value.get_value_i8()),
                         _ => unreachable!(),
                     };
                     Ok(Value::Decimal(rust_decimal::Decimal::from_i128_with_scale(
@@ -679,7 +678,7 @@ impl TryFrom<&ffi::Value> for Value {
                 }
             }
             // TODO(bmwinger): Better error message for types which are unsupported
-            x => panic!("Unsupported type {:?}", x),
+            x => panic!("Unsupported type {x:?}"),
         }
     }
 }
@@ -690,18 +689,24 @@ impl TryInto<cxx::UniquePtr<ffi::Value>> for Value {
     type Error = crate::error::Error;
 
     fn try_into(self) -> Result<cxx::UniquePtr<ffi::Value>, Self::Error> {
+        #[allow(clippy::cast_sign_loss, clippy::cast_possible_truncation)]
         fn get_high_low(value: i128) -> (i64, u64) {
             ((value >> 64) as i64, value as u64)
         }
 
         fn date_to_kuzu_date_t(value: time::Date) -> i32 {
             // Convert to days since 1970-01-01
-            (value - time::Date::from_ordinal_date(1970, 1).unwrap()).whole_days() as i32
+            (value - time::Date::from_ordinal_date(1970, 1).unwrap())
+                .whole_days()
+                .to_i32()
+                .expect("kuzu_date i64->i32 overflow")
         }
 
         fn datetime_to_timestamp_t(value: time::OffsetDateTime) -> i64 {
             // Convert to microseconds since 1970-01-01
-            (value.unix_timestamp_nanos() / 1000) as i64
+            (value.unix_timestamp_nanos() / 1000)
+                .to_i64()
+                .expect("timestamp i128->i64 overflow")
         }
 
         fn get_interval_t(value: time::Duration) -> (i32, i32, i64) {
@@ -711,8 +716,13 @@ impl TryInto<cxx::UniquePtr<ffi::Value>> for Value {
             interval -= Duration::days(months * 30);
             let days = interval.whole_days();
             interval -= Duration::days(days);
-            let micros = interval.whole_microseconds() as i64;
-            (months as i32, days as i32, micros)
+            let months = months.to_i32().expect("months i64->i32 overflow");
+            let days = days.to_i32().expect("days i64->i32 overflow");
+            let micros = interval
+                .whole_microseconds()
+                .to_i64()
+                .expect("micros i128->i64 overflow");
+            (months, days, micros)
         }
 
         match self {
@@ -730,10 +740,12 @@ impl TryInto<cxx::UniquePtr<ffi::Value>> for Value {
                 let (high, low) = get_high_low(value);
                 Ok(ffi::create_value_int128_t(high, low))
             }
+            #[allow(clippy::cast_possible_wrap)]
             Value::UUID(value) => {
                 // values are stored as i128 and the first bit flipped so that they order as if
                 // they are u128
-                let (high, low) = get_high_low(value.as_u128() as i128 ^ (1 << 127));
+                let value = value.as_u128() as i128;
+                let (high, low) = get_high_low(value ^ (1 << 127));
                 Ok(ffi::create_value_uuid_t(high, low))
             }
             Value::Float(value) => Ok(ffi::create_value_float(value)),
@@ -748,16 +760,25 @@ impl TryInto<cxx::UniquePtr<ffi::Value>> for Value {
             }
             Value::TimestampTz(value) => Ok(ffi::create_value_timestamp_tz(
                 // Convert to microseconds since 1970-01-01
-                (value.unix_timestamp_nanos() / 1000) as i64,
+                (value.unix_timestamp_nanos() / 1000)
+                    .to_i64()
+                    .expect("TimestampTz i128->i64 overflow"),
             )),
             Value::TimestampNs(value) => Ok(ffi::create_value_timestamp_ns(
-                value.unix_timestamp_nanos() as i64,
+                value
+                    .unix_timestamp_nanos()
+                    .to_i64()
+                    .expect("TimestampMs i128->i64 overflow"),
             )),
             Value::TimestampMs(value) => Ok(ffi::create_value_timestamp_ms(
-                (value.unix_timestamp_nanos() / 1000000) as i64,
+                (value.unix_timestamp_nanos() / 1_000_000)
+                    .to_i64()
+                    .expect("TimestampMs i128->i64 overflow"),
             )),
             Value::TimestampSec(value) => Ok(ffi::create_value_timestamp_sec(
-                (value.unix_timestamp_nanos() / 1000000000) as i64,
+                (value.unix_timestamp_nanos() / 1_000_000_000)
+                    .to_i64()
+                    .expect("TimestampSec i128->i64 overflow"),
             )),
             Value::Date(value) => Ok(ffi::create_value_date(date_to_kuzu_date_t(value))),
             Value::Interval(value) => {
@@ -940,10 +961,9 @@ impl From<&str> for Value {
 
 #[cfg(test)]
 mod tests {
+    use crate::database::SYSTEM_CONFIG_FOR_TESTS;
     use crate::ffi::ffi;
-    use crate::{
-        Connection, Database, InternalID, LogicalType, NodeVal, RelVal, SystemConfig, Value,
-    };
+    use crate::{Connection, Database, InternalID, LogicalType, NodeVal, RelVal, Value};
     use anyhow::Result;
     use rust_decimal_macros::dec;
     use std::collections::HashSet;
@@ -956,7 +976,7 @@ mod tests {
         ($($name:ident: $value:expr,)*) => {
         $(
             #[test]
-            /// Tests that the values are correctly converted into kuzu::common::Value and back
+            /// Tests that the values are correctly converted into `kuzu::common::Value` and back
             fn $name() -> Result<()> {
                 let rust_type: LogicalType = $value;
                 let typ: cxx::UniquePtr<ffi::LogicalType> = (&rust_type).try_into()?;
@@ -972,7 +992,7 @@ mod tests {
         ($($name:ident: $value:expr,)*) => {
         $(
             #[test]
-            /// Tests that the values are correctly converted into kuzu::common::Value and back
+            /// Tests that the values are correctly converted into `kuzu::common::Value` and back
             fn $name() -> Result<()> {
                 let rust_value: Value = $value;
                 let value: cxx::UniquePtr<ffi::Value> = rust_value.clone().try_into()?;
@@ -1006,7 +1026,7 @@ mod tests {
             /// Tests that passing the values through the database returns what we put in
             fn $name() -> Result<()> {
                 let temp_dir = tempfile::tempdir()?;
-                let db = Database::new(temp_dir.path(), SystemConfig::default())?;
+                let db = Database::new(temp_dir.path().join("testdb"), SYSTEM_CONFIG_FOR_TESTS)?;
                 let conn = Connection::new(&db)?;
                 conn.query(&format!(
                     "CREATE NODE TABLE Person(name STRING, item {}, PRIMARY KEY(name));",
@@ -1203,7 +1223,7 @@ mod tests {
     /// Tests that the list value is correctly constructed
     fn test_list_get() -> Result<()> {
         let temp_dir = tempfile::tempdir()?;
-        let db = Database::new(temp_dir.path(), SystemConfig::default())?;
+        let db = Database::new(temp_dir.path().join("testdb"), SYSTEM_CONFIG_FOR_TESTS)?;
         let conn = Connection::new(&db)?;
         for result in conn.query("RETURN [\"Alice\", \"Bob\"] AS l;")? {
             assert_eq!(result.len(), 1);
@@ -1220,7 +1240,7 @@ mod tests {
     /// Test that the timestamp round-trips through kuzu's internal timestamp
     fn test_timestamp() -> Result<()> {
         let temp_dir = tempfile::tempdir()?;
-        let db = Database::new(temp_dir.path(), SystemConfig::default())?;
+        let db = Database::new(temp_dir.path().join("test"), SYSTEM_CONFIG_FOR_TESTS)?;
         let conn = Connection::new(&db)?;
         conn.query(
             "CREATE NODE TABLE Person(name STRING, registerTime TIMESTAMP, PRIMARY KEY(name));",
@@ -1267,7 +1287,7 @@ mod tests {
     #[test]
     fn test_node() -> Result<()> {
         let temp_dir = tempfile::tempdir()?;
-        let db = Database::new(temp_dir.path(), SystemConfig::default())?;
+        let db = Database::new(temp_dir.path().join("test"), SYSTEM_CONFIG_FOR_TESTS)?;
         let conn = Connection::new(&db)?;
         conn.query("CREATE NODE TABLE Person(name STRING, age INT64, PRIMARY KEY(name));")?;
         conn.query("CREATE (:Person {name: \"Alice\", age: 25});")?;
@@ -1293,7 +1313,7 @@ mod tests {
     #[test]
     fn test_recursive_rel() -> Result<()> {
         let temp_dir = tempfile::TempDir::new()?;
-        let db = Database::new(temp_dir.path(), SystemConfig::default())?;
+        let db = Database::new(temp_dir.path().join("test"), SYSTEM_CONFIG_FOR_TESTS)?;
         let conn = Connection::new(&db)?;
         conn.query("CREATE NODE TABLE Person(name STRING, age INT64, PRIMARY KEY(name));")?;
         conn.query("CREATE REL TABLE knows(FROM Person TO Person);")?;
@@ -1341,7 +1361,7 @@ mod tests {
     /// Test that null values are read correctly by the API
     fn test_null() -> Result<()> {
         let temp_dir = tempfile::tempdir()?;
-        let db = Database::new(temp_dir.path(), SystemConfig::default())?;
+        let db = Database::new(temp_dir.path().join("test"), SYSTEM_CONFIG_FOR_TESTS)?;
         let conn = Connection::new(&db)?;
         let result = conn.query("RETURN null")?.next();
         let result = &result.unwrap()[0];
@@ -1354,7 +1374,7 @@ mod tests {
     /// Tests that passing the values through the database returns what we put in
     fn test_serial() -> Result<()> {
         let temp_dir = tempfile::tempdir()?;
-        let db = Database::new(temp_dir.path(), SystemConfig::default())?;
+        let db = Database::new(temp_dir.path().join("test"), SYSTEM_CONFIG_FOR_TESTS)?;
         let conn = Connection::new(&db)?;
         conn.query("CREATE NODE TABLE Person(id SERIAL, name STRING, PRIMARY KEY(id));")?;
         conn.query("CREATE (:Person {name: \"Bob\"});")?;
@@ -1384,7 +1404,7 @@ mod tests {
         use std::fs::File;
         use std::io::Write;
         let temp_dir = tempfile::tempdir()?;
-        let db = Database::new(temp_dir.path(), SystemConfig::default())?;
+        let db = Database::new(temp_dir.path().join("test"), SYSTEM_CONFIG_FOR_TESTS)?;
         let conn = Connection::new(&db)?;
         conn.query(
             "CREATE NODE TABLE demo(a SERIAL, b UNION(num INT64, str STRING), PRIMARY KEY(a));",
@@ -1395,7 +1415,7 @@ mod tests {
             "COPY demo from '{}/demo.csv';",
             // Use forward-slashes instead of backslashes on windows, as thmay not be supported by
             // the query parser
-            temp_dir.path().display().to_string().replace("\\", "/")
+            temp_dir.path().display().to_string().replace('\\', "/")
         ))?;
         let result = conn.query("MATCH (d:demo) RETURN d.b;")?;
         let types = vec![
