@@ -26,20 +26,16 @@ def test_help(temp_db, flag) -> None:
     # database path not needed
     test = ShellTest().add_argument(flag)
     result = test.run()
-    result.check_stdout("KuzuDB Shell")
+    result.check_stdout("Kuzu shell")
     # with database path
     test = ShellTest().add_argument(temp_db).add_argument(flag)
     result = test.run()
-    result.check_stdout("KuzuDB Shell")
+    result.check_stdout("Kuzu shell")
 
 
 @pytest.mark.parametrize(
     "flag",
-    [
-        "-d",
-        "--defaultbpsize",
-        "--default_bp_size"
-    ],
+    ["-d", "--defaultbpsize", "--default_bp_size"],
 )
 def test_default_bp_size(temp_db, flag) -> None:
     # empty flag argument
@@ -62,10 +58,7 @@ def test_default_bp_size(temp_db, flag) -> None:
 
 @pytest.mark.parametrize(
     "flag",
-    [
-        "--maxdbsize",
-        "--max_db_size"
-    ],
+    ["--maxdbsize", "--max_db_size"],
 )
 def test_max_db_size(temp_db, flag) -> None:
     # empty flag argument
@@ -83,20 +76,19 @@ def test_max_db_size(temp_db, flag) -> None:
     # invalid flag
     test = ShellTest().add_argument(temp_db).add_argument(flag).add_argument("1000")
     result = test.run()
-    result.check_stderr(f"Buffer manager exception: The given max db size should be at least 4194304 bytes.")
+    result.check_stderr(
+        "Buffer manager exception: The given max db size should be at least 8388608 bytes."
+    )
 
     # successful flag
-    test = ShellTest().add_argument(temp_db).add_argument(flag).add_argument("4194304")
+    test = ShellTest().add_argument(temp_db).add_argument(flag).add_argument("8388608")
     result = test.run()
     result.check_stdout(f"Opening the database at path: {temp_db} in read-write mode.")
 
 
 @pytest.mark.parametrize(
     "flag",
-    [
-        "--nocompression",
-        "--no_compression"
-    ],
+    ["--nocompression", "--no_compression"],
 )
 def test_no_compression(temp_db, flag) -> None:
     test = ShellTest().add_argument(temp_db).add_argument(flag)
@@ -106,11 +98,7 @@ def test_no_compression(temp_db, flag) -> None:
 
 @pytest.mark.parametrize(
     "flag",
-    [
-        "-r",
-        "--readonly",
-        "--read_only"
-    ],
+    ["-r", "--readonly", "--read_only"],
 )
 def test_read_only(temp_db, flag) -> None:
     # cannot open an empty database in read only mode so initialize database
@@ -145,7 +133,7 @@ def test_history_path(temp_db, history_path) -> None:
     # invalid path
     test = ShellTest().add_argument(temp_db).add_argument("-p").add_argument("///////")
     result = test.run()
-    result.check_stderr("Invalid path to directory for history file")
+    result.check_stderr("Error: failed to open the history file: ///////history.txt")
 
     # valid path, file doesn't exist
     test = (
@@ -242,7 +230,9 @@ def test_mode(temp_db, flag) -> None:
         .add_argument(temp_db)
         .add_argument(flag)
         .add_argument("csv")
-        .statement('RETURN "This is a \\"test\\", with commas, \\"quotes\\", and\nnewlines.";')
+        .statement(
+            'RETURN "This is a \\"test\\", with commas, \\"quotes\\", and\nnewlines.";'
+        )
     )
     result = test.run()
     result.check_stdout('"This is a ""test"", with commas, ""quotes"", and\nnewlines."')
@@ -283,7 +273,9 @@ def test_mode(temp_db, flag) -> None:
         .add_argument(temp_db)
         .add_argument(flag)
         .add_argument("html")
-        .statement('RETURN "This is a <test> & \\"example\\" with \'special\' characters." AS a;')
+        .statement(
+            'RETURN "This is a <test> & \\"example\\" with \'special\' characters." AS a;'
+        )
     )
     result = test.run()
     result.check_stdout("<table>")
@@ -291,7 +283,9 @@ def test_mode(temp_db, flag) -> None:
     result.check_stdout("<th>a</th>")
     result.check_stdout("</tr>")
     result.check_stdout("<tr>")
-    result.check_stdout("<td>This is a &lt;test&gt; &amp; &quot;example&quot; with &apos;special&apos; characters.</td>")
+    result.check_stdout(
+        "<td>This is a &lt;test&gt; &amp; &quot;example&quot; with &apos;special&apos; characters.</td>"
+    )
     result.check_stdout("</tr>")
     result.check_stdout("</table>")
 
@@ -304,7 +298,7 @@ def test_mode(temp_db, flag) -> None:
         .statement('RETURN "Databases Rule" AS a, "kuzu is cool" AS b;')
     )
     result = test.run()
-    result.check_stdout('[{"a":"Databases Rule","b":"kuzu is cool"}]')
+    result.check_stdout('[\n{"a":"Databases Rule","b":"kuzu is cool"}\n]')
 
     # test json escaping
     test = (
@@ -312,10 +306,14 @@ def test_mode(temp_db, flag) -> None:
         .add_argument(temp_db)
         .add_argument(flag)
         .add_argument("json")
-        .statement('RETURN "This is a \\"test\\" with backslashes \\\\, newlines\n, and tabs \t." AS a;')
+        .statement(
+            'RETURN "This is a \\"test\\" with backslashes \\\\, newlines\n, and tabs \t." AS a;'
+        )
     )
     result = test.run()
-    result.check_stdout('[{"a":"This is a \\"test\\" with backslashes \\\\, newlines\\n, and tabs \\t."}]')
+    result.check_stdout(
+        '[\n{"a":"This is a \\"test\\" with backslashes \\\\, newlines\\n, and tabs \\t."}\n]'
+    )
 
     # test jsonlines mode
     test = (
@@ -334,10 +332,14 @@ def test_mode(temp_db, flag) -> None:
         .add_argument(temp_db)
         .add_argument(flag)
         .add_argument("jsonlines")
-        .statement('RETURN "This is a \\"test\\" with backslashes \\\\, newlines\n, and tabs \t." AS a;')
+        .statement(
+            'RETURN "This is a \\"test\\" with backslashes \\\\, newlines\n, and tabs \t." AS a;'
+        )
     )
     result = test.run()
-    result.check_stdout('{"a":"This is a \\"test\\" with backslashes \\\\, newlines\\n, and tabs \\t."}')
+    result.check_stdout(
+        '{"a":"This is a \\"test\\" with backslashes \\\\, newlines\\n, and tabs \\t."}'
+    )
 
     # test latex mode
     test = (
@@ -362,14 +364,18 @@ def test_mode(temp_db, flag) -> None:
         .add_argument(temp_db)
         .add_argument(flag)
         .add_argument("latex")
-        .statement('RETURN "This is a test with special characters: %, $, &, #, _, {, }, ~, ^, \\\\, <, and >." AS a;')
+        .statement(
+            'RETURN "This is a test with special characters: %, $, &, #, _, {, }, ~, ^, \\\\, <, and >." AS a;'
+        )
     )
     result = test.run()
     result.check_stdout("\\begin{tabular}{l}")
     result.check_stdout("\\hline")
     result.check_stdout("a\\\\")
     result.check_stdout("\\hline")
-    result.check_stdout("This is a test with special characters: \\%, \\$, \\&, \\#, \\_, \\{, \\}, \\textasciitilde{}, \\textasciicircum{}, \\textbackslash{}, \\textless{}, and \\textgreater{}.\\\\")
+    result.check_stdout(
+        "This is a test with special characters: \\%, \\$, \\&, \\#, \\_, \\{, \\}, \\textasciitilde{}, \\textasciicircum{}, \\textbackslash{}, \\textless{}, and \\textgreater{}.\\\\"
+    )
     result.check_stdout("\\hline")
     result.check_stdout("\\end{tabular}")
 
@@ -384,7 +390,7 @@ def test_mode(temp_db, flag) -> None:
     result = test.run()
     result.check_stdout("a = Databases Rule")
     result.check_stdout("b = kuzu is cool")
-    
+
     # test list mode
     test = (
         ShellTest()
@@ -403,10 +409,14 @@ def test_mode(temp_db, flag) -> None:
         .add_argument(temp_db)
         .add_argument(flag)
         .add_argument("list")
-        .statement('RETURN "This is a \\"test\\", with vertical bars |, \\"quotes\\", and\nnewlines.";')
+        .statement(
+            'RETURN "This is a \\"test\\", with vertical bars |, \\"quotes\\", and\nnewlines.";'
+        )
     )
     result = test.run()
-    result.check_stdout('"This is a ""test"", with vertical bars |, ""quotes"", and\nnewlines."')
+    result.check_stdout(
+        '"This is a ""test"", with vertical bars |, ""quotes"", and\nnewlines."'
+    )
 
     # test markdown mode
     test = (
@@ -450,10 +460,14 @@ def test_mode(temp_db, flag) -> None:
         .add_argument(temp_db)
         .add_argument(flag)
         .add_argument("tsv")
-        .statement('RETURN "This is a \\"test\\", with tabs \t, \\"quotes\\", and\nnewlines.";')
+        .statement(
+            'RETURN "This is a \\"test\\", with tabs \t, \\"quotes\\", and\nnewlines.";'
+        )
     )
     result = test.run()
-    result.check_stdout('"This is a ""test"", with tabs \t, ""quotes"", and\nnewlines."')
+    result.check_stdout(
+        '"This is a ""test"", with tabs \t, ""quotes"", and\nnewlines."'
+    )
 
     # test trash mode
     test = (
@@ -461,7 +475,7 @@ def test_mode(temp_db, flag) -> None:
         .add_argument(temp_db)
         .add_argument(flag)
         .add_argument("trash")
-        .statement('RETURN RANGE(0, 10) AS a;')
+        .statement("RETURN RANGE(0, 10) AS a;")
     )
     result = test.run()
     result.check_not_stdout("[0,1,2,3,4,5,6,7,8,9,10]")
@@ -474,7 +488,9 @@ def test_mode(temp_db, flag) -> None:
         .statement('RETURN "Databases Rule" AS a, "kuzu is cool" AS b;')
     )
     result = test.run()
-    result.check_stderr(f"Flag '{flag.replace('-', '')}' requires an argument but received none")
+    result.check_stderr(
+        f"Flag '{flag.replace('-', '')}' requires an argument but received none"
+    )
 
     # test invalid mode
     test = (
@@ -485,8 +501,8 @@ def test_mode(temp_db, flag) -> None:
         .statement('RETURN "Databases Rule" AS a, "kuzu is cool" AS b;')
     )
     result = test.run()
-    result.check_stderr("Cannot parse 'invalid' as output mode.")
-    
+    result.check_stderr("Error: cannot parse output mode: invalid")
+
 
 @pytest.mark.parametrize(
     "flag",
@@ -510,23 +526,16 @@ def test_no_stats(temp_db, flag) -> None:
     result.check_not_stdout("Time: ")
 
     # test stats on
-    test = (
-        ShellTest()
-        .add_argument(temp_db)
-        .statement('RETURN "Databases Rule" AS a;')
-    )
+    test = ShellTest().add_argument(temp_db).statement('RETURN "Databases Rule" AS a;')
     result = test.run()
     result.check_stdout("(1 tuple)")
     result.check_stdout("(1 column)")
     result.check_stdout("Time: ")
 
+
 @pytest.mark.parametrize(
     "flag",
-    [
-        "-b",
-        "--no_progress_bar",
-        "--noprogressbar"
-    ],
+    ["-b", "--no_progress_bar", "--noprogressbar"],
 )
 def test_no_progress_bar(temp_db, flag) -> None:
     # progress bar on by default
@@ -560,11 +569,7 @@ def test_no_progress_bar(temp_db, flag) -> None:
 )
 def test_init(temp_db, init_path, flag) -> None:
     # without init
-    test = (
-        ShellTest()
-        .add_argument(temp_db)
-        .statement("CALL show_tables() RETURN *;")
-    )
+    test = ShellTest().add_argument(temp_db).statement("CALL show_tables() RETURN *;")
     result = test.run()
     result.check_not_stdout("person")
     result.check_not_stdout("organisation")
